@@ -66,9 +66,7 @@ int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdL
 	DWORD dwErr;
 	BOOL bErr;
 	BOOL bDetect = FALSE;
-	int i;
 	HANDLE hStdin, hStdout;
-	DWORD dwLen;
 	lgLcdConnectContextEx conn;
 	lgLcdOpenByTypeContext ctx;
 	lgLcdBitmap160x43x1 bitmap;
@@ -85,11 +83,11 @@ int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdL
 
 	hStdin = GetStdHandle(STD_INPUT_HANDLE);
 	if (hStdin == INVALID_HANDLE_VALUE)
-		die(G15_ERR_PIPESTATE, "Failed to get standard input");
+		die(G15_ERR_INIT, "Failed to get standard input");
 
 	hStdout = GetStdHandle(STD_OUTPUT_HANDLE);
 	if (hStdout == INVALID_HANDLE_VALUE)
-		die(G15_ERR_PIPESTATE, "Failed to get standard output");
+		die(G15_ERR_INIT, "Failed to get standard output");
 
 	/*
 	 * Clear and set up initial structures.
@@ -163,7 +161,7 @@ int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdL
 
 		dwErr = lgLcdUpdateBitmap(ctx.device, (const lgLcdBitmapHeader *) &bitmap, bPriority ? LGLCD_SYNC_UPDATE(LGLCD_PRIORITY_ALERT) : LGLCD_SYNC_UPDATE(LGLCD_PRIORITY_NORMAL));
 		if (dwErr != ERROR_SUCCESS)
-			warn("Unable to update bitmap for device #%i successfully. (Error: %i)", i, dwErr);
+			warn("Unable to update bitmap for device. (Error: %i)", dwErr);
 	}
 
 	/*
@@ -187,10 +185,6 @@ int __stdcall WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdL
 	if (dwErr != ERROR_SUCCESS)
 		die(G15_ERR_DEINIT, "Unable to deinitialize LCD library. (Error: %i)", dwErr);
 
-	/*
-	 * Close pipe handle.
-	 */
-
 	warn("Terminated successfully.");
 
 	return 0;
