@@ -33,7 +33,7 @@
 #include <d3d10.h>
 #include <d3dx10.h>
 
-DXGIData *dxgi;
+DXGIData *dxgi = NULL;
 
 static bool bHooked = false;
 static bool bChaining = false;
@@ -61,57 +61,190 @@ struct SimpleVertex {
 	D3DXVECTOR2 Tex;
 };
 
-struct D10State {
-	LONG lHighMark;
+class D10State: protected Pipe {
+	public:
 
-	LONG initRefCount;
-	LONG refCount;
-	LONG myRefCount;
-	DWORD dwMyThread;
+		LONG lHighMark;
 
-	D3D10_VIEWPORT vp;
+		LONG initRefCount;
+		LONG refCount;
+		LONG myRefCount;
+		DWORD dwMyThread;
 
-	ID3D10Device *pDevice;
-	IDXGISwapChain *pSwapChain;
+		D3D10_VIEWPORT vp;
 
-	ID3D10StateBlock *pOrigStateBlock;
-	ID3D10StateBlock *pMyStateBlock;
-	ID3D10RenderTargetView *pRTV;
-	ID3D10Effect *pEffect;
-	ID3D10EffectTechnique *pTechnique;
-	ID3D10EffectShaderResourceVariable * pDiffuseTexture;
-	ID3D10EffectVectorVariable * pColor;
-	ID3D10InputLayout *pVertexLayout;
-	ID3D10Buffer *pVertexBuffer;
-	ID3D10Buffer *pIndexBuffer;
-	ID3D10BlendState *pBlendState;
-	ID3D10Texture2D *pTexture[NUM_TEXTS];
-	ID3D10ShaderResourceView *pSRView[NUM_TEXTS];
-	unsigned int uiCounter[NUM_TEXTS];
+		ID3D10Device *pDevice;
+		IDXGISwapChain *pSwapChain;
 
-	D10State(IDXGISwapChain *, ID3D10Device *);
-	~D10State();
-	void init();
-	void draw();
+		ID3D10StateBlock *pOrigStateBlock;
+		ID3D10StateBlock *pMyStateBlock;
+		ID3D10RenderTargetView *pRTV;
+		ID3D10Effect *pEffect;
+		ID3D10EffectTechnique *pTechnique;
+		ID3D10EffectShaderResourceVariable * pDiffuseTexture;
+		ID3D10InputLayout *pVertexLayout;
+		ID3D10Buffer *pVertexBuffer;
+		ID3D10Buffer *pIndexBuffer;
+		ID3D10BlendState *pBlendState;
+
+		ID3D10Texture2D *pTexture;
+		ID3D10ShaderResourceView *pSRView;
+
+		D10State(IDXGISwapChain *, ID3D10Device *);
+		~D10State();
+		void init();
+		void draw();
+
+		void blit(unsigned int x, unsigned int y, unsigned int w, unsigned int h);
+		void setRect();
+		void newTexture(unsigned int w, unsigned int h);
 };
 
 map<IDXGISwapChain *, D10State *> chains;
 map<ID3D10Device *, D10State *> devices;
 
 D10State::D10State(IDXGISwapChain *pSwapChain, ID3D10Device *pDevice) {
-	memset(this, 0, sizeof(*this));
-
 	this->pSwapChain = pSwapChain;
 	this->pDevice = pDevice;
 
+	lHighMark = initRefCount  = refCount = myRefCount = 0;
 	dwMyThread = 0;
-	refCount = 0;
-	myRefCount = 0;
+
+	ZeroMemory(&vp, sizeof(vp));
+
+	pOrigStateBlock = NULL;
+	pMyStateBlock = NULL;
+	pRTV = NULL;
+	pEffect = NULL;
+	pTechnique = NULL;
+	pDiffuseTexture = NULL;
+	pVertexLayout = NULL;
+	pVertexBuffer = NULL;
+	pIndexBuffer = NULL;
+	pBlendState = NULL;
+	pTexture = NULL;
+	pSRView = NULL;
 
 	pDevice->AddRef();
 	initRefCount = pDevice->Release();
 }
 
+void D10State::blit(unsigned int x, unsigned int y, unsigned int w, unsigned int h) {
+	HRESULT hr;
+
+	ods("D3D10: Blit %d %d %d %d", x, y, w, h);
+
+	if (! pTexture || ! pSRView)
+		return;
+
+	D3D10_MAPPED_TEXTURE2D mappedTex;
+	hr = pTexture->Map(D3D10CalcSubresource(0, 0, 1), D3D10_MAP_WRITE_DISCARD, 0, &mappedTex);
+	if (FAILED(hr)) {
+		ods("D3D10: Failed map");
+	}
+
+	UCHAR* pTexels = (UCHAR*)mappedTex.pData;
+
+	for (int r=0;r< uiHeight; ++r) {
+		unsigned char *sptr = a_ucTexture + r * uiWidth * 4;
+		unsigned char *dptr = reinterpret_cast<unsigned char *>(pTexels) + r * mappedTex.RowPitch;
+		memcpy(dptr, sptr, uiWidth * 4);
+	}
+
+	pTexture->Unmap(D3D10CalcSubresource(0, 0, 1));
+}
+
+void D10State::setRect() {
+	HRESULT hr;
+
+	ods("D3D10: Setrect");
+
+	float w = static_cast<float>(uiWidth);
+	float h = static_cast<float>(uiHeight);
+
+	float left   = static_cast<float>(uiLeft) - 0.5f;
+	float top    = static_cast<float>(uiTop) - 0.5f;
+	float right  = static_cast<float>(uiRight) + 0.5f;
+	float bottom = static_cast<float>(uiBottom) + 0.5f;
+
+	float texl = (left) / w;
+	float text = (top) / h;
+	float texr = (right + 1.0f) / w;
+	float texb = (bottom + 1.0f) / h;
+
+	left = 2.0f * (left / vp.Width) - 1.0f;
+	right = 2.0f * (right / vp.Width) - 1.0f;
+	top = -2.0f * (top / vp.Height) + 1.0f;
+	bottom = -2.0f * (bottom / vp.Height) + 1.0f;
+
+	ods("Vertex (%f %f) (%f %f)", left, top, right, bottom);
+
+	// Create vertex buffer
+	SimpleVertex vertices[] = {
+		{ D3DXVECTOR3(left, top, 0.5f), D3DXVECTOR2(texl, text) },
+		{ D3DXVECTOR3(right, top, 0.5f), D3DXVECTOR2(texr, text) },
+		{ D3DXVECTOR3(right, bottom, 0.5f), D3DXVECTOR2(texr, texb) },
+		{ D3DXVECTOR3(left, bottom, 0.5f), D3DXVECTOR2(texl, texb) },
+	};
+
+	void *pData = NULL;
+
+	hr = pVertexBuffer->Map(D3D10_MAP_WRITE_DISCARD, 0, &pData);
+	memcpy(pData, vertices, sizeof(vertices));
+	ods("Map: %lx %d", hr, sizeof(vertices));
+	pVertexBuffer->Unmap();
+}
+
+void D10State::newTexture(unsigned int w, unsigned int h) {
+	HRESULT hr;
+
+	ods("D3D10: newTex %d %d", w, h);
+
+	if (pTexture) {
+		pTexture->Release();
+		pTexture = NULL;
+	}
+	if (pSRView) {
+		pSRView->Release();
+		pSRView = NULL;
+	}
+
+	D3D10_TEXTURE2D_DESC desc;
+	ZeroMemory(&desc, sizeof(desc));
+
+	desc.Width = w;
+	desc.Height = h;
+	desc.MipLevels = desc.ArraySize = 1;
+	desc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
+	desc.SampleDesc.Count = 1;
+	desc.Usage = D3D10_USAGE_DYNAMIC;
+	desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
+	desc.CPUAccessFlags = D3D10_CPU_ACCESS_WRITE;
+	hr = pDevice->CreateTexture2D(&desc, NULL, &pTexture);
+
+	if (! SUCCEEDED(hr)) {
+		pTexture = NULL;
+		ods("D3D10: Failed to create texture.");
+		return;
+	}
+
+	D3D10_SHADER_RESOURCE_VIEW_DESC srvDesc;
+	ZeroMemory(&srvDesc, sizeof(srvDesc));
+	srvDesc.Format = desc.Format;
+	srvDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
+	srvDesc.Texture2D.MostDetailedMip = 0;
+	srvDesc.Texture2D.MipLevels = desc.MipLevels;
+
+	hr = pDevice->CreateShaderResourceView(pTexture, &srvDesc, &pSRView);
+	if (! SUCCEEDED(hr)) {
+		pSRView = NULL;
+		pTexture->Release();
+		pTexture = NULL;
+		ods("D3D10: Failed to create resource view.");
+		return;
+	}
+}
+
 void D10State::init() {
 	static HMODREF(GetModuleHandleW(L"D3D10.DLL"), D3D10CreateEffectFromMemory);
 	static HMODREF(GetModuleHandleW(L"D3D10.DLL"), D3D10CreateStateBlock);
@@ -169,13 +302,9 @@ void D10State::init() {
 
 	pTechnique = pEffect->GetTechniqueByName("Render");
 	pDiffuseTexture = pEffect->GetVariableByName("txDiffuse")->AsShaderResource();
-	pColor = pEffect->GetVariableByName("fColor")->AsVector();
 
-	for (int i=0;i<NUM_TEXTS;++i) {
-		pTexture[i] = NULL;
-		pSRView[i] = NULL;
-		uiCounter[i] = 0;
-	}
+	pTexture = NULL;
+	pSRView = NULL;
 
 	// Define the input layout
 	D3D10_INPUT_ELEMENT_DESC layout[] = {
@@ -235,12 +364,10 @@ D10State::~D10State() {
 	pVertexLayout->Release();
 	pEffect->Release();
 	pRTV->Release();
-	for (int i=0;i<NUM_TEXTS;++i) {
-		if (pTexture[i])
-			pTexture[i]->Release();
-		if (pSRView[i])
-			pSRView[i]->Release();
-	}
+	if (pTexture)
+		pTexture->Release();
+	if (pSRView)
+		pSRView->Release();
 
 	pMyStateBlock->ReleaseAllDeviceObjects();
 	pMyStateBlock->Release();
@@ -250,198 +377,34 @@ D10State::~D10State() {
 }
 
 void D10State::draw() {
+	HRESULT hr;
 	dwMyThread = GetCurrentThreadId();
 
-	pOrigStateBlock->Capture();
-	pMyStateBlock->Apply();
+	checkMessage(vp.Width, vp.Height);
 
-	int idx = 0;
-	HRESULT hr;
+	if (a_ucTexture && pSRView && (uiLeft != uiRight)) {
+		pOrigStateBlock->Capture();
+		pMyStateBlock->Apply();
 
-	vector<ID3D10ShaderResourceView *> texs;
-	vector<unsigned int> widths;
-	vector<unsigned int> yofs;
-	vector<DWORD> colors;
-
-	unsigned int y = 0;
-
-	if (sm->fFontSize < 0.01f)
-		sm->fFontSize = 0.01f;
-	else if (sm->fFontSize > 1.0f)
-		sm->fFontSize = 1.0f;
-
-	int iHeight = lround(vp.Height * sm->fFontSize);
-
-	if (iHeight > TEXT_HEIGHT)
-		iHeight = TEXT_HEIGHT;
-
-	float s = iHeight / 60.0f;
-
-	ods("D3D10: Init: Scale %f. iH %d. Final scale %f", sm->fFontSize, iHeight, s);
-
-	DWORD dwWaitResult = WaitForSingleObject(hSharedMutex, 50L);
-	if (dwWaitResult == WAIT_OBJECT_0) {
-		for (int i=0;i<NUM_TEXTS;i++) {
-			if (sm->texts[i].width == 0) {
-				y += iHeight / 4;
-			} else if (sm->texts[i].width > 0) {
-				if (!pSRView[i] || (sm->texts[i].uiCounter != uiCounter[i])) {
-					if (pTexture[i])
-						pTexture[i]->Release();
-					if (pSRView[i])
-						pSRView[i]->Release();
-
-					pTexture[i] = NULL;
-					pSRView[i] = NULL;
-
-					D3D10_TEXTURE2D_DESC desc;
-					ZeroMemory(&desc, sizeof(desc));
-
-					desc.Width = sm->texts[i].width;
-					desc.Height = TEXT_HEIGHT;
-					desc.MipLevels = desc.ArraySize = 1;
-					desc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
-					desc.SampleDesc.Count = 1;
-					desc.Usage = D3D10_USAGE_DYNAMIC;
-					desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
-					desc.CPUAccessFlags = D3D10_CPU_ACCESS_WRITE;
-					hr = pDevice->CreateTexture2D(&desc, NULL, &pTexture[i]);
-					ods("Setting %d by %d texture %d", desc.Width, desc.Height, i);
-					ods("%lx %p", hr, pTexture[i]);
-
-					D3D10_MAPPED_TEXTURE2D mappedTex;
-					hr = pTexture[i]->Map(D3D10CalcSubresource(0, 0, 1), D3D10_MAP_WRITE_DISCARD, 0, &mappedTex);
-
-					UCHAR* pTexels = (UCHAR*)mappedTex.pData;
-
-					for (int r=0;r<TEXT_HEIGHT;r++) {
-						unsigned char *dptr = reinterpret_cast<unsigned char *>(pTexels) + r * mappedTex.RowPitch;
-						memcpy(dptr, sm->texts[i].texture + r * TEXT_WIDTH * 4, sm->texts[i].width * 4);
-					}
-
-					pTexture[i]->Unmap(D3D10CalcSubresource(0, 0, 1));
-
-					D3D10_SHADER_RESOURCE_VIEW_DESC srvDesc;
-					ZeroMemory(&srvDesc, sizeof(srvDesc));
-					srvDesc.Format = desc.Format;
-					srvDesc.ViewDimension = D3D10_SRV_DIMENSION_TEXTURE2D;
-					srvDesc.Texture2D.MostDetailedMip = 0;
-					srvDesc.Texture2D.MipLevels = desc.MipLevels;
-					pDevice->CreateShaderResourceView(pTexture[i], &srvDesc, &pSRView[i]);
-
-					uiCounter[i] = sm->texts[i].uiCounter;
-				}
-				unsigned int w = lround(sm->texts[i].width * s);
-				texs.push_back(pSRView[i]);
-				colors.push_back(sm->texts[i].color);
-				widths.push_back(w);
-				yofs.push_back(y);
-				idx++;
-				y += iHeight;
-			}
-		}
-		ReleaseMutex(hSharedMutex);
-	}
-
-	if (idx == 0)
-		return;
-
-	int height = y;
-	y = lround(vp.Height * sm->fY);
-
-	if (sm->bTop) {
-		y -= height;
-	} else if (sm->bBottom) {
-	} else {
-		y -= height / 2;
-	}
-
-
-	if (y < 1)
-		y = 1;
-	if ((y + height + 1) > vp.Height)
-		y = vp.Height - height - 1;
-
-	D3D10_TECHNIQUE_DESC techDesc;
-	pTechnique->GetDesc(&techDesc);
-
-	for (int i=0;i<idx;i++) {
-		unsigned int width = widths[i];
-
-		int x = lround(vp.Width * sm->fX);
-
-		if (sm->bLeft) {
-			x -= width;
-		} else if (sm->bRight) {
-		} else {
-			x -= width / 2;
-		}
-
-		if (x < 1)
-			x = 1;
-		if ((x + width + 1) > vp.Width)
-			x = vp.Width - width - 1;
-
-//		D3DCOLOR color = colors[i];
-
-		float cols[] = {
-			((colors[i] >> 24) & 0xFF) / 255.0f,
-			((colors[i] >> 16) & 0xFF) / 255.0f,
-			((colors[i] >> 8) & 0xFF) / 255.0f,
-			((colors[i] >> 0) & 0xFF) / 255.0f,
-		};
-
-		float left   = static_cast<float>(x);
-		float top    = static_cast<float>(y + yofs[i]);
-		float right  = left + width;
-		float bottom = top + iHeight;
-
-		left = 2.0f * (left / vp.Width) - 1.0f;
-		right = 2.0f * (right / vp.Width) - 1.0f;
-		top = -2.0f * (top / vp.Height) + 1.0f;
-		bottom = -2.0f * (bottom / vp.Height) + 1.0f;
-
-		ods("Vertex (%f %f) (%f %f)", left, top, right, bottom);
-
-		// Create vertex buffer
-		SimpleVertex vertices[] = {
-			{ D3DXVECTOR3(left, top, 0.5f), D3DXVECTOR2(0.0f, 0.0f) },
-			{ D3DXVECTOR3(right, top, 0.5f), D3DXVECTOR2(1.0f, 0.0f) },
-			{ D3DXVECTOR3(right, bottom, 0.5f), D3DXVECTOR2(1.0f, 1.0f) },
-			{ D3DXVECTOR3(left, bottom, 0.5f), D3DXVECTOR2(0.0f, 1.0f) },
-		};
-
-		void *pData = NULL;
-
-		hr = pVertexBuffer->Map(D3D10_MAP_WRITE_DISCARD, 0, &pData);
-		memcpy(pData, vertices, sizeof(vertices));
-		ods("Map: %lx %d", hr, sizeof(vertices));
-		pVertexBuffer->Unmap();
+		D3D10_TECHNIQUE_DESC techDesc;
+		pTechnique->GetDesc(&techDesc);
 
 		// Set vertex buffer
 		UINT stride = sizeof(SimpleVertex);
 		UINT offset = 0;
 		pDevice->IASetVertexBuffers(0, 1, &pVertexBuffer, &stride, &offset);
 
-		hr = pDiffuseTexture->SetResource(texs[i]);
-		ods("setres %p %lx", pDiffuseTexture, hr);
-
-		hr = pColor->SetFloatVector(cols);
-		ods("setres %p %lx", pColor, hr);
-
-		ods("%f %f %f %f", cols[0], cols[1], cols[2], cols[3]);
+		hr = pDiffuseTexture->SetResource(pSRView);
+		if (! SUCCEEDED(hr))
+			ods("D3D10: Failed to set resource");
 
 		for (UINT p = 0; p < techDesc.Passes; ++p) {
-			//		ods("Pass %d", p);
 			pTechnique->GetPassByIndex(p)->Apply(0);
 			pDevice->DrawIndexed(6, 0, 0);
 		}
+		pOrigStateBlock->Apply();
 	}
 
-
-	// Render a triangle
-	pOrigStateBlock->Apply();
-
 	dwMyThread = 0;
 }
 
@@ -559,6 +522,9 @@ void checkDXGIHook(bool preonly) {
 		ods("DXGI: Causing a chain");
 	}
 
+	if (! dxgi->iOffsetPresent || ! dxgi->iOffsetResize)
+		return;
+
 	bChaining = true;
 
 	HMODULE hDXGI = GetModuleHandleW(L"DXGI.DLL");
@@ -598,11 +564,10 @@ void checkDXGIHook(bool preonly) {
 }
 
 extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
-	ods("Preparing static data for DXGI Injection");
+	if (! dxgi)
+		return;
 
-	HMODULE hD3D10 = LoadLibrary("D3D10.DLL");
-	HMODULE hDXGI = LoadLibrary("DXGI.DLL");
-	HRESULT hr;
+	ods("Preparing static data for DXGI Injection");
 
 	dxgi->wcDXGIFileName[0] = 0;
 	dxgi->wcD3D10FileName[0] = 0;
@@ -611,120 +576,132 @@ extern "C" __declspec(dllexport) void __cdecl PrepareDXGI() {
 	dxgi->iOffsetAddRef = 0;
 	dxgi->iOffsetRelease = 0;
 
-	if (hDXGI != NULL && hD3D10 != NULL) {
-		CreateDXGIFactoryType pCreateDXGIFactory = reinterpret_cast<CreateDXGIFactoryType>(GetProcAddress(hDXGI, "CreateDXGIFactory"));
-		ods("Got %p", pCreateDXGIFactory);
-		if (pCreateDXGIFactory) {
-			IDXGIFactory * pFactory;
-			hr = pCreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
-			if (pFactory) {
-				HWND hwnd = CreateWindowW(L"STATIC", L"Mumble DXGI Window", WS_OVERLAPPEDWINDOW,
-				                          CW_USEDEFAULT, CW_USEDEFAULT, 640, 480, 0,
-				                          NULL, NULL, 0);
+	OSVERSIONINFOEXW ovi;
+	memset(&ovi, 0, sizeof(ovi));
+	ovi.dwOSVersionInfoSize = sizeof(ovi);
+	GetVersionExW(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
+	if ((ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) && (ovi.dwBuildNumber >= 6001))) {
+		HMODULE hD3D10 = LoadLibrary("D3D10.DLL");
+		HMODULE hDXGI = LoadLibrary("DXGI.DLL");
+		HRESULT hr;
 
-				IDXGIAdapter *pAdapter = NULL;
-				pFactory->EnumAdapters(0, &pAdapter);
+		if (hDXGI != NULL && hD3D10 != NULL) {
+			CreateDXGIFactoryType pCreateDXGIFactory = reinterpret_cast<CreateDXGIFactoryType>(GetProcAddress(hDXGI, "CreateDXGIFactory"));
+			ods("Got %p", pCreateDXGIFactory);
+			if (pCreateDXGIFactory) {
+				IDXGIFactory * pFactory;
+				hr = pCreateDXGIFactory(__uuidof(IDXGIFactory), (void**)(&pFactory));
+				if (pFactory) {
+					HWND hwnd = CreateWindowW(L"STATIC", L"Mumble DXGI Window", WS_OVERLAPPEDWINDOW,
+					                          CW_USEDEFAULT, CW_USEDEFAULT, 640, 480, 0,
+					                          NULL, NULL, 0);
 
-				D3D10CreateDeviceAndSwapChainType pD3D10CreateDeviceAndSwapChain = reinterpret_cast<D3D10CreateDeviceAndSwapChainType>(GetProcAddress(hD3D10, "D3D10CreateDeviceAndSwapChain"));
+					IDXGIAdapter *pAdapter = NULL;
+					pFactory->EnumAdapters(0, &pAdapter);
 
-				IDXGISwapChain *pSwapChain = NULL;
-				ID3D10Device *pDevice = NULL;
+					D3D10CreateDeviceAndSwapChainType pD3D10CreateDeviceAndSwapChain = reinterpret_cast<D3D10CreateDeviceAndSwapChainType>(GetProcAddress(hD3D10, "D3D10CreateDeviceAndSwapChain"));
 
-				DXGI_SWAP_CHAIN_DESC desc;
-				ZeroMemory(&desc, sizeof(desc));
+					IDXGISwapChain *pSwapChain = NULL;
+					ID3D10Device *pDevice = NULL;
 
-				RECT rcWnd;
-				GetClientRect(hwnd, &rcWnd);
-				desc.BufferDesc.Width = rcWnd.right - rcWnd.left;
-				desc.BufferDesc.Height = rcWnd.bottom - rcWnd.top;
+					DXGI_SWAP_CHAIN_DESC desc;
+					ZeroMemory(&desc, sizeof(desc));
 
-				ods("W %d H %d", desc.BufferDesc.Width, desc.BufferDesc.Height);
+					RECT rcWnd;
+					GetClientRect(hwnd, &rcWnd);
+					desc.BufferDesc.Width = rcWnd.right - rcWnd.left;
+					desc.BufferDesc.Height = rcWnd.bottom - rcWnd.top;
 
-				desc.BufferDesc.RefreshRate.Numerator = 60;
-				desc.BufferDesc.RefreshRate.Denominator = 1;
-				desc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB;
-				desc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
-				desc.BufferDesc.Scaling = DXGI_MODE_SCALING_CENTERED;
+					ods("W %d H %d", desc.BufferDesc.Width, desc.BufferDesc.Height);
 
-				desc.SampleDesc.Count = 1;
-				desc.SampleDesc.Quality = 0;
+					desc.BufferDesc.RefreshRate.Numerator = 60;
+					desc.BufferDesc.RefreshRate.Denominator = 1;
+					desc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM_SRGB;
+					desc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
+					desc.BufferDesc.Scaling = DXGI_MODE_SCALING_CENTERED;
 
-				desc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
+					desc.SampleDesc.Count = 1;
+					desc.SampleDesc.Quality = 0;
 
-				desc.BufferCount = 2;
+					desc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
 
-				desc.OutputWindow = hwnd;
+					desc.BufferCount = 2;
 
-				desc.Windowed = true;
+					desc.OutputWindow = hwnd;
 
-				desc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
+					desc.Windowed = true;
 
-				hr = pD3D10CreateDeviceAndSwapChain(pAdapter, D3D10_DRIVER_TYPE_HARDWARE, NULL, 0, D3D10_SDK_VERSION, &desc, &pSwapChain, &pDevice);
+					desc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
 
-				if (pDevice && pSwapChain) {
-					HMODULE hRef;
-					void ***vtbl = (void ***) pSwapChain;
-					void *pPresent = (*vtbl)[8];
-					if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pPresent, &hRef)) {
-						ods("DXGI: Failed to get module for Present");
-					} else {
-						GetModuleFileNameW(hRef, dxgi->wcDXGIFileName, 2048);
-						unsigned char *b = (unsigned char *) pPresent;
-						unsigned char *a = (unsigned char *) hRef;
-						dxgi->iOffsetPresent = b-a;
-						ods("DXGI: Successfully found Present offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetPresent);
-					}
+					hr = pD3D10CreateDeviceAndSwapChain(pAdapter, D3D10_DRIVER_TYPE_HARDWARE, NULL, 0, D3D10_SDK_VERSION, &desc, &pSwapChain, &pDevice);
 
-					void *pResize = (*vtbl)[13];
-					if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pResize, &hRef)) {
-						ods("DXGI: Failed to get module for ResizeBuffers");
-					} else {
-						wchar_t buff[2048];
-						GetModuleFileNameW(hRef, buff, 2048);
-						if (wcscmp(buff, dxgi->wcDXGIFileName) == 0) {
-							unsigned char *b = (unsigned char *) pResize;
+					if (pDevice && pSwapChain) {
+						HMODULE hRef;
+						void ***vtbl = (void ***) pSwapChain;
+						void *pPresent = (*vtbl)[8];
+						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pPresent, &hRef)) {
+							ods("DXGI: Failed to get module for Present");
+						} else {
+							GetModuleFileNameW(hRef, dxgi->wcDXGIFileName, 2048);
+							unsigned char *b = (unsigned char *) pPresent;
 							unsigned char *a = (unsigned char *) hRef;
-							dxgi->iOffsetResize = b-a;
-							ods("DXGI: Successfully found ResizeBuffers offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetPresent);
+							dxgi->iOffsetPresent = b-a;
+							ods("DXGI: Successfully found Present offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetPresent);
+						}
+
+						void *pResize = (*vtbl)[13];
+						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pResize, &hRef)) {
+							ods("DXGI: Failed to get module for ResizeBuffers");
+						} else {
+							wchar_t buff[2048];
+							GetModuleFileNameW(hRef, buff, 2048);
+							if (wcscmp(buff, dxgi->wcDXGIFileName) == 0) {
+								unsigned char *b = (unsigned char *) pResize;
+								unsigned char *a = (unsigned char *) hRef;
+								dxgi->iOffsetResize = b-a;
+								ods("DXGI: Successfully found ResizeBuffers offset: %ls: %d", dxgi->wcDXGIFileName, dxgi->iOffsetPresent);
+							}
+						}
+
+						vtbl = (void ***) pDevice;
+
+						void *pAddRef = (*vtbl)[1];
+						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pAddRef, &hRef)) {
+							ods("D3D10: Failed to get module for AddRef");
+						} else {
+							GetModuleFileNameW(hRef, dxgi->wcD3D10FileName, 2048);
+							unsigned char *b = (unsigned char *) pAddRef;
+							unsigned char *a = (unsigned char *) hRef;
+							dxgi->iOffsetAddRef = b-a;
+							ods("D3D10: Successfully found AddRef offset: %ls: %d", dxgi->wcD3D10FileName, dxgi->iOffsetAddRef);
+						}
+
+						void *pRelease = (*vtbl)[2];
+						if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pRelease, &hRef)) {
+							ods("D3D10: Failed to get module for Release");
+						} else {
+							wchar_t buff[2048];
+							GetModuleFileNameW(hRef, buff, 2048);
+							if (wcscmp(buff, dxgi->wcD3D10FileName) == 0) {
+								unsigned char *b = (unsigned char *) pRelease;
+								unsigned char *a = (unsigned char *) hRef;
+								dxgi->iOffsetRelease = b-a;
+								ods("D3D10: Successfully found Release offset: %ls: %d", dxgi->wcD3D10FileName, dxgi->iOffsetRelease);
+							}
 						}
 					}
 
-					vtbl = (void ***) pDevice;
+					if (pDevice)
+						pDevice->Release();
+					if (pSwapChain)
+						pSwapChain->Release();
+					DestroyWindow(hwnd);
 
-					void *pAddRef = (*vtbl)[1];
-					if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pAddRef, &hRef)) {
-						ods("D3D10: Failed to get module for AddRef");
-					} else {
-						GetModuleFileNameW(hRef, dxgi->wcD3D10FileName, 2048);
-						unsigned char *b = (unsigned char *) pAddRef;
-						unsigned char *a = (unsigned char *) hRef;
-						dxgi->iOffsetAddRef = b-a;
-						ods("D3D10: Successfully found AddRef offset: %ls: %d", dxgi->wcD3D10FileName, dxgi->iOffsetAddRef);
-					}
-
-					void *pRelease = (*vtbl)[2];
-					if (! GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) pRelease, &hRef)) {
-						ods("D3D10: Failed to get module for Release");
-					} else {
-						wchar_t buff[2048];
-						GetModuleFileNameW(hRef, buff, 2048);
-						if (wcscmp(buff, dxgi->wcD3D10FileName) == 0) {
-							unsigned char *b = (unsigned char *) pRelease;
-							unsigned char *a = (unsigned char *) hRef;
-							dxgi->iOffsetRelease = b-a;
-							ods("D3D10: Successfully found Release offset: %ls: %d", dxgi->wcD3D10FileName, dxgi->iOffsetRelease);
-						}
-					}
+					pFactory->Release();
 				}
-
-				if (pDevice)
-					pDevice->Release();
-				if (pSwapChain)
-					pSwapChain->Release();
-				DestroyWindow(hwnd);
-
-				pFactory->Release();
 			}
 		}
+	} else {
+		ods("No DXGI pre-Vista");
 	}
 }
