@@ -34,8 +34,6 @@ static HANDLE hMapObject = NULL;
 static HANDLE hHookMutex = NULL;
 static HHOOK hhookWnd = 0;
 
-SharedMem *sm = NULL;
-HANDLE hSharedMutex = NULL;
 HMODULE hSelf = NULL;
 static BOOL bMumble = FALSE;
 static BOOL bDebug = FALSE;
@@ -45,6 +43,8 @@ static HardHook hhLoadW;
 void *HardHook::pCode = NULL;
 unsigned int HardHook::uiCode = 0;
 
+static SharedData *sd;
+
 HardHook::HardHook() {
 	int i;
 	baseptr = NULL;
@@ -372,7 +372,7 @@ void __cdecl fods(const char *format, ...) {
 
 void __cdecl ods(const char *format, ...) {
 #ifndef DEBUG
-	if (!bDebug && (!sm || ! sm->bDebug))
+	if (!bDebug)
 		return;
 #endif
 	char    buf[4096], *p = buf;
@@ -397,6 +397,205 @@ void __cdecl ods(const char *format, ...) {
 	OutputDebugStringA(buf);
 }
 
+Pipe::Pipe() {
+	hSocket = INVALID_HANDLE_VALUE;
+	hMemory = NULL;
+	a_ucTexture = NULL;
+
+	omMsg.omh.iLength = -1;
+
+	uiWidth = uiHeight = 0;
+	uiLeft = uiRight = uiTop = uiBottom = 0;
+}
+
+Pipe::~Pipe() {
+	disconnect();
+}
+
+void Pipe::release() {
+	if (hMemory) {
+		CloseHandle(hMemory);
+		hMemory = NULL;
+		if (a_ucTexture) {
+			UnmapViewOfFile(a_ucTexture);
+			a_ucTexture = NULL;
+		}
+
+		uiLeft = uiRight = uiTop = uiBottom = 0;
+	}
+}
+
+void Pipe::disconnect() {
+	release();
+	if (hSocket != INVALID_HANDLE_VALUE) {
+		ods("Pipe: Disconnect");
+		CloseHandle(hSocket);
+		hSocket = INVALID_HANDLE_VALUE;
+	}
+	uiWidth = 0;
+	uiHeight = 0;
+	omMsg.omh.iLength = -1;
+}
+
+bool Pipe::sendMessage(const OverlayMsg &om) {
+	DWORD dwBytesToWrite = sizeof(OverlayMsgHeader) + om.omh.iLength;
+	DWORD dwBytesWritten = dwBytesToWrite;
+
+	if (WriteFile(hSocket, om.headerbuffer, sizeof(OverlayMsgHeader) + om.omh.iLength, &dwBytesWritten, NULL))
+		if (dwBytesToWrite == dwBytesWritten)
+			return true;
+
+	ods("Pipe: Short write");
+	disconnect();
+	return false;
+}
+
+void Pipe::checkMessage(unsigned int w, unsigned int h) {
+	if (!w || ! h)
+		return;
+
+	if (hSocket == INVALID_HANDLE_VALUE) {
+		hSocket = CreateFileW(L"\\\\.\\pipe\\MumbleOverlayPipe", GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);
+		if (hSocket == INVALID_HANDLE_VALUE) {
+			ods("Pipe: Connection failed");
+			return;
+		}
+		ods("Pipe: Connected");
+
+		uiWidth = 0;
+		uiHeight = 0;
+	}
+
+	if ((uiWidth != w) || (uiHeight != h)) {
+		release();
+
+		uiWidth = w;
+		uiHeight = h;
+
+		OverlayMsg om;
+		om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+		om.omh.uiType = OVERLAY_MSGTYPE_INIT;
+		om.omh.iLength = sizeof(OverlayMsgInit);
+		om.omi.uiWidth = uiWidth;
+		om.omi.uiHeight = uiHeight;
+
+		if (!sendMessage(om))
+			return;
+
+		ods("Pipe: SentInit %d %d", w, h);
+	}
+
+	while (1) {
+		DWORD dwBytesLeft;
+		DWORD dwBytesRead;
+
+		if (! PeekNamedPipe(hSocket, NULL, 0, NULL, &dwBytesLeft, NULL)) {
+			ods("Pipe: No peek");
+			disconnect();
+			return;
+		}
+
+		if (omMsg.omh.iLength == -1) {
+			if (dwBytesLeft < sizeof(OverlayMsgHeader))
+				break;
+
+			ReadFile(hSocket, omMsg.headerbuffer, sizeof(OverlayMsgHeader), &dwBytesRead, NULL);
+			dwBytesLeft -= sizeof(OverlayMsgHeader);
+
+			if (dwBytesRead != sizeof(OverlayMsgHeader)) {
+				ods("Pipe: Short header read %d", dwBytesRead);
+				disconnect();
+				return;
+			}
+		}
+
+		if (static_cast<int>(dwBytesLeft) < omMsg.omh.iLength)
+			break;
+
+		ReadFile(hSocket, omMsg.msgbuffer, omMsg.omh.iLength, &dwBytesRead, NULL);
+		if (dwBytesRead != omMsg.omh.iLength) {
+			ods("Pipe: Short body read %d/%d", dwBytesRead, omMsg.omh.iLength);
+			disconnect();
+			return;
+		}
+
+		switch (omMsg.omh.uiType) {
+			case OVERLAY_MSGTYPE_SHMEM: {
+					wchar_t memname[2048];
+					memname[0] = 0;
+
+					MultiByteToWideChar(CP_UTF8, 0, omMsg.oms.a_cName, omMsg.omh.iLength, memname, 2048);
+
+					release();
+
+					hMemory = CreateFileMappingW(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, uiWidth * uiHeight * 4, memname);
+
+					if (GetLastError() != ERROR_ALREADY_EXISTS) {
+						ods("Pipe: Memory %s(%d) => %ls doesn't exist", omMsg.oms.a_cName, omMsg.omh.iLength, memname);
+						if (hMemory) {
+							CloseHandle(hMemory);
+							hMemory = NULL;
+							break;
+						}
+					}
+
+					if (! hMemory) {
+						ods("Pipe: CreateFileMapping failed");
+						break;
+					}
+
+					a_ucTexture = reinterpret_cast<unsigned char *>(MapViewOfFile(hMemory, FILE_MAP_ALL_ACCESS, 0, 0, 0));
+
+					if (a_ucTexture == NULL) {
+						ods("Pipe: Failed to map memory");
+						CloseHandle(hMemory);
+						hMemory = NULL;
+					}
+
+					MEMORY_BASIC_INFORMATION mbi;
+					memset(&mbi, 0, sizeof(mbi));
+					if ((VirtualQuery(a_ucTexture, &mbi, sizeof(mbi)) == 0) || (mbi.RegionSize < (uiHeight * uiWidth * 4))) {
+						ods("Pipe: Memory too small");
+						UnmapViewOfFile(a_ucTexture);
+						CloseHandle(hMemory);
+						a_ucTexture = NULL;
+						hMemory = NULL;
+						break;
+					}
+
+					OverlayMsg om;
+					om.omh.uiMagic = OVERLAY_MAGIC_NUMBER;
+					om.omh.uiType = OVERLAY_MSGTYPE_SHMEM;
+					om.omh.iLength = 0;
+
+					if (!sendMessage(om))
+						return;
+
+					newTexture(uiWidth, uiHeight);
+				}
+				break;
+			case OVERLAY_MSGTYPE_BLIT: {
+					if (a_ucTexture)
+						blit(omMsg.omb.x, omMsg.omb.y, omMsg.omb.w, omMsg.omb.h);
+				}
+				break;
+			case OVERLAY_MSGTYPE_ACTIVE: {
+					uiLeft = omMsg.oma.x;
+					uiTop = omMsg.oma.y;
+					uiRight = omMsg.oma.x + omMsg.oma.w;
+					uiBottom = omMsg.oma.y + omMsg.oma.h;
+					if (a_ucTexture)
+						setRect();
+				}
+				break;
+			default:
+				break;
+		}
+		omMsg.omh.iLength = -1;
+	}
+}
+
+static bool bBlackListed = false;
 static const char *blacklist[] = {
 	"iexplore.exe",
 	"ieuser.exe",
@@ -413,7 +612,7 @@ static HMODULE WINAPI MyLoadLibrary(const char *lpFileName) {
 	hhLoad.restore();
 
 	HMODULE h = oLoadLibrary(lpFileName);
-	ods("Library %s loaded to %p", lpFileName, h);
+//	ods("Library %s loaded to %p", lpFileName, h);
 
 	if (! bMumble) {
 		checkD3D9Hook();
@@ -450,23 +649,21 @@ static LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam) {
 extern "C" __declspec(dllexport) void __cdecl RemoveHooks() {
 	DWORD dwWaitResult = WaitForSingleObject(hHookMutex, 1000L);
 	if (dwWaitResult == WAIT_OBJECT_0) {
-		if (sm->bHooked) {
+		if (sd->bHooked) {
 			if (hhookWnd) {
 				UnhookWindowsHookEx(hhookWnd);
 				hhookWnd = NULL;
 			}
-			sm->bHooked = false;
+			sd->bHooked = false;
 		}
 		ReleaseMutex(hHookMutex);
 	}
 }
 
 extern "C" __declspec(dllexport) void __cdecl InstallHooks() {
-	sm->lastAppAlive = GetTickCount();
-
 	DWORD dwWaitResult = WaitForSingleObject(hHookMutex, 1000L);
 	if (dwWaitResult == WAIT_OBJECT_0) {
-		if (! sm->bHooked) {
+		if (! sd->bHooked) {
 			GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (char *) &InstallHooks, &hSelf);
 			if (hSelf == NULL) {
 				ods("Lib: Failed to find myself");
@@ -476,7 +673,7 @@ extern "C" __declspec(dllexport) void __cdecl InstallHooks() {
 					ods("Lib: Failed to insert WNDProc hook");
 			}
 
-			sm->bHooked = true;
+			sd->bHooked = true;
 		}
 		ReleaseMutex(hHookMutex);
 	}
@@ -486,19 +683,13 @@ extern "C" __declspec(dllexport) unsigned int __cdecl GetOverlayMagicVersion() {
 	return OVERLAY_MAGIC_NUMBER;
 }
 
-extern "C" __declspec(dllexport) SharedMem * __cdecl GetSharedMemory() {
-	return sm;
-}
-
 extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
-
 	char procname[1024+64];
 	GetModuleFileName(NULL, procname, 1024);
 
 	switch (fdwReason) {
 		case DLL_PROCESS_ATTACH: {
 				Mutex::init();
-				ods("Lib: ProcAttach: %s", procname);
 				char *p = strrchr(procname, '\\');
 
 				if (p) {
@@ -507,7 +698,7 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 					int i =0;
 					while (blacklist[i]) {
 						if (_stricmp(p+1,blacklist[i])==0) {
-							ods("Process %s is blacklisted", procname);
+							bBlackListed = true;
 							return TRUE;
 						}
 						i++;
@@ -531,57 +722,38 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 					}
 				}
 
-				hSharedMutex = CreateMutex(NULL, false, "MumbleSharedMutex");
+				ods("Lib: ProcAttach: %s", procname);
+
 				hHookMutex = CreateMutex(NULL, false, "MumbleHookMutex");
-				if ((hSharedMutex == NULL) || (hHookMutex == NULL)) {
+				if (hHookMutex == NULL) {
 					ods("Lib: CreateMutex failed");
 					return TRUE;
 				}
 
-				DWORD dwWaitResult = WaitForSingleObject(hSharedMutex, 1000L);
-				if (dwWaitResult != WAIT_OBJECT_0) {
-					ods("Lib: WaitForMutex failed");
-					return TRUE;
-				}
+				DWORD dwSharedSize = sizeof(SharedData) + sizeof(Direct3D9Data) + sizeof(DXGIData);
 
-				DWORD dwSharedSize = sizeof(SharedMem) + sizeof(Direct3D9Data) + sizeof(DXGIData);
-
-				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, dwSharedSize, "MumbleSharedMemory");
+				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, dwSharedSize, "MumbleOverlayPrivate");
 				if (hMapObject == NULL) {
 					ods("Lib: CreateFileMapping failed");
-					ReleaseMutex(hSharedMutex);
 					return TRUE;
 				}
 
 				bool bInit = (GetLastError() != ERROR_ALREADY_EXISTS);
 
-				sm = (SharedMem *) MapViewOfFile(hMapObject, FILE_MAP_ALL_ACCESS, 0, 0, dwSharedSize);
+				sd = (SharedData *) MapViewOfFile(hMapObject, FILE_MAP_ALL_ACCESS, 0, 0, dwSharedSize);
 
-				if (sm == NULL) {
+				if (sd == NULL) {
 					ods("MapViewOfFile Failed");
-					ReleaseMutex(hSharedMutex);
 					return TRUE;
 				}
 
-				unsigned char *raw = (unsigned char *) sm;
-				d3dd = (Direct3D9Data *)(raw + sizeof(SharedMem));
-				dxgi = (DXGIData *)(raw + sizeof(SharedMem) + sizeof(Direct3D9Data));
+				if (bInit)
+					memset(sd, 0, dwSharedSize);
+
+				unsigned char *raw = (unsigned char *) sd;
+				d3dd = (Direct3D9Data *)(raw + sizeof(SharedData));
+				dxgi = (DXGIData *)(raw + sizeof(SharedData) + sizeof(Direct3D9Data));
 
-				if (bInit) {
-					memset(sm, 0, sizeof(SharedMem) + sizeof(Direct3D9Data) + sizeof(DXGIData));
-					sm->lastAppAlive = 0;
-					sm->bHooked = false;
-					sm->bDebug = false;
-					sm->bShow = true;
-					sm->bReset = false;
-					sm->fX = sm->fY = 1.0;
-					sm->bTop = false;
-					sm->bBottom = true;
-					sm->bLeft = true;
-					sm->bRight = false;
-					sm->fFontSize = 72;
-				}
-				ReleaseMutex(hSharedMutex);
 
 				if (! bMumble) {
 					hhLoad.setup(reinterpret_cast<voidFunc>(LoadLibraryA), reinterpret_cast<voidFunc>(MyLoadLibrary));
@@ -596,22 +768,22 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 			}
 			break;
 		case DLL_PROCESS_DETACH: {
+				if (bBlackListed)
+					return TRUE;
 				ods("Lib: ProcDetach: %s", procname);
 				hhLoad.restore(true);
 				hhLoadW.restore(true);
-				if (sm)
-					UnmapViewOfFile(sm);
+				if (sd)
+					UnmapViewOfFile(sd);
 				if (hMapObject)
 					CloseHandle(hMapObject);
-				if (hSharedMutex)
-					CloseHandle(hSharedMutex);
 				if (hHookMutex)
 					CloseHandle(hHookMutex);
 			}
 			break;
 		case DLL_THREAD_ATTACH: {
 				static bool bTriedHook = false;
-				if (sm && ! bTriedHook && ! bMumble) {
+				if (sd && ! bTriedHook && ! bMumble) {
 					bTriedHook = true;
 					checkD3D9Hook();
 					checkDXGIHook();
