@@ -1,3 +1,34 @@
+/* Copyright (C) 2009-2010, Stefan Hacker <dD0t@users.sourceforge.net>
+   Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+
+   All rights reserved.
+
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+
+   - Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
+   - Redistributions in binary form must reproduce the above copyright notice,
+     this list of conditions and the following disclaimer in the documentation
+     and/or other materials provided with the distribution.
+   - Neither the name of the Mumble Developers nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
 #define _USE_MATH_DEFINES
 
 #include <stdio.h>
@@ -42,12 +73,12 @@ static void about(HWND h) {
 }
 
 
-static int fetch(float *pos, float *front, float *top) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
 	float viewHor, viewVer;
 	char state;
 
 	for (int i=0;i<3;i++)
-		pos[i]=front[i]=top[i]=0.0f;
+		avatar_pos[i] = avatar_front[i] = avatar_top[i] = camera_pos[i] = camera_front[i] = camera_top[i] = 0.0f;
 
 	bool ok;
 
@@ -70,9 +101,9 @@ static int fetch(float *pos, float *front, float *top) {
 	if (state == 4)
 		return true; // If this magic value is 4 we are spectating, so switch of PA
 
-	ok = peekProc((BYTE *) 0x01516608, pos+2, 4) &&	//Z
-	     peekProc((BYTE *) 0x0151660C, pos, 4) &&	//X
-	     peekProc((BYTE *) 0x01516610, pos+1, 4) && //Y
+	ok = peekProc((BYTE *) 0x01516608, avatar_pos+2, 4) &&	//Z
+	     peekProc((BYTE *) 0x0151660C, avatar_pos, 4) &&	//X
+	     peekProc((BYTE *) 0x01516610, avatar_pos+1, 4) && //Y
 	     peekProc((BYTE *) 0x0151A114, &viewHor, 4) && //Hor
 	     peekProc((BYTE *) 0x0151A110, &viewVer, 4); //Ver
 
@@ -90,8 +121,10 @@ static int fetch(float *pos, float *front, float *top) {
 	   40 units = 1 meter (not confirmed)
 	*/
 	for (int i=0;i<3;i++)
-		pos[i]/=40.0f; // Scale to meters
-	pos[0]*=(-1.0f); // Convert right to left handed
+		avatar_pos[i]/=40.0f; // Scale to meters
+	avatar_pos[0]*=(-1.0f); // Convert right to left handed
+
+	avatar_top[2] = -1; // Head movement is in front vector
 
 	// Calculate view unit vector
 	/*
@@ -109,11 +142,11 @@ static int fetch(float *pos, float *front, float *top) {
 	viewVer *= static_cast<float>(M_PI / 180.0f);
 	viewHor *= static_cast<float>(M_PI / 180.0f);
 
-	front[0] = -sin(viewHor) * cos(viewVer);
-	front[1] = -sin(viewVer);
-	front[2] = cos(viewHor) * cos(viewVer);
+	avatar_front[0] = -sin(viewHor) * cos(viewVer);
+	avatar_front[1] = -sin(viewVer);
+	avatar_front[2] = cos(viewHor) * cos(viewVer);
 
-	return ok;
+	return true;
 }
 
 static int trylock() {
@@ -126,8 +159,11 @@ static int trylock() {
 	if (!h)
 		return false;
 
-	float pos[3], front[3], top[3];
-	if (fetch(pos, front, top))
+	float apos[3], afront[3], atop[3], cpos[3], cfront[3], ctop[3];
+	std::string context;
+	std::wstring identity;
+
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, context, identity))
 		return true;
 
 	CloseHandle(h);
@@ -142,14 +178,22 @@ static void unlock() {
 	}
 }
 
+static const std::wstring longdesc() {
+	return std::wstring(L"Supports Call of Duty 2 v1.3. No context or identity support yet.");
+}
+
+static std::wstring description(L"Call of Duty 2 v1.3");
+static std::wstring shortname(L"Call of Duty 2");
+
 static MumblePlugin cod2plug = {
 	MUMBLE_PLUGIN_MAGIC,
-	L"Call of Duty 2 v1.3",
-	L"Call of Duty 2",
+	description,
+	shortname,
 	about,
 	NULL,
 	trylock,
 	unlock,
+	longdesc,
 	fetch
 };
 
