@@ -70,7 +70,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Call of Duty: Modern Warfare 2 Multiplayer v1.0.174). IP:Port context without team discriminator.", L"Mumble CoDMW2 MP Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Call of Duty: Modern Warfare 2 Multiplayer v1.0.182.", L"Mumble CoDMW2 MP Plugin", MB_OK);
 }
 
 
@@ -79,7 +79,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	char state;
 
 	for (int i=0;i<3;i++)
-		avatar_pos[i]=avatar_front[i]=avatar_top[i]=0.0f;
+		avatar_pos[i] = avatar_front[i] = avatar_top[i] = camera_pos[i] = camera_front[i] = camera_top[i] = 0.0f;
 
 	bool ok;
 
@@ -88,34 +88,34 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 			Address			Type	Description
 			===================================
-			0x0085C668		float	Z-Coordinate
-			0x0085C66C		float	X-Coordinate
-			0x0085C670		float	Y-Coordinate
-			0x0085C654		float	Horizontal view (degrees)
-			0x0085C650		float	Vertical view (degrees)
+			0x008F0F78		float	Z-Coordinate
+			0x008F0F7C		float	X-Coordinate
+			0x008F0F80		float	Y-Coordinate
+			0x008F0F88		float	Horizontal view (degrees)
+			0x008F0F84		float	Vertical view (degrees)
 
-			0x007EE19D		byte	Magical state value
+			0x007F7A34		byte	Magical state value
 	*/
-	ok = peekProc((BYTE *) 0x007EF19D, &state, 1); // Magical state value
+	ok = peekProc((BYTE *) 0x007F7A34, &state, 1); // Magical state value
 	if (! ok)
 		return false;
 	/*
 		state value is:
 		0		while not in game
-		4		while playing
-		0		while spectating
+		1		while playing
+		5		while spectating
 
 		This value is used for disabling PA for spectators
 		or people not on a server.
 	*/
-	if (state != 4)
+	if (state != 1)
 		return true; // This results in all vectors beeing zero which tells mumble to ignore them.
 
-	ok = peekProc((BYTE *) 0x00862E5C, avatar_pos+2, 4) &&	//Z
-	     peekProc((BYTE *) 0x00862E60, avatar_pos, 4) &&	//X
-	     peekProc((BYTE *) 0x00862E64, avatar_pos+1, 4) && //Y
-	     peekProc((BYTE *) 0x00862E54, &viewHor, 4) && //Hor
-	     peekProc((BYTE *) 0x00862E50, &viewVer, 4); //Ver
+	ok = peekProc((BYTE *) 0x008F0F78, avatar_pos+2, 4) &&	//Z
+	     peekProc((BYTE *) 0x008F0F7C, avatar_pos, 4) &&	//X
+	     peekProc((BYTE *) 0x008F0F80, avatar_pos+1, 4) && //Y
+	     peekProc((BYTE *) 0x008F0F88, &viewHor, 4) && //Hor
+	     peekProc((BYTE *) 0x008F0F84, &viewVer, 4); //Ver
 
 	if (! ok)
 		return false;
@@ -162,7 +162,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 		camera_top[i] = avatar_top[i];
 	}
 
-	return ok;
+	return true;
 }
 
 static int trylock() {
@@ -195,10 +195,10 @@ static void unlock() {
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports Call of Duty: Modern Warfare 2 MP v1.0.174 only. No identity support yet.");
+	return std::wstring(L"Supports Call of Duty: Modern Warfare 2 MP v1.0.182 only. No context or identity support yet.");
 }
 
-static std::wstring description(L"Call of Duty: Modern Warfare 2 MP v1.0.174");
+static std::wstring description(L"Call of Duty: Modern Warfare 2 MP v1.0.182");
 static std::wstring shortname(L"Call of Duty: Modern Warfare 2 MP");
 
 static MumblePlugin codmw2plug = {
