@@ -1,3 +1,34 @@
+/* Copyright (C) 2009-2010, deKarl <dekarl@users.sourceforge.net>
+   Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+
+   All rights reserved.
+
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+
+   - Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
+   - Redistributions in binary form must reproduce the above copyright notice,
+     this list of conditions and the following disclaimer in the documentation
+     and/or other materials provided with the distribution.
+   - Neither the name of the Mumble Developers nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
 /* stdint.h */
 typedef int uint32_t;
 typedef long long uint64_t;
@@ -23,10 +54,10 @@ uint64_t g_playerGUID;
  * To update visit http://www.mmowned.com/forums/wow-memory-editing/
  * and look for a thread called [WoW][TheVersion] Info Dump Thread.
  */
-static uint32_t ptr_ClientConnection=0x00C923C0;
+static uint32_t ptr_ClientConnection=0x00C93410;
 static size_t off_ObjectManager=0x2E04;
-static uint32_t ptr_WorldFrame=0x00B0C544;
-static size_t off_CameraOffset=0x7E20;
+static uint32_t ptr_WorldFrame=0x00B0D578;
+static size_t off_CameraOffset=0x7E24;
 
 static DWORD getProcess(const wchar_t *exename) {
 	PROCESSENTRY32 pe;
@@ -199,7 +230,7 @@ uint32_t getPlayerBase() {
 	return playerBase;
 }
 
-static const unsigned long nameStorePtr        = 0x00C76018 + 0x8;  // Player name database
+static const unsigned long nameStorePtr        = 0x00C77068 + 0x8;  // Player name database
 static const unsigned long nameMaskOffset      = 0x024;  // Offset for the mask used with GUID to select a linked list
 static const unsigned long nameBaseOffset      = 0x01c;  // Offset for the start of the name linked list
 static const unsigned long nameStringOffset    = 0x020;  // Offset to the C string in a name structure
@@ -450,10 +481,10 @@ static void unlock() {
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports World of Warcraft 3.3.0 (11159) (Release) (Euro). With identity support.");
+	return std::wstring(L"Supports World of Warcraft 3.3.2 (11403) (Release) (Euro). With identity support.");
 }
 
-static std::wstring description(L"World of Warcraft 3.3.0 (Euro)");
+static std::wstring description(L"World of Warcraft 3.3.2 (Euro)");
 
 static std::wstring shortname(L"World of Warcraft");
 
