@@ -22,7 +22,7 @@ try {
     echo "<table><tr><th>Name</th><th>Channel</th></tr>\n";
 
     $channels = $s->getChannels();
-    $players = $s->getPlayers();
+    $players = $s->getUsers();
 
     foreach($players as $id => $state) {
       $chan = $channels[$state->channel];
