@@ -131,8 +131,8 @@ void Connection::socketRead() {
 			unsigned char a_ucBuffer[6];
 
 			qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 6);
-			uiType = qFromBigEndian(* reinterpret_cast<quint16 *>(& a_ucBuffer[0]));
-			iPacketLength = qFromBigEndian(* reinterpret_cast<quint32 *>(& a_ucBuffer[2]));
+			uiType = qFromBigEndian<quint16>(&a_ucBuffer[0]);
+			iPacketLength = qFromBigEndian<quint32>(&a_ucBuffer[2]);
 			iAvailable -= 6;
 		}
 
@@ -176,8 +176,8 @@ void Connection::messageToNetwork(const ::google::protobuf::Message &msg, unsign
 		return;
 	cache.resize(len + 6);
 	unsigned char *uc = reinterpret_cast<unsigned char *>(cache.data());
-	* reinterpret_cast<quint16 *>(& uc[0]) = qToBigEndian(static_cast<quint16>(msgType));
-	* reinterpret_cast<quint32 *>(& uc[2]) = qToBigEndian(static_cast<quint32>(len));
+	qToBigEndian<quint16>(msgType, & uc[0]);
+	qToBigEndian<quint32>(len, & uc[2]);
 
 	msg.SerializeToArray(uc + 6, len);
 }
