@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -33,19 +33,16 @@
 
 #include "murmur_pch.h"
 #include "CryptState.h"
-
-class Message;
+#include "Mumble.pb.h"
 
 class Connection : public QObject {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(Connection)
-	private:
-		static int iReceiveLevel;
-		static QSet<Connection *> qsReceivers;
 	protected:
 		QSslSocket *qtsSocket;
 		QTime qtLastPacket;
+		unsigned int uiType;
 		int iPacketLength;
 		bool bDisconnectedEmitted;
 #ifdef Q_OS_WIN
@@ -60,15 +57,15 @@ class Connection : public QObject {
 	public slots:
 		void proceedAnyway();
 	signals:
-		void connectionClosed(QString reason);
-		void message(QByteArray &);
+		void encrypted();
+		void connectionClosed(QAbstractSocket::SocketError, const QString &reason);
+		void message(unsigned int type, const QByteArray &);
 		void handleSslErrors(const QList<QSslError> &);
-
-		void recheckBuffer();
 	public:
 		Connection(QObject *parent, QSslSocket *qtsSocket);
 		~Connection();
-		void sendMessage(const Message *mMsg);
+		static void messageToNetwork(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache);
+		void sendMessage(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache);
 		void sendMessage(const QByteArray &qbaMsg);
 		void disconnectSocket(bool force=false);
 		void forceFlush();
@@ -76,15 +73,6 @@ class Connection : public QObject {
 
 		CryptState csCrypt;
 
-		double dUDPPingAvg;
-		double dUDPPingVar;
-		quint32 uiUDPPackets;
-		double dTCPPingAvg;
-		double dTCPPingVar;
-		quint32 uiTCPPackets;
-
-		static void updatePing(double &avg, double &var, quint32 &samples, quint64 usec);
-
 		QList<QSslCertificate> peerCertificateChain() const;
 		QSslCipher sessionCipher() const;
 		QHostAddress peerAddress() const;
@@ -96,6 +84,8 @@ class Connection : public QObject {
 #endif
 };
 
+Q_DECLARE_METATYPE(QAbstractSocket::SocketError);
+
 #else
 class Connection;
 #endif
