@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -34,7 +34,8 @@
 #include "murmur_pch.h"
 
 class Channel;
-class Player;
+class User;
+class ServerUser;
 
 class Group {
 	private:
@@ -47,13 +48,15 @@ class Group {
 		QSet<int> qsAdd;
 		QSet<int> qsRemove;
 		QSet<int> qsTemporary;
-		QSet<int> members();
-		Group(Channel *assoc, QString name);
+		Group(Channel *assoc, const QString &name);
 
+#ifdef MURMUR
+		QSet<int> members();
 		static QSet<QString> groupNames(Channel *c);
 		static Group *getGroup(Channel *c, QString name);
 
-		static bool isMember(Channel *c, Channel *aclChan, QString name, Player *);
+		static bool isMember(Channel *c, Channel *aclChan, QString name, ServerUser *);
+#endif
 };
 
 #else
