@@ -36,7 +36,7 @@
 #include "User.h"
 #include "Global.h"
 #include "Log.h"
-
+#include "Database.h"
 
 ACLGroup::ACLGroup(const QString &name) : Group(NULL, name) {
 	bInherited = false;
@@ -55,11 +55,6 @@ ACLEditor::ACLEditor(int channelparentid, QWidget *p) : QDialog(p) {
 	qtwTab->removeTab(2);
 	qtwTab->removeTab(1);
 
-	if (!g.s.bExpert) {
-		qsbChannelPosition->hide();
-		qlChannelPosition->hide();
-	}
-
 	// Until I come around implementing it hide the password fields
 	qleChannelPassword->hide();
 	qlChannelPassword->hide();
@@ -87,12 +82,6 @@ ACLEditor::ACLEditor(int channelid, const MumbleProto::ACL &mea, QWidget *p) : Q
 
 	setupUi(this);
 
-	if (!g.s.bExpert) {
-		qtwTab->removeTab(2);
-		qtwTab->removeTab(1);
-		qsbChannelPosition->hide();
-		qlChannelPosition->hide();
-	}
 	qcbChannelTemporary->hide();
 
 	iId = mea.channel_id();
@@ -252,13 +241,7 @@ void ACLEditor::accept() {
 	}
 	// Update channel state
 	if (bAddChannelMode) {
-		MumbleProto::ChannelState mpcs;
-		mpcs.set_name(u8(qleChannelName->text()));
-		mpcs.set_description(u8(rteChannelDescription->text()));
-		mpcs.set_parent(iChannel);
-		mpcs.set_position(qsbChannelPosition->value());
-		mpcs.set_temporary(qcbChannelTemporary->isChecked());
-		g.sh->sendMessage(mpcs);
+		g.sh->createChannel(iChannel, qleChannelName->text(), rteChannelDescription->text(), qsbChannelPosition->value(), qcbChannelTemporary->isChecked());
 	} else {
 		bool b = false;
 
@@ -271,14 +254,17 @@ void ACLEditor::accept() {
 			b = true;
 		}
 		if (rteChannelDescription->isModified() && (pChannel->qsDesc != rteChannelDescription->text())) {
-			mpcs.set_description(u8(rteChannelDescription->text()));
+			const QString &msg = rteChannelDescription->text();
+			mpcs.set_description(u8(msg));
 			b = true;
+			Database::setBlob(sha1(msg), msg.toUtf8());
 		}
 		if (pChannel->iPosition != qsbChannelPosition->value()) {
 			mpcs.set_position(qsbChannelPosition->value());
 			b = true;
 		}
-		if (b) g.sh->sendMessage(mpcs);
+		if (b)
+			g.sh->sendMessage(mpcs);
 
 		// Update ACL
 		msg.set_inherit_acls(bInheritACL);
@@ -579,8 +565,10 @@ void ACLEditor::ACLEnableCheck() {
 
 	int idx;
 	for (idx=0;idx<qlACLAllow.count();idx++) {
-		qlACLAllow[idx]->setEnabled(ena);
-		qlACLDeny[idx]->setEnabled(ena);
+		// Only enable other checkboxes if writeacl isn't set
+		bool enathis = ena && (qlPerms[idx] == ChanACL::Write || !(as && static_cast<int>(as->pAllow) & ChanACL::Write) || qlPerms[idx] == ChanACL::Speak);
+		qlACLAllow[idx]->setEnabled(enathis);
+		qlACLDeny[idx]->setEnabled(enathis);
 	}
 
 	if (as) {
@@ -821,6 +809,7 @@ void ACLEditor::ACLPermissions_clicked() {
 	a = 0;
 	d = 0;
 
+	bool ena = true;
 	for (idx=0;idx<qlACLAllow.count();idx++) {
 		ChanACL::Perm p = qlPerms[idx];
 		if (qlACLAllow[idx]->isChecked() && qlACLDeny[idx]->isChecked()) {
@@ -829,6 +818,13 @@ void ACLEditor::ACLPermissions_clicked() {
 			else
 				qlACLAllow[idx]->setChecked(false);
 		}
+
+		qlACLAllow[idx]->setEnabled(ena || p == ChanACL::Speak);
+		qlACLDeny[idx]->setEnabled(ena || p == ChanACL::Speak);
+
+		if (p == ChanACL::Write && qlACLAllow[idx]->isChecked())
+			ena = false;
+
 		if (qlACLAllow[idx]->isChecked())
 			a |= p;
 		if (qlACLDeny[idx]->isChecked())
