@@ -1,4 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -130,7 +131,7 @@ class AudioOutputSpeech : public AudioOutputUser {
 		JitterBuffer *jbJitter;
 		int iMissCount;
 
-		CELTMode *cmMode;
+		CELTCodec *cCodec;
 		CELTDecoder *cdDecoder;
 
 		SpeexBits sbBits;
@@ -197,6 +198,7 @@ class AudioOutputSample : public AudioOutputUser {
 		void playbackFinished();
 	public:
 		static SoundFile* loadSndfile(const QString &filename);
+		static QString browseForSndfile();
 		virtual bool needSamples(unsigned int snum);
 		AudioOutputSample(const QString &name, SoundFile *psndfile, bool repeat, unsigned int freq);
 		~AudioOutputSample();
