@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -93,6 +93,13 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 		qcbOutput->setEnabled(false);
 	}
 
+	qcbHighContrast->setChecked(g.s.bHighContrast);
+#ifdef Q_OS_WIN
+	// On windows we can autodetect this
+	qcbHighContrast->setVisible(false);
+#endif
+
+	// Settings
 	if (g.s.iQuality == 16000 && g.s.iFramesPerPacket == 6)
 		qrbQualityLow->setChecked(true);
 	else if (g.s.iQuality == 40000 && g.s.iFramesPerPacket == 2)
@@ -102,6 +109,20 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	else
 		qrbQualityCustom->setChecked(true);
 
+	quint32 iMessage = Settings::LogNone;
+	for (int i = Log::firstMsgType;i <= Log::lastMsgType; ++i) {
+		iMessage |= (g.s.qmMessages[i] & (Settings::LogSoundfile | Settings::LogTTS));
+	}
+
+	if (iMessage == Settings::LogTTS)
+		qrbNotificationTTS->setChecked(true);
+	else if (iMessage == Settings::LogSoundfile)
+		qrbNotificationSounds->setChecked(true);
+	else
+		qrbNotificationCustom->setChecked(true);
+
+	qrbNotificationCustom->setVisible(qrbNotificationCustom->isChecked());
+
 	qrbQualityCustom->setVisible(qrbQualityCustom->isChecked());
 	qlQualityCustom->setVisible(qrbQualityCustom->isChecked());
 
@@ -373,7 +394,9 @@ void AudioWizard::restartAudio() {
 	g.ao.reset();
 
 	while (! ai.unique() || ! ao.unique()) {
-		// Where is QThread::yield() ?
+#if QT_VERSION >= 0x040500
+		QThread::yieldCurrentThread();
+#endif
 	}
 
 	ai.reset();
@@ -550,7 +573,7 @@ void AudioWizard::on_qrPTT_clicked(bool on) {
 	}
 }
 
-void AudioWizard::on_skwPTT_keySet(bool valid) {
+void AudioWizard::on_skwPTT_keySet(bool valid, bool last) {
 	if (valid)
 		qrPTT->setChecked(true);
 	else if (qrPTT->isChecked())
@@ -558,31 +581,33 @@ void AudioWizard::on_skwPTT_keySet(bool valid) {
 	updateTriggerWidgets(valid);
 	bTransmitChanged = true;
 
-	const QList<QVariant> &buttons = skwPTT->getShortcut();
-	QList<Shortcut> ql;
-	bool found = false;
-	foreach(Shortcut s, g.s.qlShortcuts) {
-		if (s.iIndex == g.mw->gsPushTalk->idx) {
-			if (buttons.isEmpty())
-				continue;
-			else if (! found) {
-				s.qlButtons = buttons;
-				found = true;
-			}
-		}
-		ql << s;
-	}
-	if (! found && ! buttons.isEmpty()) {
-		Shortcut s;
-		s.iIndex = g.mw->gsPushTalk->idx;
-		s.bSuppress = false;
-		s.qlButtons = buttons;
-		ql << s;
-	}
-	g.s.qlShortcuts = ql;
+	if (last) {
 
-	GlobalShortcutEngine::engine->bNeedRemap = true;
-	GlobalShortcutEngine::engine->needRemap();
+		const QList<QVariant> &buttons = skwPTT->getShortcut();
+		QList<Shortcut> ql;
+		bool found = false;
+		foreach(Shortcut s, g.s.qlShortcuts) {
+			if (s.iIndex == g.mw->gsPushTalk->idx) {
+				if (buttons.isEmpty())
+					continue;
+				else if (! found) {
+					s.qlButtons = buttons;
+					found = true;
+				}
+			}
+			ql << s;
+		}
+		if (! found && ! buttons.isEmpty()) {
+			Shortcut s;
+			s.iIndex = g.mw->gsPushTalk->idx;
+			s.bSuppress = false;
+			s.qlButtons = buttons;
+			ql << s;
+		}
+		g.s.qlShortcuts = ql;
+		GlobalShortcutEngine::engine->bNeedRemap = true;
+		GlobalShortcutEngine::engine->needRemap();
+	}
 }
 
 void AudioWizard::on_qcbEcho_clicked(bool on) {
@@ -610,6 +635,10 @@ void AudioWizard::on_qcbAttenuateOthers_clicked(bool checked) {
 	g.s.bAttenuateOthers = checked;
 }
 
+void AudioWizard::on_qcbHighContrast_clicked(bool checked) {
+	g.s.bHighContrast = checked;
+}
+
 void AudioWizard::on_qrbQualityLow_clicked() {
 	g.s.iQuality = 16000;
 	g.s.iFramesPerPacket = 6;
