@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -34,15 +34,17 @@
 #include "mumble_pch.hpp"
 #include "ui_BanEditor.h"
 #include "Message.h"
+#include "Net.h"
 
 class BanEditor : public QDialog, public Ui::BanEditor {
 	private:
 		Q_OBJECT;
 		Q_DISABLE_COPY(BanEditor)
 	protected:
-		QList<QPair<quint32, int> > qlBans;
+		QList<Ban> qlBans;
+		Ban toBan(bool &);
 	public:
-		BanEditor(const MessageServerBanList *msbl, QWidget *p = NULL);
+		BanEditor(const MumbleProto::BanList &msbl, QWidget *p = NULL);
 	public slots:
 		void accept();
 	public slots:
