@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -68,6 +68,13 @@ CertView::CertView(QWidget *p) : QGroupBox(p) {
 	qlIssuerName->setWordWrap(true);
 	grid->addWidget(qlIssuerName, 2, 1, 1, 1);
 
+	l = new QLabel(tr("Expiry Date"));
+	grid->addWidget(l, 3, 0, 1, 1, Qt::AlignRight);
+
+	qlExpiry = new QLabel();
+	qlExpiry->setWordWrap(true);
+	grid->addWidget(qlExpiry, 3, 1, 1, 1);
+
 	grid->setColumnStretch(1, 1);
 }
 
@@ -78,6 +85,7 @@ void CertView::setCert(const QList<QSslCertificate> &cert) {
 		qlSubjectName->setText(QString());
 		qlSubjectEmail->setText(QString());
 		qlIssuerName->setText(QString());
+		qlExpiry->setText(QString());
 	} else {
 		QSslCertificate qscCert = qlCert.at(0);
 
@@ -96,6 +104,11 @@ void CertView::setCert(const QList<QSslCertificate> &cert) {
 		else
 			qlSubjectEmail->setText(tr("(none)"));
 
+		if (qscCert.expiryDate() <= QDateTime::currentDateTime())
+			qlExpiry->setText(QString::fromLatin1("<font color=\"red\"><b>%1</b></font>").arg(qscCert.expiryDate().toString(Qt::SystemLocaleDate)));
+		else
+			qlExpiry->setText(qscCert.expiryDate().toString(Qt::SystemLocaleDate));
+
 		if (qlCert.count() > 1)
 			qscCert = qlCert.last();
 
@@ -344,6 +357,10 @@ void CertWizard::on_qlePassword_textChanged(const QString &) {
 	on_qleImportFile_textChanged(qleImportFile->text());
 }
 
+void CertWizard::on_qlIntroText_linkActivated(const QString &url) {
+	QDesktopServices::openUrl(QUrl(url));
+}
+
 void CertWizard::lookedUp(QHostInfo info) {
 	bPendingDns = false;
 	if (info.error() == QHostInfo::NoError) {
