@@ -246,6 +246,8 @@ ServerItem::ServerItem(const FavoriteServer &fs) : QTreeWidgetItem(QTreeWidgetIt
 
 	qsUrl = fs.qsUrl;
 
+	bCA = false;
+
 	if (fs.qsHostname.startsWith(QLatin1Char('@'))) {
 		qsBonjourHost = fs.qsHostname.mid(1);
 		brRecord = BonjourRecord(qsBonjourHost, QLatin1String("_mumble._tcp."), QLatin1String("local."));
@@ -267,6 +269,7 @@ ServerItem::ServerItem(const PublicInfo &pi) : QTreeWidgetItem(QTreeWidgetItem::
 	qsCountry = pi.qsCountry;
 	qsCountryCode = pi.qsCountryCode;
 	qsContinentCode = pi.qsContinentCode;
+	bCA = pi.bCA;
 
 	init();
 }
@@ -278,6 +281,7 @@ ServerItem::ServerItem(const QString &name, const QString &host, unsigned short
 	qsName = name;
 	usPort = port;
 	qsUsername = username;
+	bCA = false;
 
 	if (host.startsWith(QLatin1Char('@'))) {
 		qsBonjourHost = host.mid(1);
@@ -297,6 +301,7 @@ ServerItem::ServerItem(const BonjourRecord &br) : QTreeWidgetItem(QTreeWidgetIte
 	qsBonjourHost = qsName;
 	brRecord = br;
 	usPort = 0;
+	bCA = false;
 
 	init();
 }
@@ -311,6 +316,7 @@ ServerItem::ServerItem(const QString &name, ItemType itype, const QString &conti
 		qsContinentCode = continent;
 	}
 	setFlags(flags() & ~Qt::ItemIsDragEnabled);
+	bCA = false;
 
 	init();
 }
@@ -332,6 +338,7 @@ ServerItem::ServerItem(const ServerItem *si) {
 	qsBonjourHost = si->qsBonjourHost;
 	brRecord = si->brRecord;
 	qlAddresses = si->qlAddresses;
+	bCA = si->bCA;
 
 	uiVersion = si->uiVersion;
 	uiPing = si->uiPing;
@@ -418,13 +425,13 @@ QVariant ServerItem::data(int column, int role) const {
 					return qsName;
 				case Qt::DecorationRole:
 					if (itType == FavoriteType)
-						return loadIcon(QLatin1String(":/emblems/emblem-favorite.svg"));
+						return loadIcon(QLatin1String("skin:emblems/emblem-favorite.svg"));
 					else if (itType == LANType)
-						return loadIcon(QLatin1String(":/places/network-workgroup.svg"));
+						return loadIcon(QLatin1String("skin:places/network-workgroup.svg"));
 					else if (! qsCountryCode.isEmpty())
 						return loadIcon(QString::fromLatin1(":/flags/%1.png").arg(qsCountryCode));
 					else
-						return loadIcon(QLatin1String(":/categories/applications-internet.svg"));
+						return loadIcon(QLatin1String("skin:categories/applications-internet.svg"));
 			}
 		}
 	} else {
@@ -478,6 +485,12 @@ QVariant ServerItem::data(int column, int role) const {
 			}
 			qs += QLatin1String("</table>");
 			return qs;
+		} else if (role == Qt::BackgroundRole) {
+			if (bCA) {
+				QColor qc(Qt::green);
+				qc.setAlpha(32);
+				return qc;
+			}
 		}
 	}
 	return QTreeWidgetItem::data(column, role);
@@ -808,6 +821,11 @@ ConnectDialog::ConnectDialog(QWidget *p) : QDialog(p) {
 	bLastFound = false;
 
 	qmPingCache = Database::getPingCache();
+
+	if (! g.s.qbaConnectDialogGeometry.isEmpty())
+		restoreGeometry(g.s.qbaConnectDialogGeometry);
+	if (! g.s.qbaConnectDialogHeader.isEmpty())
+		qtwServers->header()->restoreState(g.s.qbaConnectDialogHeader);
 }
 
 ConnectDialog::~ConnectDialog() {
@@ -826,6 +844,9 @@ ConnectDialog::~ConnectDialog() {
 	}
 	Database::setFavorites(ql);
 	Database::setPingCache(qmPingCache);
+
+	g.s.qbaConnectDialogHeader = qtwServers->header()->saveState();
+	g.s.qbaConnectDialogGeometry = saveGeometry();
 }
 
 void ConnectDialog::accept() {
@@ -1124,6 +1145,7 @@ void ConnectDialog::fillList() {
 				si->qsCountryCode = pi.qsCountryCode;
 				si->qsContinentCode = pi.qsContinentCode;
 				si->qsUrl = pi.quUrl.toString();
+				si->bCA = pi.bCA;
 				si->setDatas();
 
 				if (si->itType == ServerItem::PublicType)
@@ -1406,7 +1428,7 @@ void ConnectDialog::finished() {
 			return;
 		}
 
-		QMessageBox::warning(this, tr("Mumble"), tr("Failed to fetch server list"), QMessageBox::Ok);
+		QMessageBox::warning(this, QLatin1String("Mumble"), tr("Failed to fetch server list"), QMessageBox::Ok);
 		return;
 	}
 
@@ -1432,6 +1454,7 @@ void ConnectDialog::finished() {
 				pi.qsCountry = e.attribute(QLatin1String("country"));
 				pi.qsCountryCode = e.attribute(QLatin1String("country_code")).toLower();
 				pi.qsContinentCode = e.attribute(QLatin1String("continent_code")).toLower();
+				pi.bCA = e.attribute(QLatin1String("ca")).toInt() ? true : false;
 
 				qlPublicServers << pi;
 			}
