@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -52,7 +52,7 @@ class Database : public QObject {
 		static QList<FavoriteServer> getFavorites();
 		static void setFavorites(const QList<FavoriteServer> &servers);
 		static void setPassword(const QString &host, unsigned short port, const QString &user, const QString &pw);
-		static bool fuzzyMatch(QString &user, QString &pw, QString &host, unsigned short port);
+		static bool fuzzyMatch(QString &name, QString &user, QString &pw, QString &host, unsigned short port);
 
 		static bool isLocalMuted(const QString &hash);
 		static void setLocalMuted(const QString &hash, bool muted);
@@ -60,8 +60,11 @@ class Database : public QObject {
 		static QMap<QPair<QString, unsigned short>, unsigned int> getPingCache();
 		static void setPingCache(const QMap<QPair<QString, unsigned short>, unsigned int> &cache);
 
-		static bool seenComment(const QString &hash, const QString &comment);
-		static void setSeenComment(const QString &hash, const QString &comment);
+		static bool seenComment(const QString &hash, const QByteArray &commenthash);
+		static void setSeenComment(const QString &hash, const QByteArray &commenthash);
+
+		static QByteArray blob(const QByteArray &hash);
+		static void setBlob(const QByteArray &hash, const QByteArray &blob);
 
 		static QStringList getTokens(const QByteArray &digest);
 		static void setTokens(const QByteArray &digest, QStringList &tokens);
