@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,7 +31,7 @@
 #include "DirectSound.h"
 #include "MainWindow.h"
 #include "Plugins.h"
-#include "Player.h"
+#include "User.h"
 #include "Global.h"
 
 #undef FAILED
@@ -85,6 +85,7 @@ void DirectSoundInit::initialize() {
 	ovi.dwOSVersionInfoSize=sizeof(ovi);
 	GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
 
+#ifdef QT_NO_DEBUG
 	if ((ovi.dwMajorVersion > 6) || ((ovi.dwMajorVersion == 6) && (ovi.dwBuildNumber >= 6001))) {
 		HMODULE hLib = LoadLibrary(L"AVRT.DLL");
 		if (hLib != NULL) {
@@ -93,6 +94,7 @@ void DirectSoundInit::initialize() {
 			return;
 		}
 	}
+#endif
 #endif
 
 	airReg = new DXAudioInputRegistrar();
@@ -364,17 +366,17 @@ void DXAudioOutput::run() {
 	dsbdesc.lpwfxFormat     = reinterpret_cast<WAVEFORMATEX *>(&wfx);
 
 	if (FAILED(hr = pDS->CreateSoundBuffer(&dsbdesc, &pDSBOutput, NULL))) {
-		qWarning("DXAudioOutputPlayer: CreateSoundBuffer (Secondary): 0x%08lx", hr);
+		qWarning("DXAudioOutputUser: CreateSoundBuffer (Secondary): 0x%08lx", hr);
 		goto cleanup;
 	}
 
 
 	if (FAILED(hr = pDSBOutput->QueryInterface(IID_IDirectSoundNotify, reinterpret_cast<void **>(&pDSNotify)))) {
-		qWarning("DXAudioOutputPlayer: QueryInterface (Notify)");
+		qWarning("DXAudioOutputUser: QueryInterface (Notify)");
 		goto cleanup;
 	}
 
-	qWarning("DXAudioOutputPlayer: New %dHz output buffer of %ld bytes", SAMPLE_RATE, dsbdesc.dwBufferBytes);
+	qWarning("DXAudioOutputUser: New %dHz output buffer of %ld bytes", SAMPLE_RATE, dsbdesc.dwBufferBytes);
 
 	if (failed)
 		g.mw->msgBox(tr("Opening chosen DirectSound Output failed. Default device will be used."));
@@ -382,7 +384,7 @@ void DXAudioOutput::run() {
 	initializeMixer(chanmasks, bHead);
 
 	if (FAILED(hr = pDSBOutput->Lock(0, 0, &aptr1, &nbytes1, &aptr2, &nbytes2, DSBLOCK_ENTIREBUFFER))) {
-		qWarning("DXAudioOutputPlayer: Initial Lock");
+		qWarning("DXAudioOutputUser: Initial Lock");
 		goto cleanup;
 	}
 
@@ -393,12 +395,12 @@ void DXAudioOutput::run() {
 		ZeroMemory(aptr2, nbytes2);
 
 	if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
-		qWarning("DXAudioOutputPlayer: Initial Unlock");
+		qWarning("DXAudioOutputUser: Initial Unlock");
 		goto cleanup;
 	}
 
 	if (FAILED(hr = pDSBOutput->Play(0, 0, DSBPLAY_LOOPING))) {
-		qWarning("DXAudioOutputPlayer: Play");
+		qWarning("DXAudioOutputUser: Play");
 		goto cleanup;
 	}
 
@@ -412,7 +414,7 @@ void DXAudioOutput::run() {
 
 	while (bRunning && ! FAILED(hr)) {
 		if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
-			qWarning("DXAudioOutputPlayer: GetCurrentPosition");
+			qWarning("DXAudioOutputUser: GetCurrentPosition");
 			break;
 		}
 
@@ -443,7 +445,7 @@ void DXAudioOutput::run() {
 			}
 
 			if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
-				qWarning("DXAudioOutputPlayer: GetCurrentPosition");
+				qWarning("DXAudioOutputUser: GetCurrentPosition");
 				break;
 			}
 
@@ -478,6 +480,29 @@ cleanup:
 #define NBUFFBLOCKS 50
 
 DXAudioInput::DXAudioInput() {
+	bRunning = true;
+}
+
+DXAudioInput::~DXAudioInput() {
+	bRunning = false;
+	wait();
+}
+
+void DXAudioInput::run() {
+	LPDIRECTSOUNDCAPTURE8      pDSCapture;
+	LPDIRECTSOUNDCAPTUREBUFFER pDSCaptureBuffer;
+	LPDIRECTSOUNDNOTIFY8       pDSNotify;
+
+	DWORD dwBufferSize;
+	bool bOk;
+	DWORD dwReadyBytes;
+	DWORD dwLastReadPos = 0;
+	DWORD dwReadPosition;
+	DWORD dwCapturePosition;
+
+	LPVOID aptr1, aptr2;
+	DWORD nbytes1, nbytes2;
+
 	HRESULT       hr;
 	WAVEFORMATEX  wfx;
 	DSCBUFFERDESC dscbd;
@@ -489,6 +514,11 @@ DXAudioInput::DXAudioInput() {
 	bOk = false;
 
 	bool failed = false;
+	float safety = 2.0f;
+	bool didsleep = false;
+	bool firstsleep = false;
+
+	Timer t;
 
 	ZeroMemory(&wfx, sizeof(wfx));
 	wfx.wFormatTag = WAVE_FORMAT_PCM;
@@ -500,7 +530,7 @@ DXAudioInput::DXAudioInput() {
 	dscbd.lpwfxFormat = &wfx;
 
 	wfx.nChannels = 1;
-	wfx.nSamplesPerSec = SAMPLE_RATE;
+	wfx.nSamplesPerSec = iSampleRate;
 	wfx.nBlockAlign = 2;
 	wfx.nAvgBytesPerSec = wfx.nSamplesPerSec * wfx.nBlockAlign;
 	wfx.wBitsPerSample = 16;
@@ -523,49 +553,14 @@ DXAudioInput::DXAudioInput() {
 		bOk = true;
 
 
-	if (! bOk) {
-		g.mw->msgBox(tr("Opening chosen DirectSound Input device failed. No microphone capture will be done."));
-		return;
-	}
 
 	if (failed)
 		g.mw->msgBox(tr("Opening chosen DirectSound Input failed. Default device will be used."));
 
 	qWarning("DXAudioInput: Initialized");
 
-	bRunning = true;
-}
-
-DXAudioInput::~DXAudioInput() {
-	bRunning = false;
-	wait();
-
-	if (pDSNotify)
-		pDSNotify->Release();
-	if (pDSCaptureBuffer)
-		pDSCaptureBuffer->Release();
-	if (pDSCapture)
-		pDSCapture->Release();
-}
-
-void DXAudioInput::run() {
-	HRESULT       hr;
-	DWORD dwReadyBytes;
-	DWORD dwLastReadPos = 0;
-	DWORD dwReadPosition;
-	DWORD dwCapturePosition;
-
-	LPVOID aptr1, aptr2;
-	DWORD nbytes1, nbytes2;
-
 	if (! bOk)
-		return;
-
-	float safety = 2.0f;
-	bool didsleep = false;
-	bool firstsleep = false;
-
-	Timer t;
+		goto cleanup;
 
 	if (FAILED(hr = pDSCaptureBuffer->Start(DSCBSTART_LOOPING))) {
 		qWarning("DXAudioInput: Start failed");
@@ -587,7 +582,6 @@ void DXAudioInput::run() {
 
 				if (static_cast<int>(dwReadyBytes) < sizeof(short) * iFrameSize) {
 					double msecleft = 20.0 - (dwReadyBytes * 20.0) / (sizeof(short) * iFrameSize);
-					Timer t;
 
 					if (didsleep)
 						safety *= 1.1f;
@@ -637,4 +631,15 @@ void DXAudioInput::run() {
 	if (FAILED(hr)) {
 		g.mw->msgBox(tr("Lost DirectSound input device."));
 	}
+
+cleanup:
+	if (! bOk) {
+		g.mw->msgBox(tr("Opening chosen DirectSound Input device failed. No microphone capture will be done."));
+	}
+	if (pDSNotify)
+		pDSNotify->Release();
+	if (pDSCaptureBuffer)
+		pDSCaptureBuffer->Release();
+	if (pDSCapture)
+		pDSCapture->Release();
 }
