@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -36,6 +36,7 @@
 #include "ConfigDialog.h"
 #include "Timer.h"
 #include "ui_GlobalShortcut.h"
+#include "ui_GlobalShortcutTarget.h"
 
 class GlobalShortcut : public QObject {
 		friend class GlobalShortcutEngine;
@@ -46,18 +47,17 @@ class GlobalShortcut : public QObject {
 	protected:
 		bool bActive;
 	signals:
-		void down();
-		void up();
-		void triggered(bool);
+		void down(QVariant);
+		void triggered(bool, QVariant);
 	public:
 		QString qsToolTip;
 		QString qsWhatsThis;
 		QString name;
-		QVariant data;
+		QVariant qvDefault;
 		bool bExpert;
 		int idx;
 
-		GlobalShortcut(QObject *parent, int index, QString qsName, bool expert = true);
+		GlobalShortcut(QObject *parent, int index, QString qsName, bool expert = true, QVariant def = QVariant());
 		~GlobalShortcut();
 
 		bool active() const {
@@ -80,25 +80,72 @@ class ShortcutKeyWidget : public QLineEdit {
 		bool bModified;
 		ShortcutKeyWidget(QWidget *p = NULL);
 		QList<QVariant> getShortcut() const;
+		void displayKeys(bool last = true);
 	public slots:
 		void updateKeys(bool last);
-		void displayKeys();
 		void setShortcut(const QList<QVariant> &shortcut);
 	signals:
-		void keySet(bool);
+		void keySet(bool, bool);
 };
 
 class ShortcutActionWidget : public QComboBox {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(ShortcutActionWidget)
-		Q_PROPERTY(int index READ index WRITE setIndex USER true)
+		Q_PROPERTY(unsigned int index READ index WRITE setIndex USER true)
 	public:
 		ShortcutActionWidget(QWidget *p = NULL);
+		unsigned int index() const;
+		void setIndex(int);
+};
+
+class ShortcutToggleWidget : public QComboBox {
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(ShortcutToggleWidget)
+		Q_PROPERTY(int index READ index WRITE setIndex USER true)
+	public:
+		ShortcutToggleWidget(QWidget *p = NULL);
 		int index() const;
 		void setIndex(int);
 };
 
+class ShortcutTargetDialog : public QDialog, public Ui::GlobalShortcutTarget {
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(ShortcutTargetDialog);
+	protected:
+		QMap<QString, QString> qmHashNames;
+		ShortcutTarget stTarget;
+	public:
+		ShortcutTargetDialog(const ShortcutTarget &, QWidget *p = NULL);
+		ShortcutTarget target() const;
+	public slots:
+		void accept();
+		void on_qrbUsers_clicked();
+		void on_qrbChannel_clicked();
+		void on_qpbAdd_clicked();
+		void on_qpbRemove_clicked();
+};
+
+class ShortcutTargetWidget : public QFrame {
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(ShortcutTargetWidget)
+		Q_PROPERTY(ShortcutTarget target READ target WRITE setTarget USER true)
+	protected:
+		ShortcutTarget stTarget;
+		QLineEdit *qleTarget;
+		QToolButton *qtbEdit;
+	public:
+		ShortcutTargetWidget(QWidget *p = NULL);
+		ShortcutTarget target() const;
+		void setTarget(const ShortcutTarget &);
+		static QString targetString(const ShortcutTarget &);
+	public slots:
+		void on_qtbEdit_clicked();
+};
+
 class ShortcutDelegate : public QStyledItemDelegate {
 		Q_OBJECT
 		Q_DISABLE_COPY(ShortcutDelegate)
@@ -127,6 +174,7 @@ class GlobalShortcutConfig : public ConfigWidget, public Ui::GlobalShortcut {
 		void load(const Settings &r);
 		void reload();
 		bool expert(bool);
+		void commit();
 		void on_qpbAdd_clicked(bool);
 		void on_qpbRemove_clicked(bool);
 		void on_qtwShortcuts_currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *);
