@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -246,6 +246,18 @@ const char *Log::colorClasses[] = {
 	"privilege"
 };
 
+const QStringList Log::allowedSchemes() {
+	QStringList qslAllowedSchemeNames;
+	qslAllowedSchemeNames << QLatin1String("mumble");
+	qslAllowedSchemeNames << QLatin1String("http");
+	qslAllowedSchemeNames << QLatin1String("https");
+	qslAllowedSchemeNames << QLatin1String("ftp");
+	qslAllowedSchemeNames << QLatin1String("clientid");
+	qslAllowedSchemeNames << QLatin1String("channelid");
+	qslAllowedSchemeNames << QLatin1String("spotify");
+	return qslAllowedSchemeNames;
+}
+
 QString Log::msgColor(const QString &text, LogColorType t) {
 	QString classname;
 
@@ -342,34 +354,33 @@ QString Log::imageToImg(QImage img) {
 	return QString();
 }
 
-QString Log::validHtml(const QString &html, bool allowReplacement) {
+QString Log::validHtml(const QString &html, bool allowReplacement, QTextCursor *tc) {
 	QDesktopWidget dw;
 	ValidDocument qtd(allowReplacement);
 	bool valid = false;
 
-	QStringList qslValid;
-	qslValid << QLatin1String("mumble");
-	qslValid << QLatin1String("http");
-	qslValid << QLatin1String("https");
-	qslValid << QLatin1String("ftp");
-	qslValid << QLatin1String("clientid");
-	qslValid << QLatin1String("channelid");
-	qslValid << QLatin1String("spotify");
-
 	QRectF qr = dw.availableGeometry(dw.screenNumber(g.mw));
 	qtd.setTextWidth(qr.width() / 2);
+	qtd.setDefaultStyleSheet(qApp->styleSheet());
 
 	qtd.setHtml(html);
 	valid = qtd.isValid();
 
+	QStringList qslAllowed = allowedSchemes();
 	for (QTextBlock qtb = qtd.begin(); qtb != qtd.end(); qtb = qtb.next()) {
 		for (QTextBlock::iterator qtbi = qtb.begin(); qtbi != qtb.end(); ++qtbi) {
 			const QTextFragment &qtf = qtbi.fragment();
 			QTextCharFormat qcf = qtf.charFormat();
 			if (! qcf.anchorHref().isEmpty()) {
 				QUrl url(qcf.anchorHref());
-				if (! url.isValid() || ! qslValid.contains(url.scheme()))
-					valid = false;
+				if (! url.isValid() || ! qslAllowed.contains(url.scheme())) {
+					QTextCharFormat qcfn = QTextCharFormat();
+					QTextCursor qtc(&qtd);
+					qtc.setPosition(qtf.position(), QTextCursor::MoveAnchor);
+					qtc.setPosition(qtf.position()+qtf.length(), QTextCursor::KeepAnchor);
+					qtc.setCharFormat(qcfn);
+					qtbi = qtb.begin();
+				}
 			}
 			if (qcf.isImageFormat()) {
 				QTextImageFormat qtif = qcf.toImageFormat();
@@ -388,12 +399,25 @@ QString Log::validHtml(const QString &html, bool allowReplacement) {
 		qtd.adjustSize();
 		s = qtd.size();
 
-		if ((s.width() > qr.width()) || (s.height() > qr.height()))
-			return tr("[[ Text object too large to display ]]");
-		return qtd.toHtml();
+		if ((s.width() > qr.width()) || (s.height() > qr.height())) {
+			QString errorMessage = tr("[[ Text object too large to display ]]");
+			if (tc) {
+				tc->insertText(errorMessage);
+				return QString();
+			} else {
+				return errorMessage;
+			}
+		}
 	}
 
-	return Qt::mightBeRichText(html) ? html : (QLatin1String("<span></span>")+html);
+	if (tc) {
+		QTextCursor tcNew(&qtd);
+		tcNew.movePosition(QTextCursor::End, QTextCursor::KeepAnchor);
+		tc->insertFragment(tcNew.selection());
+		return QString();
+	} else {
+		return qtd.toHtml();
+	}
 }
 
 void Log::log(MsgType mt, const QString &console, const QString &terse) {
@@ -406,7 +430,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		return;
 	}
 
-	const QString plain = QTextDocumentFragment::fromHtml(console).toPlainText();
+	QString plain = QTextDocumentFragment::fromHtml(console).toPlainText();
 
 	quint32 flags = g.s.qmMessages.value(mt);
 
@@ -431,7 +455,8 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		} else if (! g.mw->qteLog->document()->isEmpty()) {
 			tc.insertBlock();
 		}
-		tc.insertHtml(QString::fromLatin1("[%2] %1\n").arg(validHtml(console, true), Log::msgColor(dt.time().toString(Qt::DefaultLocaleShortDate), Log::Time)));
+		tc.insertHtml(Log::msgColor(QString::fromLatin1("[%1] ").arg(dt.time().toString(Qt::DefaultLocaleShortDate)), Log::Time));
+		validHtml(console, true, &tc);
 		tc.movePosition(QTextCursor::End);
 		g.mw->qteLog->setTextCursor(tc);
 		g.mw->qteLog->ensureCursorVisible();
@@ -530,7 +555,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		QString sSound = g.s.qmMessageSounds.value(mt);
 		AudioOutputPtr ao = g.ao;
 		if (!ao || !ao->playSample(sSound, false)) {
-			qWarning() << "Sound file" << sSound << "is not a valid speex file, fallback to TTS.";
+			qWarning() << "Sound file" << sSound << "is not a valid audio file, fallback to TTS.";
 			flags ^= Settings::LogSoundfile | Settings::LogTTS; // Fallback to TTS
 		}
 	}
@@ -539,7 +564,34 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 	if (! g.s.bTTS || !(flags & Settings::LogTTS))
 		return;
 
-	/* TTS threshold limiter. */
+	// Apply simplifications to spoken text
+	QRegExp identifyURL(QLatin1String("\\b(([\\w-]+://?|www[.])[^\\s()<>]+(?:\\([\\w\\d]+\\)|([^!\"#$%&'()*+,-./:;<=>?@\\[\\\\\\]^_`{|}~\\s]|/)))"));
+	QStringList qslAllowed = allowedSchemes();
+	int pos = 0;
+	while ((pos = identifyURL.indexIn(plain, pos)) != -1) {
+		QUrl url(identifyURL.cap(0));
+		int len = identifyURL.matchedLength();
+		if (url.isValid() && qslAllowed.contains(url.scheme())) {
+			// Replace it appropriatly
+			QString replacement;
+			if (url.scheme() == QLatin1String("http") || url.scheme() == QLatin1String("https"))
+				replacement = tr("link to %1").arg(url.host());
+			else if (url.scheme() == QLatin1String("ftp"))
+				replacement = tr("ftp link to %1").arg(url.host());
+			else if (url.scheme() == QLatin1String("clientid"))
+				replacement = tr("player link");
+			else if (url.scheme() == QLatin1String("channelid"))
+				replacement = tr("channel link");
+			else
+				replacement = tr("%1 link").arg(url.scheme());
+
+			plain.replace(pos, len, replacement);
+		} else {
+			pos += len;
+		}
+	}
+
+	// TTS threshold limiter.
 	if (plain.length() <= g.s.iTTSThreshold)
 		tts->say(plain);
 	else if ((! terse.isEmpty()) && (terse.length() <= g.s.iTTSThreshold))
