@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -80,12 +80,13 @@ class Log : public QObject {
 		TextToSpeech *tts;
 		unsigned int uiLastId;
 		QDate qdDate;
+		static const QStringList allowedSchemes();
 	public:
 		Log(QObject *p = NULL);
 		QString msgName(MsgType t) const;
 		void setIgnore(MsgType t, int ignore = 1 << 30);
 		void clearIgnore();
-		static QString validHtml(const QString &html, bool allowReplacement = false);
+		static QString validHtml(const QString &html, bool allowReplacement = false, QTextCursor *tc = NULL);
 		static QString imageToImg(const QByteArray &format, const QByteArray &image);
 		static QString imageToImg(QImage img);
 		static QString msgColor(const QString &text, LogColorType t);
