@@ -56,6 +56,7 @@
 #include "TextMessage.h"
 #include "NetworkConfig.h"
 #include "ACL.h"
+#include "UserInformation.h"
 
 #ifdef Q_OS_WIN
 #include "TaskList.h"
@@ -152,18 +153,24 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	tokenEdit = NULL;
 	bNoHide = false;
 
-	cmUid = -1;
-	cmCid = -1;
+	uiContextSession = ~0;
+	iContextChannel = -1;
 
 	qtReconnect = new QTimer(this);
 	qtReconnect->setInterval(10000);
 	qtReconnect->setSingleShot(true);
 	qtReconnect->setObjectName(QLatin1String("Reconnect"));
 
+	qmUser = new QMenu(tr("&User"), this);
+	qmChannel = new QMenu(tr("&Channel"), this);
+
 	createActions();
 	setupUi(this);
 	setupGui();
 
+	connect(qmUser, SIGNAL(aboutToShow()), this, SLOT(qmUser_aboutToShow()));
+	connect(qmChannel, SIGNAL(aboutToShow()), this, SLOT(qmChannel_aboutToShow()));
+
 	connect(g.sh, SIGNAL(connected()), this, SLOT(serverConnected()));
 	connect(g.sh, SIGNAL(disconnected(QAbstractSocket::SocketError, QString)), this, SLOT(serverDisconnected(QAbstractSocket::SocketError, QString)));
 
@@ -173,8 +180,9 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 		a->setShortcutContext(Qt::ApplicationShortcut);
 
 	on_qmServer_aboutToShow();
-	on_qmChannel_aboutToShow();
-	on_qmUser_aboutToShow();
+	on_qmSelf_aboutToShow();
+	qmChannel_aboutToShow();
+	qmUser_aboutToShow();
 	on_qmConfig_aboutToShow();
 
 	setOnTop(g.s.aotbAlwaysOnTop == Settings::OnTopAlways ||
@@ -308,13 +316,17 @@ void MainWindow::setupGui()  {
 		restoreGeometry(g.s.qbaMainWindowGeometry);
 
 	Settings::WindowLayout wlTmp = g.s.wlWindowLayout;
-	restoreState(g.s.qbaMainWindowState);
+	if (g.s.bMinimalView && ! g.s.qbaMinimalViewState.isNull())
+		restoreState(g.s.qbaMinimalViewState);
+	else if (! g.s.bMinimalView && ! g.s.qbaMainWindowState.isNull())
+		restoreState(g.s.qbaMainWindowState);
 	g.s.wlWindowLayout = wlTmp;
 
 	setupView(false);
 
 	qmTray = new QMenu(this);
-	setupIconMenu(false);
+	connect(qmTray, SIGNAL(aboutToShow()), this, SLOT(trayAboutToShow()));
+	trayAboutToShow();
 	qstiIcon->setContextMenu(qmTray);
 
 	updateTrayIcon();
@@ -343,7 +355,7 @@ bool MainWindow::winEvent(MSG *msg, long *) {
 void MainWindow::closeEvent(QCloseEvent *e) {
 #ifndef Q_OS_MAC
 	if (g.sh && g.sh->isRunning() && g.s.bAskOnQuit && !bSuppressAskOnQuit) {
-		QMessageBox mb(QMessageBox::Warning, tr("Mumble"), tr("Mumble is currently connected to a server. Do you want to Close or Minimize it?"), QMessageBox::NoButton, this);
+		QMessageBox mb(QMessageBox::Warning, QLatin1String("Mumble"), tr("Mumble is currently connected to a server. Do you want to Close or Minimize it?"), QMessageBox::NoButton, this);
 		QPushButton *qpbClose = mb.addButton(tr("Close"), QMessageBox::YesRole);
 		QPushButton *qpbMinimize = mb.addButton(tr("Minimize"), QMessageBox::NoRole);
 		mb.setDefaultButton(qpbMinimize);
@@ -358,11 +370,12 @@ void MainWindow::closeEvent(QCloseEvent *e) {
 	g.uiSession = 0;
 	g.pPermissions = ChanACL::None;
 
-	g.s.qbaMainWindowState = saveState();
 	if (g.s.bMinimalView) {
 		g.s.qbaMinimalViewGeometry = saveGeometry();
+		g.s.qbaMinimalViewState = saveState();
 	} else {
 		g.s.qbaMainWindowGeometry = saveGeometry();
+		g.s.qbaMainWindowState = saveState();
 		g.s.qbaHeaderState = qtvUsers->header()->saveState();
 	}
 	QMainWindow::closeEvent(e);
@@ -415,15 +428,15 @@ void MainWindow::updateTrayIcon() {
 }
 
 Channel *MainWindow::getContextMenuChannel() {
-	if (cmCid >= 0)
-		return Channel::get(cmCid);
+	if (iContextChannel >= 0)
+		return Channel::get(iContextChannel);
 
 	return pmModel->getChannel(qtvUsers->currentIndex());
 }
 
 ClientUser *MainWindow::getContextMenuUser() {
-	if (cmUid >= 0)
-		return ClientUser::get(cmUid);
+	if (uiContextSession != ~0)
+		return ClientUser::get(uiContextSession);
 
 	return pmModel->getUser(qtvUsers->currentIndex());
 }
@@ -435,33 +448,37 @@ bool MainWindow::handleSpecialContextMenu(const QUrl &url, const QPoint &_pos, b
 		if (x.length() == 40) {
 			ClientUser *cu = ClientUser::getByHash(x);
 			if (cu) {
-				cmUid = ClientUser::getUiSession(cu);
+				uiContextSession = cu->uiSession;
 				ok = true;
 			}
 		} else {
 			QByteArray qbaServerDigest = QByteArray::fromBase64(url.path().remove(0, 1).toLatin1());
-			cmUid = url.host().toInt(&ok, 10);
+			uiContextSession = url.host().toInt(&ok, 10);
 			ok = ok && (qbaServerDigest == g.sh->qbaDigest);
 		}
-		if (ok && ClientUser::isValid(cmUid)) {
-			if (focus)
-				qtvUsers->setCurrentIndex(pmModel->index(ClientUser::get(cmUid)));
-			else
+		if (ok && ClientUser::isValid(uiContextSession)) {
+			if (focus) {
+				qtvUsers->setCurrentIndex(pmModel->index(ClientUser::get(uiContextSession)));
+				qleChat->setFocus();
+			} else {
 				qmUser->exec(_pos, NULL);
+			}
 		}
-		cmUid = -1;
+		uiContextSession = ~0;
 	} else if (url.scheme() == QString::fromLatin1("channelid")) {
 		bool ok;
 		QByteArray qbaServerDigest = QByteArray::fromBase64(url.path().remove(0, 1).toLatin1());
-		cmCid = url.host().toInt(&ok, 10);
+		iContextChannel = url.host().toInt(&ok, 10);
 		ok = ok && (qbaServerDigest == g.sh->qbaDigest);
 		if (ok) {
-			if (focus)
-				qtvUsers->setCurrentIndex(pmModel->index(Channel::get(cmCid)));
-			else
+			if (focus) {
+				qtvUsers->setCurrentIndex(pmModel->index(Channel::get(iContextChannel)));
+				qleChat->setFocus();
+			} else {
 				qmChannel->exec(_pos, NULL);
+			}
 		}
-		cmCid = -1;
+		iContextChannel = -1;
 	} else {
 		return false;
 	}
@@ -539,7 +556,7 @@ void MainWindow::openUrl(const QUrl &url) {
 		return;
 	} else
 #endif
-		if ((major != 1) || (minor != 2) || (patch > 1)) {
+		if ((major != 1) || (minor != 2) || (patch > 2)) {
 			g.l->log(Log::Warning, tr("This version of Mumble can't handle URLs for Mumble version %1.%2.%3").arg(major).arg(minor).arg(patch));
 			return;
 		}
@@ -549,6 +566,9 @@ void MainWindow::openUrl(const QUrl &url) {
 	QString user = url.userName();
 	QString pw = url.password();
 	qsDesiredChannel = url.path();
+	QString name;
+	if (url.hasQueryItem(QLatin1String("title")))
+		name = url.queryItemValue(QLatin1String("title"));
 
 	if (g.sh && g.sh->isRunning()) {
 		QString oHost, oUser, oPw;
@@ -562,7 +582,7 @@ void MainWindow::openUrl(const QUrl &url) {
 		}
 	}
 
-	Database::fuzzyMatch(user, pw, host, port);
+	Database::fuzzyMatch(name, user, pw, host, port);
 
 	if (user.isEmpty()) {
 		bool ok;
@@ -572,12 +592,16 @@ void MainWindow::openUrl(const QUrl &url) {
 		g.s.qsUsername = user;
 	}
 
+	if (name.isEmpty())
+		name = QString::fromLatin1("%1@%2").arg(user).arg(host);
+
 	if (g.sh && g.sh->isRunning()) {
 		on_qaServerDisconnect_triggered();
 		g.sh->wait();
 		QCoreApplication::instance()->processEvents();
 	}
 
+	g.s.qsLastServer = name;
 	rtLast = MumbleProto::Reject_RejectType_None;
 	qaServerDisconnect->setEnabled(true);
 	g.l->log(Log::Information, tr("Connecting to server %1.").arg(Log::msgColor(host, Log::Server)));
@@ -609,10 +633,7 @@ void MainWindow::findDesiredChannel() {
 	}
 	if (found) {
 		if (chan != ClientUser::get(g.uiSession)->cChannel) {
-			MumbleProto::UserState mpus;
-			mpus.set_session(g.uiSession);
-			mpus.set_channel_id(chan->iId);
-			g.sh->sendMessage(mpus);
+			g.sh->joinChannel(chan->iId);
 		}
 		qtvUsers->setCurrentIndex(pmModel->index(chan));
 	} else if (g.uiSession) {
@@ -648,20 +669,22 @@ void MainWindow::setupView(bool toggle_minimize) {
 			addDockWidget(Qt::LeftDockWidgetArea, qdwLog);
 			qdwLog->show();
 			splitDockWidget(qdwLog, qdwChat, Qt::Vertical);
+			qdwChat->show();
 			break;
 		case Settings::LayoutStacked:
 			removeDockWidget(qdwLog);
 			addDockWidget(Qt::BottomDockWidgetArea, qdwLog);
 			qdwLog->show();
 			splitDockWidget(qdwLog, qdwChat, Qt::Vertical);
+			qdwChat->show();
 			break;
 		case Settings::LayoutHybrid:
 			removeDockWidget(qdwLog);
+			removeDockWidget(qdwChat);
 			addDockWidget(Qt::LeftDockWidgetArea, qdwLog);
 			qdwLog->show();
-			removeDockWidget(qdwChat);
 			addDockWidget(Qt::BottomDockWidgetArea, qdwChat);
-			if (g.s.bShowChatbar) qdwChat->show();
+			qdwChat->show();
 			break;
 		default:
 			wlTmp = Settings::LayoutCustom;
@@ -674,9 +697,11 @@ void MainWindow::setupView(bool toggle_minimize) {
 	if (toggle_minimize) {
 		if (! showit) {
 			g.s.qbaMainWindowGeometry = saveGeometry();
+			g.s.qbaMainWindowState = saveState();
 			g.s.qbaHeaderState = qtvUsers->header()->saveState();
 		} else {
 			g.s.qbaMinimalViewGeometry = saveGeometry();
+			g.s.qbaMinimalViewState = saveState();
 		}
 	}
 
@@ -686,6 +711,8 @@ void MainWindow::setupView(bool toggle_minimize) {
 #ifndef Q_OS_MAC
 	else if (!showit)
 		f = Qt::Tool;
+#else
+	f |= Qt::MacWindowToolBarButtonHint | Qt::WindowTitleHint | Qt::WindowSystemMenuHint | Qt::WindowMinMaxButtonsHint | Qt::WindowCloseButtonHint;
 #endif
 
 	if (g.s.aotbAlwaysOnTop == Settings::OnTopAlways ||
@@ -695,15 +722,43 @@ void MainWindow::setupView(bool toggle_minimize) {
 
 	setWindowFlags(f);
 
-	qdwLog->setVisible(showit);
-	qdwChat->setVisible(showit && g.s.bShowChatbar);
+	if (g.s.bShowContextMenuInMenuBar) {
+		bool found = false;
+		foreach(QAction *a, menuBar()->actions()) {
+			if (a == qmUser->menuAction()) {
+				found = true;
+				break;
+			}
+		}
+
+		if (!found) {
+			menuBar()->insertMenu(qmConfig->menuAction(), qmUser);
+			menuBar()->insertMenu(qmConfig->menuAction(), qmChannel);
+		}
+	} else {
+		menuBar()->removeAction(qmUser->menuAction());
+		menuBar()->removeAction(qmChannel->menuAction());
+	}
+
+	if (! showit) {
+		qdwLog->setVisible(showit);
+		qdwChat->setVisible(showit);
+		qtIconToolbar->setVisible(showit);
+	}
 	menuBar()->setVisible(showit);
 
 	if (toggle_minimize) {
-		if (! showit && ! g.s.qbaMinimalViewGeometry.isNull())
-			restoreGeometry(g.s.qbaMinimalViewGeometry);
-		else if (showit && ! g.s.qbaMainWindowGeometry.isNull())
-			restoreGeometry(g.s.qbaMainWindowGeometry);
+		if (! showit) {
+			if (! g.s.qbaMinimalViewGeometry.isNull())
+				restoreGeometry(g.s.qbaMinimalViewGeometry);
+			if (! g.s.qbaMinimalViewState.isNull())
+				restoreState(g.s.qbaMinimalViewState);
+		} else {
+			if (! g.s.qbaMainWindowGeometry.isNull())
+				restoreGeometry(g.s.qbaMainWindowGeometry);
+			if (! g.s.qbaMainWindowState.isNull())
+				restoreState(g.s.qbaMainWindowState);
+		}
 	} else {
 		QRect newgeom = frameGeometry();
 		resize(geometry().width()-newgeom.width()+geom.width(),
@@ -714,7 +769,6 @@ void MainWindow::setupView(bool toggle_minimize) {
 	show();
 	activateWindow();
 	bNoHide = false;
-
 }
 
 void MainWindow::on_qaServerConnect_triggered() {
@@ -748,6 +802,65 @@ void MainWindow::on_Reconnect_timeout() {
 	g.sh->start(QThread::TimeCriticalPriority);
 }
 
+void MainWindow::on_qmSelf_aboutToShow() {
+	qaServerTexture->setEnabled(g.uiSession != 0);
+
+	ClientUser *user = ClientUser::get(g.uiSession);
+	qaServerTextureRemove->setEnabled(user && ! user->qbaTextureHash.isEmpty());
+
+	qaSelfRegister->setEnabled(user && user->iId < 0 && (g.pPermissions & (ChanACL::SelfRegister | ChanACL::Write)));
+}
+
+void MainWindow::on_qaSelfComment_triggered() {
+	ClientUser *p = ClientUser::get(g.uiSession);
+	if (!p)
+		return;
+
+	if (! p->qbaCommentHash.isEmpty() && p->qsComment.isEmpty()) {
+		p->qsComment = QString::fromUtf8(Database::blob(p->qbaCommentHash));
+		if (p->qsComment.isEmpty()) {
+			pmModel->uiSessionComment = ~(p->uiSession);
+			MumbleProto::RequestBlob mprb;
+			mprb.add_session_comment(p->uiSession);
+			g.sh->sendMessage(mprb);
+			return;
+		}
+	}
+
+	unsigned int session = p->uiSession;
+
+	::TextMessage *texm = new ::TextMessage(this, tr("Change your comment"));
+
+	texm->rteMessage->setText(p->qsComment);
+	int res = texm->exec();
+
+	p = ClientUser::get(session);
+
+	if (p && (res == QDialog::Accepted)) {
+		const QString &msg = texm->message();
+		MumbleProto::UserState mpus;
+		mpus.set_session(session);
+		mpus.set_comment(u8(msg));
+		g.sh->sendMessage(mpus);
+
+		if (! msg.isEmpty())
+			Database::setBlob(sha1(msg), msg.toUtf8());
+	}
+	delete texm;
+}
+
+void MainWindow::on_qaSelfRegister_triggered() {
+	ClientUser *p = ClientUser::get(g.uiSession);
+	if (!p)
+		return;
+
+	QMessageBox::StandardButton result;
+	result = QMessageBox::question(this, tr("Register yourself as %1").arg(p->qsName), tr("<p>You are about to register yourself on this server. This action cannot be undone, and your username cannot be changed once this is done. You will forever be known as '%1' on this server.</p><p>Are you sure you want to register yourself?</p>").arg(p->qsName), QMessageBox::Yes|QMessageBox::No);
+
+	if (result == QMessageBox::Yes)
+		g.sh->registerUser(p->uiSession);
+}
+
 void MainWindow::on_qmServer_aboutToShow() {
 	qmServer->clear();
 	qmServer->addAction(qaServerConnect);
@@ -755,8 +868,6 @@ void MainWindow::on_qmServer_aboutToShow() {
 	qmServer->addAction(qaServerBanList);
 	qmServer->addAction(qaServerUserList);
 	qmServer->addAction(qaServerInformation);
-	qmServer->addAction(qaServerTexture);
-	qmServer->addAction(qaServerTextureRemove);
 	qmServer->addAction(qaServerTokens);
 	qmServer->addSeparator();
 	qmServer->addAction(qaQuit);
@@ -764,12 +875,8 @@ void MainWindow::on_qmServer_aboutToShow() {
 	qaServerBanList->setEnabled(g.pPermissions & (ChanACL::Ban | ChanACL::Write));
 	qaServerUserList->setEnabled(g.pPermissions & (ChanACL::Register | ChanACL::Write));
 	qaServerInformation->setEnabled(g.uiSession != 0);
-	qaServerTexture->setEnabled(g.uiSession != 0);
 	qaServerTokens->setEnabled(g.uiSession != 0);
 
-	ClientUser *user = ClientUser::get(g.uiSession);
-	qaServerTextureRemove->setEnabled(user && ! user->qbaTexture.isEmpty());
-
 	if (! qlServerActions.isEmpty()) {
 		qmServer->addSeparator();
 		foreach(QAction *a, qlServerActions)
@@ -785,9 +892,7 @@ void MainWindow::on_qaServerDisconnect_triggered() {
 }
 
 void MainWindow::on_qaServerBanList_triggered() {
-	MumbleProto::BanList mpbl;
-	mpbl.set_query(true);
-	g.sh->sendMessage(mpbl);
+	g.sh->requestBanList();
 
 	if (banEdit) {
 		banEdit->reject();
@@ -797,8 +902,7 @@ void MainWindow::on_qaServerBanList_triggered() {
 }
 
 void MainWindow::on_qaServerUserList_triggered() {
-	MumbleProto::UserList mpul;
-	g.sh->sendMessage(mpul);
+	g.sh->requestUserList();
 
 	if (userEdit) {
 		userEdit->reject();
@@ -814,11 +918,9 @@ void MainWindow::on_qaServerInformation_triggered() {
 		return;
 
 	CryptState &cs = c->csCrypt;
-
 	QSslCipher qsc = g.sh->qscCipher;
 
 	unsigned int version = g.sh->uiVersion;
-
 	QString qsVersion=tr("<h2>Version</h2><p>Protocol %1.%2.%3.</p>").arg(QString::number((version >> 16) & 0xFF),
 	                  QString::number((version >> 8) & 0xFF),
 	                  QString::number(version & 0xFF));
@@ -830,10 +932,17 @@ void MainWindow::on_qaServerInformation_triggered() {
 		                 .arg(g.sh->qsRelease, g.sh->qsOS, g.sh->qsOSVersion));
 	}
 
-	QString qsControl=tr("<h2>Control channel</h2><p>Encrypted with %1 bit %2<br />%3 ms average latency (%4 deviation)</p>").arg(QString::number(qsc.usedBits()),
+	QString host, uname, pw;
+	unsigned short port;
+
+	g.sh->getConnectionInfo(host,port,uname,pw);
+
+	QString qsControl=tr("<h2>Control channel</h2><p>Encrypted with %1 bit %2<br />%3 ms average latency (%4 deviation)</p><p>Remote host %5 (port %6)</p>").arg(QString::number(qsc.usedBits()),
 	                  qsc.name(),
 	                  QString::fromLatin1("%1").arg(boost::accumulators::mean(g.sh->accTCP), 0, 'f', 2),
-	                  QString::fromLatin1("%1").arg(sqrt(boost::accumulators::variance(g.sh->accTCP)),0,'f',2));
+	                  QString::fromLatin1("%1").arg(sqrt(boost::accumulators::variance(g.sh->accTCP)),0,'f',2),
+	                  host,
+	                  QString::number(port));
 	QString qsVoice, qsCrypt, qsAudio;
 
 	if (NetworkConfig::TcpModeEnabled()) {
@@ -869,30 +978,14 @@ void MainWindow::on_qaServerTexture_triggered() {
 	if (choice.first.isEmpty())
 		return;
 
-	QImage &img = choice.second;
-	img = img.convertToFormat(QImage::Format_ARGB32);
-	if ((img.height() != 60) || (img.width() != 600)) {
-		QImage final(600, 60, QImage::Format_ARGB32);
-		img = img.scaled(600, 60, Qt::KeepAspectRatio, Qt::SmoothTransformation);
-		qWarning() << img.width() << img.height();
-		final.fill(0);
-		{
-			QPainter p(&final);
-			p.drawImage(0, 0, img);
-		}
-		img = final;
-	}
-	QByteArray qba(reinterpret_cast<const char *>(img.bits()), img.numBytes());
-	qba = qCompress(qba);
-	MumbleProto::UserState mpus;
-	mpus.set_texture(qba.constData(), qba.length());
-	g.sh->sendMessage(mpus);
+	const QImage &img = choice.second;
+
+	if ((img.height() <= 1024) && (img.width() <= 1024))
+		g.sh->setTexture(choice.first);
 }
 
 void MainWindow::on_qaServerTextureRemove_triggered() {
-	MumbleProto::UserState mpus;
-	mpus.set_texture(NULL, 0);
-	g.sh->sendMessage(mpus);
+	g.sh->setTexture(QByteArray());
 }
 
 void MainWindow::on_qaServerTokens_triggered() {
@@ -906,7 +999,7 @@ void MainWindow::on_qaServerTokens_triggered() {
 	tokenEdit->show();
 }
 
-void MainWindow::on_qmUser_aboutToShow() {
+void MainWindow::qmUser_aboutToShow() {
 	ClientUser *p = getContextMenuUser();
 
 	bool self = p && (p->uiSession == g.uiSession);
@@ -922,13 +1015,15 @@ void MainWindow::on_qmUser_aboutToShow() {
 	qmUser->addAction(qaUserLocalMute);
 
 	if (self)
-		qmUser->addAction(qaUserComment);
+		qmUser->addAction(qaSelfComment);
 	else {
 		qmUser->addAction(qaUserCommentView);
 		qmUser->addAction(qaUserCommentReset);
 	}
 
 	qmUser->addAction(qaUserTextMessage);
+	if (g.sh->uiVersion >= 0x010202)
+		qmUser->addAction(qaUserInformation);
 
 	if (p && (p->iId < 0) && (g.pPermissions & ((self ? ChanACL::SelfRegister : ChanACL::Register) | ChanACL::Write))) {
 		qmUser->addSeparator();
@@ -955,7 +1050,7 @@ void MainWindow::on_qmUser_aboutToShow() {
 	if (g.s.bMinimalView) {
 		qmUser->addSeparator();
 		qmUser->addMenu(qmServer);
-		qmUser->addMenu(qmAudio);
+		qmUser->addMenu(qmSelf);
 		qmUser->addMenu(qmConfig);
 		qmUser->addMenu(qmHelp);
 	}
@@ -969,23 +1064,17 @@ void MainWindow::on_qmUser_aboutToShow() {
 	if (! p) {
 		qaUserKick->setEnabled(false);
 		qaUserBan->setEnabled(false);
-		qaUserMute->setEnabled(false);
-		qaUserDeaf->setEnabled(false);
 		qaUserTextMessage->setEnabled(false);
 		qaUserLocalMute->setEnabled(false);
-		qaUserComment->setEnabled(false);
 		qaUserCommentReset->setEnabled(false);
 		qaUserCommentView->setEnabled(false);
 	} else {
 		qaUserKick->setEnabled(! self);
 		qaUserBan->setEnabled(! self);
-		qaUserMute->setEnabled(! self || p->bMute || p->bSuppress);
-		qaUserDeaf->setEnabled(! self || p->bDeaf);
 		qaUserTextMessage->setEnabled(true);
 		qaUserLocalMute->setEnabled(! self);
-		qaUserComment->setEnabled(self);
-		qaUserCommentReset->setEnabled(!p->qsComment.isEmpty() && (g.pPermissions & (ChanACL::Move | ChanACL::Write)));
-		qaUserCommentView->setEnabled(! p->qsComment.isEmpty());
+		qaUserCommentReset->setEnabled(! p->qbaCommentHash.isEmpty() && (g.pPermissions & (ChanACL::Move | ChanACL::Write)));
+		qaUserCommentView->setEnabled(! p->qbaCommentHash.isEmpty());
 
 		qaUserMute->setChecked(p->bMute || p->bSuppress);
 		qaUserDeaf->setChecked(p->bDeaf);
@@ -1053,10 +1142,7 @@ void MainWindow::on_qaUserRegister_triggered() {
 		p = ClientUser::get(session);
 		if (! p)
 			return;
-		MumbleProto::UserState mpus;
-		mpus.set_session(p->uiSession);
-		mpus.set_user_id(0);
-		g.sh->sendMessage(mpus);
+		g.sh->registerUser(p->uiSession);
 	}
 }
 
@@ -1097,10 +1183,7 @@ void MainWindow::on_qaUserKick_triggered() {
 		return;
 
 	if (ok) {
-		MumbleProto::UserRemove mpur;
-		mpur.set_session(p->uiSession);
-		mpur.set_reason(u8(reason));
-		g.sh->sendMessage(mpur);
+		g.sh->kickBanUser(p->uiSession, reason, false);
 	}
 }
 
@@ -1118,11 +1201,7 @@ void MainWindow::on_qaUserBan_triggered() {
 		return;
 
 	if (ok) {
-		MumbleProto::UserRemove mpur;
-		mpur.set_session(p->uiSession);
-		mpur.set_reason(u8(reason));
-		mpur.set_ban(true);
-		g.sh->sendMessage(mpur);
+		g.sh->kickBanUser(p->uiSession, reason, true);
 	}
 }
 
@@ -1143,46 +1222,30 @@ void MainWindow::on_qaUserTextMessage_triggered() {
 		QString msg = texm->message();
 
 		if (! msg.isEmpty()) {
-			MumbleProto::TextMessage mptm;
-			mptm.add_session(p->uiSession);
-			mptm.set_message(u8(msg));
-			g.sh->sendMessage(mptm);
+			g.sh->sendUserTextMessage(p->uiSession, msg);
 			g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::msgColor(p->qsName, Log::Target)).arg(texm->message()), tr("Message to %1").arg(p->qsName));
 		}
 	}
 	delete texm;
 }
 
-void MainWindow::on_qaUserComment_triggered() {
-	ClientUser *p = getContextMenuUser();
-
-	if (!p)
-		return;
-
-	unsigned int session = p->uiSession;
-
-	::TextMessage *texm = new ::TextMessage(this, tr("Change comment on user %1").arg(p->qsName));
-
-	texm->rteMessage->setText(p->qsComment);
-	int res = texm->exec();
-
-	p = ClientUser::get(session);
-
-	if (p && (res==QDialog::Accepted)) {
-		MumbleProto::UserState mpus;
-		mpus.set_session(session);
-		mpus.set_comment(u8(texm->message()));
-		g.sh->sendMessage(mpus);
-	}
-	delete texm;
-}
-
 void MainWindow::on_qaUserCommentView_triggered() {
 	ClientUser *p = getContextMenuUser();
 
 	if (!p)
 		return;
 
+	if (! p->qbaCommentHash.isEmpty() && p->qsComment.isEmpty()) {
+		p->qsComment = QString::fromUtf8(Database::blob(p->qbaCommentHash));
+		if (p->qsComment.isEmpty()) {
+			pmModel->uiSessionComment = ~(p->uiSession);
+			MumbleProto::RequestBlob mprb;
+			mprb.add_session_comment(p->uiSession);
+			g.sh->sendMessage(mprb);
+			return;
+		}
+	}
+
 	::TextMessage *texm = new ::TextMessage(this, tr("View comment on user %1").arg(p->qsName));
 
 	texm->rteMessage->setText(p->qsComment, true);
@@ -1198,17 +1261,23 @@ void MainWindow::on_qaUserCommentReset_triggered() {
 
 	unsigned int session = p->uiSession;
 
-	int ret = QMessageBox::question(this, tr("Mumble"),
+	int ret = QMessageBox::question(this, QLatin1String("Mumble"),
 	                                tr("Are you sure you want to reset the comment of user %1?").arg(p->qsName),
 	                                QMessageBox::Yes, QMessageBox::No);
 	if (ret == QMessageBox::Yes) {
-		MumbleProto::UserState mpus;
-		mpus.set_session(session);
-		mpus.set_comment(std::string());
-		g.sh->sendMessage(mpus);
+		g.sh->setUserComment(session, QString());
 	}
 }
 
+void MainWindow::on_qaUserInformation_triggered() {
+	ClientUser *p = getContextMenuUser();
+
+	if (!p)
+		return;
+
+	g.sh->requestUserStats(p->uiSession, false);
+}
+
 void MainWindow::on_qaQuit_triggered() {
 	bSuppressAskOnQuit = true;
 	this->close();
@@ -1220,7 +1289,6 @@ void MainWindow::on_qleChat_returnPressed() {
 	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
 	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
 
-	MumbleProto::TextMessage mptm;
 	QString qsText;
 
 	qsText = qleChat->text();
@@ -1228,22 +1296,19 @@ void MainWindow::on_qleChat_returnPressed() {
 		qsText = TextMessage::autoFormat(qsText);
 	}
 
-	mptm.set_message(u8(qsText));
-
 	if (p == NULL || p->uiSession == g.uiSession) {
-		// Channel tree message
+		// Channel message
 		if (c == NULL) // If no channel selected fallback to current one
 			c = ClientUser::get(g.uiSession)->cChannel;
 
-		mptm.add_channel_id(c->iId);
+		g.sh->sendChannelTextMessage(c->iId, qsText, false);
 		g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::formatChannel(c)).arg(qsText), tr("Message to channel %1").arg(c->qsName));
 	} else {
 		// User message
-		mptm.add_session(p->uiSession);
+		g.sh->sendUserTextMessage(p->uiSession, qsText);
 		g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::formatClientUser(p, Log::Target)).arg(qsText), tr("Message to %1").arg(p->qsName));
 	}
 
-	g.sh->sendMessage(mptm);
 	qleChat->clear();
 }
 
@@ -1265,17 +1330,22 @@ void MainWindow::on_qmConfig_aboutToShow() {
 	qmConfig->addAction(qaAudioWizard);
 	qmConfig->addAction(qaConfigCert);
 	qmConfig->addSeparator();
+	qmConfig->addAction(qaAudioTTS);
+	qmConfig->addSeparator();
 	qmConfig->addAction(qaConfigMinimal);
 	if (g.s.bMinimalView)
 		qmConfig->addAction(qaConfigHideFrame);
 }
 
-void MainWindow::on_qmChannel_aboutToShow() {
+void MainWindow::qmChannel_aboutToShow() {
 	qmChannel->clear();
 
-	if (cmCid >= 0 && g.uiSession != 0 && cmCid != ClientUser::get(g.uiSession)->cChannel->iId) {
-		qmChannel->addAction(qaChannelJoin);
-		qmChannel->addSeparator();
+	if (g.uiSession != 0) {
+		Channel *c = getContextMenuChannel();
+		if (c && c->iId != ClientUser::get(g.uiSession)->cChannel->iId) {
+			qmChannel->addAction(qaChannelJoin);
+			qmChannel->addSeparator();
+		}
 	}
 	qmChannel->addAction(qaChannelAdd);
 	qmChannel->addAction(qaChannelACL);
@@ -1290,7 +1360,7 @@ void MainWindow::on_qmChannel_aboutToShow() {
 	if (g.s.bMinimalView) {
 		qmChannel->addSeparator();
 		qmChannel->addMenu(qmServer);
-		qmChannel->addMenu(qmAudio);
+		qmChannel->addMenu(qmSelf);
 		qmChannel->addMenu(qmConfig);
 		qmChannel->addMenu(qmHelp);
 	}
@@ -1346,10 +1416,7 @@ void MainWindow::on_qaChannelJoin_triggered() {
 	Channel *c = getContextMenuChannel();
 
 	if (c) {
-		MumbleProto::UserState mpus;
-		mpus.set_session(g.uiSession);
-		mpus.set_channel_id(c->iId);
-		g.sh->sendMessage(mpus);
+		g.sh->joinChannel(c->iId);
 	}
 }
 
@@ -1378,28 +1445,33 @@ void MainWindow::on_qaChannelRemove_triggered() {
 
 	int id = c->iId;
 
-	ret=QMessageBox::question(this, tr("Mumble"), tr("Are you sure you want to delete %1 and all its sub-channels?").arg(c->qsName), QMessageBox::Yes, QMessageBox::No);
+	ret=QMessageBox::question(this, QLatin1String("Mumble"), tr("Are you sure you want to delete %1 and all its sub-channels?").arg(c->qsName), QMessageBox::Yes, QMessageBox::No);
 
 	c = Channel::get(id);
 	if (!c)
 		return;
 
 	if (ret == QMessageBox::Yes) {
-		MumbleProto::ChannelRemove mpcr;
-		mpcr.set_channel_id(c->iId);
-		g.sh->sendMessage(mpcr);
+		g.sh->removeChannel(c->iId);
 	}
 }
 
 void MainWindow::on_qaChannelACL_triggered() {
 	Channel *c = getContextMenuChannel();
-	int id = c ? c->iId : 0;
+	if (! c)
+		c = Channel::get(0);
+	int id = c->iId;
 
-	MumbleProto::ACL mpacl;
-	mpacl.set_channel_id(id);
-	mpacl.set_query(true);
+	if (! c->qbaDescHash.isEmpty() && c->qsDesc.isEmpty()) {
+		c->qsDesc = QString::fromUtf8(Database::blob(c->qbaDescHash));
+		if (c->qsDesc.isEmpty()) {
+			MumbleProto::RequestBlob mprb;
+			mprb.add_channel_description(id);
+			g.sh->sendMessage(mprb);
+		}
+	}
 
-	g.sh->sendMessage(mpacl);
+	g.sh->requestACL(id);
 
 	if (aclEdit) {
 		aclEdit->reject();
@@ -1414,10 +1486,7 @@ void MainWindow::on_qaChannelLink_triggered() {
 	if (! l)
 		l = Channel::get(0);
 
-	MumbleProto::ChannelState mpcs;
-	mpcs.set_channel_id(c->iId);
-	mpcs.add_links_add(l->iId);
-	g.sh->sendMessage(mpcs);
+	g.sh->addChannelLink(c->iId, l->iId);
 }
 
 void MainWindow::on_qaChannelUnlink_triggered() {
@@ -1426,10 +1495,7 @@ void MainWindow::on_qaChannelUnlink_triggered() {
 	if (! l)
 		l = Channel::get(0);
 
-	MumbleProto::ChannelState mpcs;
-	mpcs.set_channel_id(c->iId);
-	mpcs.add_links_remove(l->iId);
-	g.sh->sendMessage(mpcs);
+	g.sh->removeChannelLink(c->iId, l->iId);
 }
 
 void MainWindow::on_qaChannelUnlinkAll_triggered() {
@@ -1456,13 +1522,7 @@ void MainWindow::on_qaChannelSendMessage_triggered() {
 	c = Channel::get(id);
 
 	if (c && (res==QDialog::Accepted)) {
-		MumbleProto::TextMessage mptm;
-		if (texm->bTreeMessage)
-			mptm.add_tree_id(id);
-		else
-			mptm.add_channel_id(id);
-		mptm.set_message(u8(texm->message()));
-		g.sh->sendMessage(mptm);
+		g.sh->sendChannelTextMessage(id, texm->message(), texm->bTreeMessage);
 
 		if (texm->bTreeMessage)
 			g.l->log(Log::TextMessage, tr("(Tree) %1: %2").arg(Log::formatChannel(c)).arg(texm->message()), tr("Message to tree %1").arg(c->qsName));
@@ -1478,9 +1538,7 @@ void MainWindow::updateMenuPermissions() {
 	ChanACL::Permissions p = static_cast<ChanACL::Permissions>(c ? c->uiPermissions : ChanACL::None);
 
 	if (c && ! p) {
-		MumbleProto::PermissionQuery mppq;
-		mppq.set_channel_id(c->iId);
-		g.sh->sendMessage(mppq);
+		g.sh->requestChannelPermissions(c->iId);
 		if (c->iId == 0)
 			p = g.pPermissions;
 		else
@@ -1493,9 +1551,7 @@ void MainWindow::updateMenuPermissions() {
 	ChanACL::Permissions pparent = static_cast<ChanACL::Permissions>(cparent ? cparent->uiPermissions : ChanACL::None);
 
 	if (cparent && ! pparent) {
-		MumbleProto::PermissionQuery mppq;
-		mppq.set_channel_id(cparent->iId);
-		g.sh->sendMessage(mppq);
+		g.sh->requestChannelPermissions(cparent->iId);
 		if (cparent->iId == 0)
 			pparent = g.pPermissions;
 		else
@@ -1509,9 +1565,7 @@ void MainWindow::updateMenuPermissions() {
 	ChanACL::Permissions homep = static_cast<ChanACL::Permissions>(homec ? homec->uiPermissions : ChanACL::None);
 
 	if (homec && ! homep) {
-		MumbleProto::PermissionQuery mppq;
-		mppq.set_channel_id(homec->iId);
-		g.sh->sendMessage(mppq);
+		g.sh->requestChannelPermissions(homec->iId);
 		if (homec->iId == 0)
 			homep = g.pPermissions;
 		else
@@ -1520,9 +1574,19 @@ void MainWindow::updateMenuPermissions() {
 		homec->uiPermissions = homep;
 	}
 
-	qaUserMute->setEnabled(p & (ChanACL::Write | ChanACL::MuteDeafen));
-	qaUserDeaf->setEnabled(p & (ChanACL::Write | ChanACL::MuteDeafen));
-	qaUserTextMessage->setEnabled(p & (ChanACL::Write | ChanACL::TextMessage));
+	if (cu) {
+		qaUserMute->setEnabled(p & (ChanACL::Write | ChanACL::MuteDeafen) && ((cu != user) || cu->bMute || cu->bSuppress));
+		qaUserDeaf->setEnabled(p & (ChanACL::Write | ChanACL::MuteDeafen) && ((cu != user) || cu->bDeaf));
+		qaUserTextMessage->setEnabled(p & (ChanACL::Write | ChanACL::TextMessage));
+		qaUserInformation->setEnabled((g.pPermissions & (ChanACL::Write | ChanACL::Register)) || (p & (ChanACL::Write | ChanACL::Enter)) || (cu == user));
+	} else {
+		qaUserMute->setEnabled(false);
+		qaUserDeaf->setEnabled(false);
+		qaUserTextMessage->setEnabled(false);
+		qaUserInformation->setEnabled(false);
+	}
+
+	qaChannelJoin->setEnabled(p & (ChanACL::Write | ChanACL::Enter));
 
 	qaChannelAdd->setEnabled(p & (ChanACL::Write | ChanACL::MakeChannel | ChanACL::MakeTempChannel));
 	qaChannelRemove->setEnabled(p & ChanACL::Write);
@@ -1579,10 +1643,7 @@ void MainWindow::on_qaAudioMute_triggered() {
 		g.l->log(Log::SelfMute, tr("Muted."));
 	}
 
-	MumbleProto::UserState mpus;
-	mpus.set_self_mute(g.s.bMute);
-	mpus.set_self_deaf(g.s.bDeaf);
-	g.sh->sendMessage(mpus);
+	g.sh->setSelfMuteDeafState(g.s.bMute, g.s.bDeaf);
 
 	updateTrayIcon();
 }
@@ -1610,10 +1671,7 @@ void MainWindow::on_qaAudioDeaf_triggered() {
 		g.l->log(Log::SelfMute, tr("Undeafened."));
 	}
 
-	MumbleProto::UserState mpus;
-	mpus.set_self_mute(g.s.bMute);
-	mpus.set_self_deaf(g.s.bDeaf);
-	g.sh->sendMessage(mpus);
+	g.sh->setSelfMuteDeafState(g.s.bMute, g.s.bDeaf);
 
 	updateTrayIcon();
 }
@@ -1869,10 +1927,7 @@ void MainWindow::on_gsWhisper_triggered(bool down, QVariant scdata) {
 			if (! st.bUsers) {
 				Channel *c = mapChannel(st.iChannel);
 				if (c) {
-					MumbleProto::UserState mpus;
-					mpus.set_session(g.uiSession);
-					mpus.set_channel_id(c->iId);
-					g.sh->sendMessage(mpus);
+					g.sh->joinChannel(c->iId);
 				}
 				return;
 			}
@@ -1909,6 +1964,7 @@ void MainWindow::serverConnected() {
 	unsigned short port;
 	g.sh->getConnectionInfo(host, port, uname, pw);
 	g.l->log(Log::ServerConnected, tr("Connected."));
+	qaSelfComment->setEnabled(true);
 	qaServerDisconnect->setEnabled(true);
 	qaServerInformation->setEnabled(true);
 	qaServerBanList->setEnabled(true);
@@ -1920,15 +1976,16 @@ void MainWindow::serverConnected() {
 
 	qtvUsers->setRowHidden(0, QModelIndex(), false);
 
+	g.bAllowHTML = true;
+	g.uiMessageLength = 5000;
+	g.uiImageLength = 131072;
+
 	if (g.s.bMute || g.s.bDeaf) {
-		MumbleProto::UserState mpus;
-		mpus.set_self_mute(g.s.bMute);
-		mpus.set_self_deaf(g.s.bDeaf);
-		g.sh->sendMessage(mpus);
+		g.sh->setSelfMuteDeafState(g.s.bMute, g.s.bDeaf);
 	}
 
 #ifdef Q_OS_WIN
-	TaskList::addToRecentList(g.s.qsLastServer, host, port);
+	TaskList::addToRecentList(g.s.qsLastServer, uname, host, port);
 #endif
 }
 
@@ -1959,6 +2016,7 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 
 	g.uiSession = 0;
 	g.pPermissions = ChanACL::None;
+	qaSelfComment->setEnabled(false);
 	qaServerDisconnect->setEnabled(false);
 	qaServerInformation->setEnabled(false);
 	qaServerBanList->setEnabled(false);
@@ -2027,7 +2085,7 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 			foreach(QSslError e, g.sh->qlErrors)
 				qsl << QString::fromLatin1("<li>%1</li>").arg(e.errorString());
 
-			QMessageBox qmb(QMessageBox::Warning, tr("Mumble"),
+			QMessageBox qmb(QMessageBox::Warning, QLatin1String("Mumble"),
 			                tr("<p>%1.<br />The specific errors with this certificate are: </p><ol>%2</ol>"
 			                   "<p>Do you wish to accept this certificate anyway?<br />(It will also be stored so you won't be asked this again.)</p>"
 			                  ).arg(basereason).arg(qsl.join(QString())), QMessageBox::Yes | QMessageBox::No, this);
@@ -2120,10 +2178,25 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 			qtReconnect->start();
 		}
 	}
+	qstiIcon->setToolTip(tr("Mumble -- %1").arg(QLatin1String(MUMBLE_RELEASE)));
 	AudioInput::setMaxBandwidth(-1);
 }
 
-void MainWindow::setupIconMenu(bool top) {
+void MainWindow::trayAboutToShow() {
+	bool top = false;
+
+	QPoint p = qstiIcon->geometry().center();
+	if (p.isNull()) {
+		p = QCursor::pos();
+	}
+
+	QDesktopWidget dw;
+
+	QRect qr = dw.screenGeometry(p);
+
+	if (p.y() < (qr.height() / 2))
+		top = true;
+
 	qmTray->clear();
 	if (top) {
 		qmTray->addAction(qaQuit);
@@ -2155,24 +2228,6 @@ void MainWindow::on_Icon_activated(QSystemTrayIcon::ActivationReason reason) {
 		return;
 	}
 
-	if (reason == QSystemTrayIcon::Context) {
-		bool top = false;
-
-		QPoint p = qstiIcon->geometry().center();
-		if (p.isNull()) {
-			p = QCursor::pos();
-		}
-
-		QDesktopWidget dw;
-
-		QRect qr = dw.screenGeometry(p);
-
-		if (p.y() < (qr.height() / 2))
-			top = true;
-
-		setupIconMenu(top);
-	}
-
 	if (reason == QSystemTrayIcon::Trigger) {
 		if (! isVisible()) {
 			show();
@@ -2286,7 +2341,10 @@ void MainWindow::context_triggered() {
 QPair<QByteArray, QImage> MainWindow::openImageFile() {
 	QPair<QByteArray, QImage> retval;
 
-	QString fname = QFileDialog::getOpenFileName(this, tr("Choose image file"), g.s.qsImagePath, tr("Images (*.png *.jpg)"));
+	if (g.s.qsImagePath.isEmpty() || ! QDir::root().exists(g.s.qsImagePath))
+		g.s.qsImagePath = QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
+
+	QString fname = QFileDialog::getOpenFileName(this, tr("Choose image file"), g.s.qsImagePath, tr("Images (*.png *.jpg *.svg)"));
 
 	if (fname.isNull())
 		return retval;
@@ -2303,8 +2361,13 @@ QPair<QByteArray, QImage> MainWindow::openImageFile() {
 	QByteArray qba = f.readAll();
 	f.close();
 
-	QImage img;
-	if (! img.loadFromData(qba)) {
+	QBuffer qb(&qba);
+	qb.open(QIODevice::ReadOnly);
+
+	QImageReader qir(&qb, fi.suffix().toUtf8());
+
+	QImage img = qir.read();
+	if (img.isNull()) {
 		QMessageBox::warning(this, tr("Failed to load image"), tr("Image format not recognized."));
 		return retval;
 	}
@@ -2361,3 +2424,14 @@ bool MainWindow::launchCompatibilityClient(const QUrl &url) {
 
 	return false;
 }
+
+void MainWindow::destroyUserInformation() {
+	UserInformation *ui = static_cast<UserInformation *>(sender());
+	QMap<unsigned int, UserInformation *>::iterator i;
+	for (i=qmUserInformations.begin(); i != qmUserInformations.end(); ++i) {
+		if (i.value() == ui) {
+			qmUserInformations.erase(i);
+			return;
+		}
+	}
+}
