@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -47,6 +47,7 @@ class TextToSpeech;
 class UserModel;
 class Tokens;
 class Channel;
+class UserInformation;
 
 struct ShortcutTarget;
 
@@ -75,6 +76,8 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 	public:
 		UserModel *pmModel;
 		QSystemTrayIcon *qstiIcon;
+		QMenu *qmUser;
+		QMenu *qmChannel;
 		QMenu *qmTray;
 		QIcon qiIcon, qiIconMuteSelf, qiIconMuteServer, qiIconDeafSelf, qiIconDeafServer, qiIconMuteSuppressed;
 		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingWhisperChannel, qiTalkingOff;
@@ -95,6 +98,9 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		bool bSuppressAskOnQuit;
 		bool bAutoUnmute;
 
+		unsigned int uiContextSession;
+		int iContextChannel;
+
 		void recheckTTS();
 		void msgBox(QString msg);
 		void setOnTop(bool top);
@@ -119,21 +125,18 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		QMap<int, int> qmTargetUse;
 		Channel *mapChannel(int idx) const;
 		int iTargetCounter;
+		QMap<unsigned int, UserInformation *> qmUserInformations;
 
 		void createActions();
 		void setupGui();
 		void customEvent(QEvent *evt);
 		void findDesiredChannel();
 		void setupView(bool toggle_minimize = true);
-		void setupIconMenu(bool top = false);
 		bool launchCompatibilityClient(const QUrl &url);
 		bool bNoHide;
 		virtual void closeEvent(QCloseEvent *e);
 		virtual void hideEvent(QHideEvent *e);
 
-	private:
-		qint32 cmUid;
-		qint32 cmCid;
 		bool handleSpecialContextMenu(const QUrl &url, const QPoint &_pos, bool focus = false);
 		Channel* getContextMenuChannel();
 		ClientUser* getContextMenuUser();
@@ -148,8 +151,10 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaServerTexture_triggered();
 		void on_qaServerTextureRemove_triggered();
 		void on_qaServerTokens_triggered();
-		void on_qmUser_aboutToShow();
-		void on_qaUserComment_triggered();
+		void on_qmSelf_aboutToShow();
+		void on_qaSelfComment_triggered();
+		void on_qaSelfRegister_triggered();
+		void qmUser_aboutToShow();
 		void on_qaUserCommentReset_triggered();
 		void on_qaUserCommentView_triggered();
 		void on_qaUserKick_triggered();
@@ -159,10 +164,11 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaUserLocalMute_triggered();
 		void on_qaUserTextMessage_triggered();
 		void on_qaUserRegister_triggered();
+		void on_qaUserInformation_triggered();
 		void on_qaUserFriendAdd_triggered();
 		void on_qaUserFriendRemove_triggered();
 		void on_qaUserFriendUpdate_triggered();
-		void on_qmChannel_aboutToShow();
+		void qmChannel_aboutToShow();
 		void on_qaChannelJoin_triggered();
 		void on_qaChannelAdd_triggered();
 		void on_qaChannelRemove_triggered();
@@ -215,6 +221,8 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void updateTarget();
 		void updateMenuPermissions();
 		void talkingChanged();
+		void destroyUserInformation();
+		void trayAboutToShow();
 	public:
 		MainWindow(QWidget *parent);
 		~MainWindow();
