@@ -34,24 +34,11 @@
 #include "../../overlay/overlay.h"
 #include "ConfigDialog.h"
 #include "ClientUser.h"
+#include "SharedMemory.h"
 #include "ui_Overlay.h"
 
 class User;
-
-class SharedMemoryPrivate;
-class SharedMemory {
-	private:
-		Q_DISABLE_COPY(SharedMemory)
-	protected:
-		SharedMemoryPrivate *d;
-	public:
-		SharedMem *sm;
-		SharedMemory();
-		~SharedMemory();
-		void resolve(QLibrary *lib);
-		bool tryLock();
-		void unlock();
-};
+class Overlay;
 
 class OverlayConfig : public ConfigWidget, public Ui::OverlayConfig {
 	private:
@@ -80,55 +67,91 @@ class OverlayConfig : public ConfigWidget, public Ui::OverlayConfig {
 		bool expert(bool);
 };
 
+struct OverlayTextLine {
+	enum Decoration { None, Muted, Deafened };
+	QString qsText;
+	unsigned int uiSession;
+	QRgb uiColor;
+	Decoration dDecor;
+	int iPriority;
+	QRect qrRect;
+	QByteArray qbaHash;
+	OverlayTextLine() : uiSession(0), uiColor(0), dDecor(None), iPriority(0) { };
+	OverlayTextLine(const QString &t, quint32 c, int priority = 0, unsigned int session = 0, Decoration d = None) : qsText(t), uiSession(session), uiColor(c), dDecor(d), iPriority(priority) { };
+	bool operator <(const OverlayTextLine &o) const;
+};
+
+class OverlayClient : public QObject {
+		friend class Overlay;
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(OverlayClient);
+	protected:
+		typedef QPair<QByteArray, QRgb> TextImageKey;
+		struct TextImage {
+			QImage qiImage;
+			float iOffset;
+			TextImage(const QImage &img, int i) : qiImage(img), iOffset(i) {};
+		};
+
+		OverlayMsg omMsg;
+		QLocalSocket *qlsSocket;
+		SharedMemory2 *smMem;
+		unsigned int uiWidth, uiHeight;
+		float fItemHeight, fEdge, fBase;
+		int iItemHeight;
+		QList<OverlayTextLine> qlLines;
+		QRect qrLast;
+		QFont fFont;
+		Timer t;
+		QImage qiMuted, qiDeafened;
+		QCache<TextImageKey, TextImage> qcTexts;
+
+		void setupRender();
+	protected slots:
+		void readyRead();
+	public:
+		OverlayClient(QLocalSocket *, QObject *);
+		~OverlayClient();
+		bool setTexts(const QList<OverlayTextLine> &lines);
+		void reset();
+};
+
 class OverlayPrivate : public QObject {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(OverlayPrivate);
 	public:
-		OverlayPrivate() : QObject() {};
+		OverlayPrivate(QObject *p) : QObject(p) {};
 };
 
 class Overlay : public QObject {
 		friend class OverlayConfig;
+		friend class OverlayClient;
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(Overlay)
 	protected:
 		OverlayPrivate *d;
 
-		enum Decoration { None, Muted, Deafened };
+		QList<OverlayTextLine> qlCurrentTexts;
+		QSet<unsigned int> qsQueried;
 
-		struct TextLine {
-			QString qsText;
-			unsigned int uiSession;
-			quint32 uiColor;
-			Decoration dDecor;
-			int iPriority;
-			TextLine(const QString &t, quint32 c, int priority = 0, unsigned int session = 0, Decoration d = None) : qsText(t), uiSession(session), uiColor(c), dDecor(d), iPriority(priority) { };
-			bool operator <(const TextLine &o) const;
-		};
-
-		QByteArray qbaMuted, qbaDeafened;
-		QList<TextLine> qlCurrentTexts;
-		QHash<QString, unsigned char *> qhTextures;
-		QHash<QString, short> qhWidths;
-		QSet<unsigned int> qsForce;
-		QLibrary *qlOverlay;
-		QTimer *qtTimer;
-		QFont qfFont;
-		float fFontBase;
-		SharedMemory sm;
 		void platformInit();
-		void setTexts(const QList<TextLine> &lines);
-		void fixFont();
-		void clearCache();
+		void setTexts(const QList<OverlayTextLine> &lines);
+
+		QLocalServer *qlsServer;
+		QList<OverlayClient *> qlClients;
+	protected slots:
+		void disconnected();
+		void error(QLocalSocket::LocalSocketError);
+		void newConnection();
 	public:
 		Overlay();
 		~Overlay();
 		bool isActive() const;
-		void verifyTexture(ClientUser *cp);
+		void verifyTexture(ClientUser *cp, bool allowupdate = true);
 	public slots:
-		void on_Timer_timeout();
 		void updateOverlay();
 		void setActive(bool act);
 		void toggleShow();
