@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -107,7 +107,7 @@ void PluginConfig::on_qpbConfig_clicked() {
 	if (pi->p->config)
 		pi->p->config(winId());
 	else
-		QMessageBox::information(this, tr("Mumble"), tr("Plugin has no configure function."), QMessageBox::Ok, QMessageBox::NoButton);
+		QMessageBox::information(this, QLatin1String("Mumble"), tr("Plugin has no configure function."), QMessageBox::Ok, QMessageBox::NoButton);
 }
 
 void PluginConfig::on_qpbAbout_clicked() {
@@ -119,7 +119,7 @@ void PluginConfig::on_qpbAbout_clicked() {
 	if (pi->p->about)
 		pi->p->about(winId());
 	else
-		QMessageBox::information(this, tr("Mumble"), tr("Plugin has no about function."), QMessageBox::Ok, QMessageBox::NoButton);
+		QMessageBox::information(this, QLatin1String("Mumble"), tr("Plugin has no about function."), QMessageBox::Ok, QMessageBox::NoButton);
 }
 
 void PluginConfig::on_qpbReload_clicked() {
@@ -139,6 +139,19 @@ void PluginConfig::refillPluginList() {
 		i->setText(0, pi->description);
 		qhInfos[qtwPlugins->indexOfTopLevelItem(i)] = pi;
 	}
+	qtwPlugins->setCurrentItem(qtwPlugins->topLevelItem(0));
+	on_qtwPlugins_currentItemChanged(qtwPlugins->topLevelItem(0), NULL);
+}
+
+void PluginConfig::on_qtwPlugins_currentItemChanged(QTreeWidgetItem *current, QTreeWidgetItem *) {
+	PluginInfo *pi = qhInfos.value(qtwPlugins->indexOfTopLevelItem(current));
+	if (pi) {
+		qpbAbout->setEnabled(pi->p->about != NULL);
+		qpbConfig->setEnabled(pi->p->config != NULL);
+	} else {
+		qpbAbout->setEnabled(false);
+		qpbConfig->setEnabled(false);
+	}
 }
 
 Plugins::Plugins(QObject *p) : QObject(p) {
@@ -418,7 +431,7 @@ void Plugins::finished() {
 							rescan=true;
 						}
 					} else if (f.open(QIODevice::ReadOnly)) {
-						QString h = QLatin1String(QCryptographicHash::hash(f.readAll(), QCryptographicHash::Sha1).toHex());
+						QString h = QLatin1String(sha1(f.readAll()).toHex());
 						f.close();
 						if (h == wanthash) {
 							if (qd != qdu) {
@@ -452,7 +465,7 @@ void Plugins::finished() {
 								rescan=true;
 							}
 						} else if (f.open(QIODevice::ReadOnly)) {
-							QString h = QLatin1String(QCryptographicHash::hash(f.readAll(), QCryptographicHash::Sha1).toHex());
+							QString h = QLatin1String(sha1(f.readAll()).toHex());
 							f.close();
 							if (h == wanthash) {
 								qmPluginHash.remove(filename);
@@ -476,7 +489,7 @@ void Plugins::finished() {
 			QString fname = QFileInfo(path).fileName();
 			if (qmPluginHash.contains(fname)) {
 				QByteArray qba = rep->readAll();
-				if (qmPluginHash.value(fname) == QLatin1String(QCryptographicHash::hash(qba, QCryptographicHash::Sha1).toHex())) {
+				if (qmPluginHash.value(fname) == QLatin1String(sha1(qba).toHex())) {
 					bool verified = true;
 #ifdef Q_OS_WIN
 					verified = false;
