@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -29,7 +29,7 @@
 */
 
 #include "PulseAudio.h"
-#include "Player.h"
+#include "User.h"
 #include "Global.h"
 #include "MainWindow.h"
 #include "Timer.h"
@@ -67,7 +67,6 @@ class PulseAudioInit : public DeferInit {
 		void initialize() {
 			pasys = new PulseAudioSystem();
 			pasys->qmWait.lock();
-			pasys->start(QThread::TimeCriticalPriority);
 			pasys->qwcWait.wait(&pasys->qmWait, 1000);
 			pasys->qmWait.unlock();
 			if (pasys->bPulseIsGood) {
@@ -76,6 +75,8 @@ class PulseAudioInit : public DeferInit {
 			} else {
 				airPulseAudio = NULL;
 				aorPulseAudio = NULL;
+				delete pasys;
+				pasys = NULL;
 			}
 		};
 		void destroy() {
@@ -83,8 +84,10 @@ class PulseAudioInit : public DeferInit {
 				delete airPulseAudio;
 			if (aorPulseAudio)
 				delete aorPulseAudio;
-			delete pasys;
-			pasys = NULL;
+			if (pasys) {
+				delete pasys;
+				pasys = NULL;
+			}
 		};
 };
 
@@ -97,8 +100,8 @@ PulseAudioSystem::PulseAudioSystem() {
 	bPositionalCache = false;
 	bPulseIsGood = false;
 
-	pam = pa_mainloop_new();
-	pa_mainloop_api *api = pa_mainloop_get_api(pam);
+	pam = pa_threaded_mainloop_new();
+	pa_mainloop_api *api = pa_threaded_mainloop_get_api(pam);
 
 	pacContext = pa_context_new(api, "Mumble");
 
@@ -109,30 +112,27 @@ PulseAudioSystem::PulseAudioSystem() {
 
 	pade = api->defer_new(api, defer_event_callback, this);
 	api->defer_enable(pade, false);
+
+	pa_threaded_mainloop_start(pam);
 }
 
 PulseAudioSystem::~PulseAudioSystem() {
-	pa_mainloop_quit(pam, 0);
-	wait();
+	pa_threaded_mainloop_stop(pam);
 	pa_context_disconnect(pacContext);
 	pa_context_unref(pacContext);
-	pa_mainloop_free(pam);
+	pa_threaded_mainloop_free(pam);
 }
 
 void PulseAudioSystem::wakeup() {
-	pa_mainloop_api *api = pa_mainloop_get_api(pam);
+	pa_mainloop_api *api = pa_threaded_mainloop_get_api(pam);
 	api->defer_enable(pade, true);
-	pa_mainloop_wakeup(pam);
 }
 
-void PulseAudioSystem::run() {
-	// Make sure we don't exit too early.
-	qmWait.lock();
-	qmWait.unlock();
-	int rv;
-	pa_mainloop_run(pam, &rv);
-	if (rv != 0)
-		qWarning("PulseAudio: Mainloop quit with retval %d", rv);
+void PulseAudioSystem::wakeup_lock() {
+	pa_threaded_mainloop_lock(pam);
+	pa_mainloop_api *api = pa_threaded_mainloop_get_api(pam);
+	api->defer_enable(pade, true);
+	pa_threaded_mainloop_unlock(pam);
 }
 
 void PulseAudioSystem::defer_event_callback(pa_mainloop_api *a, pa_defer_event *e, void *userdata) {
@@ -140,7 +140,6 @@ void PulseAudioSystem::defer_event_callback(pa_mainloop_api *a, pa_defer_event *
 	pas->eventCallback(a, e);
 }
 
-
 void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 	api->defer_enable(pade, false);
 
@@ -203,16 +202,17 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 			pa_buffer_attr buff;
 			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasOutput);
 			const unsigned int iBlockLen = ((pao->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
-			buff.maxlength = iBlockLen * NBLOCKS;
-			buff.tlength = g.s.iOutputDelay * iBlockLen;
-			buff.prebuf = buff.tlength;
+			buff.tlength = iBlockLen * (g.s.iOutputDelay+1);
 			buff.minreq = iBlockLen;
+			buff.maxlength = -1;
+			buff.prebuf = -1;
+			buff.fragsize = iBlockLen;
 
 			iDelayCache = g.s.iOutputDelay;
 			bPositionalCache = g.s.doPositionalAudio();
 			qsOutputCache = odev;
 
-			pa_stream_connect_playback(pasOutput, qPrintable(odev), &buff, PA_STREAM_INTERPOLATE_TIMING, NULL, NULL);
+			pa_stream_connect_playback(pasOutput, qPrintable(odev), &buff, PA_STREAM_ADJUST_LATENCY, NULL, NULL);
 		}
 	}
 
@@ -237,7 +237,7 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 						if ((pss.format != PA_SAMPLE_FLOAT32NE) && (pss.format != PA_SAMPLE_S16NE))
 							pss.format = PA_SAMPLE_FLOAT32NE;
 
-						pasInput = pa_stream_new(pacContext, "Mumble Microphone", &pss, NULL);
+						pasInput = pa_stream_new(pacContext, "Microphone", &pss, NULL);
 						pa_stream_set_state_callback(pasInput, stream_callback, this);
 						pa_stream_set_read_callback(pasInput, read_callback, this);
 					}
@@ -262,12 +262,15 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 			pa_buffer_attr buff;
 			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasInput);
 			const unsigned int iBlockLen = ((pai->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
-			buff.maxlength = iBlockLen * NBLOCKS;
+			buff.tlength = iBlockLen;
+			buff.minreq = iBlockLen;
+			buff.maxlength = -1;
+			buff.prebuf = -1;
 			buff.fragsize = iBlockLen;
 
 			qsInputCache = idev;
 
-			pa_stream_connect_record(pasInput, qPrintable(idev), &buff, PA_STREAM_INTERPOLATE_TIMING);
+			pa_stream_connect_record(pasInput, qPrintable(idev), &buff, PA_STREAM_ADJUST_LATENCY);
 		}
 	}
 
@@ -317,12 +320,15 @@ void PulseAudioSystem::eventCallback(pa_mainloop_api *api, pa_defer_event *) {
 			pa_buffer_attr buff;
 			const pa_sample_spec *pss = pa_stream_get_sample_spec(pasSpeaker);
 			const unsigned int iBlockLen = ((pai->iFrameSize * pss->rate) / SAMPLE_RATE) * pss->channels * ((pss->format == PA_SAMPLE_FLOAT32NE) ? sizeof(float) : sizeof(short));
-			buff.maxlength = iBlockLen * NBLOCKS;
+			buff.tlength = iBlockLen;
+			buff.minreq = iBlockLen;
+			buff.maxlength = -1;
+			buff.prebuf = -1;
 			buff.fragsize = iBlockLen;
 
 			qsEchoCache = edev;
 
-			pa_stream_connect_record(pasSpeaker, qPrintable(edev), &buff, PA_STREAM_INTERPOLATE_TIMING);
+			pa_stream_connect_record(pasSpeaker, qPrintable(edev), &buff, PA_STREAM_ADJUST_LATENCY);
 		}
 	}
 }
@@ -332,7 +338,21 @@ void PulseAudioSystem::context_state_callback(pa_context *c, void *userdata) {
 	pas->contextCallback(c);
 }
 
-void PulseAudioSystem::subscribe_callback(pa_context *, pa_subscription_event_type, unsigned int, void *userdata) {
+void PulseAudioSystem::subscribe_callback(pa_context *, pa_subscription_event_type evt, unsigned int, void *userdata) {
+	switch (evt & PA_SUBSCRIPTION_EVENT_TYPE_MASK) {
+		case PA_SUBSCRIPTION_EVENT_NEW:
+		case PA_SUBSCRIPTION_EVENT_REMOVE:
+			break;
+		default:
+			return;
+	}
+	switch (evt & PA_SUBSCRIPTION_EVENT_FACILITY_MASK) {
+		case PA_SUBSCRIPTION_EVENT_SINK:
+		case PA_SUBSCRIPTION_EVENT_SOURCE:
+			break;
+		default:
+			return;
+	}
 	PulseAudioSystem *pas = reinterpret_cast<PulseAudioSystem *>(userdata);
 	qWarning("PulseAudio: Sinks or inputs changed (inserted or removed sound card)");
 	pas->query();
@@ -446,8 +466,7 @@ void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata
 	AudioOutputPtr ao = g.ao;
 	PulseAudioOutput *pao = dynamic_cast<PulseAudioOutput *>(ao.get());
 
-	int samples = bytes / sizeof(float);
-	float buffer[samples];
+	unsigned char buffer[bytes];
 
 	if (! pao) {
 		// Transitioning, but most likely transitions back, so just zero.
@@ -514,12 +533,11 @@ void PulseAudioSystem::write_callback(pa_stream *s, size_t bytes, void *userdata
 		pao->initializeMixer(chanmasks);
 	}
 
-
 	const unsigned int iSampleSize = pao->iSampleSize;
-	samples = bytes / iSampleSize;
+	const unsigned int samples = bytes / iSampleSize;
 
 	if (! pao->mix(buffer, samples))
-		memset(buffer, 0, samples * iSampleSize);
+		memset(buffer, 0, bytes);
 	pa_stream_write(s, buffer, iSampleSize * samples, NULL, 0, PA_SEEK_RELATIVE);
 }
 
@@ -554,7 +572,9 @@ void PulseAudioSystem::contextCallback(pa_context *c) {
 		default:
 			return;
 	}
+	qmWait.lock();
 	qwcWait.wakeAll();
+	qmWait.unlock();
 }
 
 PulseAudioInputRegistrar::PulseAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("PulseAudio"), 10) {
@@ -620,29 +640,51 @@ void PulseAudioOutputRegistrar::setDeviceChoice(const QVariant &choice, Settings
 }
 
 PulseAudioInput::PulseAudioInput() {
-	pssMic.rate = 0;
-	pssEcho.rate = 0;
+	memset(&pssMic, 0, sizeof(pssMic));
+	memset(&pssEcho, 0, sizeof(pssEcho));
 	bRunning = true;
 	if (pasys)
-		pasys->wakeup();
+		pasys->wakeup_lock();
 };
 
 PulseAudioInput::~PulseAudioInput() {
 	bRunning = false;
+	qmMutex.lock();
+	qwcWait.wakeAll();
+	qmMutex.unlock();
+	wait();
 	if (pasys)
-		pasys->wakeup();
+		pasys->wakeup_lock();
+}
+
+void PulseAudioInput::run() {
+	qmMutex.lock();
+	while (bRunning)
+		qwcWait.wait(&qmMutex);
+	qmMutex.unlock();
 }
 
 PulseAudioOutput::PulseAudioOutput() {
-	pss.rate = 0;
-	pcm.channels = 0;
+	memset(&pss, 0, sizeof(pss));
+	memset(&pcm, 0, sizeof(pcm));
 	bRunning = true;
 	if (pasys)
-		pasys->wakeup();
+		pasys->wakeup_lock();
 }
 
 PulseAudioOutput::~PulseAudioOutput() {
 	bRunning = false;
+	qmMutex.lock();
+	qwcWait.wakeAll();
+	qmMutex.unlock();
+	wait();
 	if (pasys)
-		pasys->wakeup();
+		pasys->wakeup_lock();
+}
+
+void PulseAudioOutput::run() {
+	qmMutex.lock();
+	while (bRunning)
+		qwcWait.wait(&qmMutex);
+	qmMutex.unlock();
 }
