@@ -84,6 +84,7 @@ static HANDLE loadQoS() {
 ServerHandler::ServerHandler() {
 	cConnection.reset();
 	qusUdp = NULL;
+	bStrong = false;
 
 	// For some strange reason, on Win32, we have to call supportsSsl before the cipher list is ready.
 	qWarning("OpenSSL Support: %d", QSslSocket::supportsSsl());
@@ -248,6 +249,7 @@ void ServerHandler::sendProtoMessage(const ::google::protobuf::Message &msg, uns
 
 void ServerHandler::run() {
 	qbaDigest = QByteArray();
+	bStrong = true;
 	QSslSocket *qtsSock = new QSslSocket(this);
 
 	if (! g.s.bSuppressIdentity && CertWizard::validateCert(g.s.kpCertificate)) {
@@ -318,6 +320,7 @@ void ServerHandler::run() {
 }
 
 void ServerHandler::setSslErrors(const QList<QSslError> &errors) {
+	bStrong = false;
 	qscCert = cConnection->peerCertificateChain();
 	if ((qscCert.size() > 0)  && (QString::fromLatin1(qscCert.at(0).digest(QCryptographicHash::Sha1).toHex()) == Database::getDigest(qsHostName, usPort)))
 		cConnection->proceedAnyway();
@@ -452,7 +455,7 @@ void ServerHandler::serverConnectionConnected() {
 
 	if (! qscCert.isEmpty()) {
 		const QSslCertificate &qsc = qscCert.last();
-		qbaDigest = QCryptographicHash::hash(qsc.publicKey().toDer(), QCryptographicHash::Sha1);
+		qbaDigest = sha1(qsc.publicKey().toDer());
 		bUdp = Database::getUdp(qbaDigest);
 	} else {
 		bUdp = true;
@@ -543,9 +546,172 @@ void ServerHandler::setConnectionInfo(const QString &host, unsigned short port,
 	qsPassword = pw;
 }
 
-void ServerHandler::getConnectionInfo(QString &host, unsigned short &port, QString &username, QString &pw) {
+void ServerHandler::getConnectionInfo(QString &host, unsigned short &port, QString &username, QString &pw) const {
 	host = qsHostName;
 	port = usPort;
 	username = qsUserName;
 	pw = qsPassword;
 }
+
+bool ServerHandler::isStrong() const {
+	return bStrong;
+}
+
+void ServerHandler::requestUserStats(unsigned int uiSession, bool statsOnly) {
+	MumbleProto::UserStats mpus;
+	mpus.set_session(uiSession);
+	mpus.set_stats_only(statsOnly);
+	sendMessage(mpus);
+}
+
+void ServerHandler::joinChannel(unsigned int channel) {
+	MumbleProto::UserState mpus;
+	// TODO: remove global uiSession reference
+	mpus.set_session(g.uiSession);
+	mpus.set_channel_id(channel);
+	sendMessage(mpus);
+}
+
+void ServerHandler::createChannel(unsigned int parent_, const QString &name, const QString &description, unsigned int position, bool temporary) {
+	MumbleProto::ChannelState mpcs;
+	mpcs.set_parent(parent_);
+	mpcs.set_name(u8(name));
+	mpcs.set_description(u8(description));
+	mpcs.set_position(position);
+	mpcs.set_temporary(temporary);
+	sendMessage(mpcs);
+}
+
+void ServerHandler::setTexture(const QByteArray &qba) {
+	QByteArray texture;
+	if ((uiVersion >= 0x010202) || qba.isEmpty()) {
+		texture = qba;
+	} else {
+		QByteArray raw = qba;
+		QBuffer qb(& raw);
+		qb.open(QIODevice::ReadOnly);
+		QImageReader qir;
+		if (qba.startsWith("<?xml"))
+			qir.setFormat("svg");
+		qir.setDevice(&qb);
+
+		QSize sz = qir.size();
+		sz.scale(600, 60, Qt::KeepAspectRatio);
+		qir.setScaledSize(sz);
+
+		QImage tex = qir.read();
+
+		if (tex.isNull())
+			return;
+
+		qWarning() << tex.width() << tex.height();
+
+		raw = QByteArray(600*60*4, 0);
+		QImage img(reinterpret_cast<unsigned char *>(raw.data()), 600, 60, QImage::Format_ARGB32);
+
+		QPainter imgp(&img);
+		imgp.setRenderHint(QPainter::Antialiasing);
+		imgp.setRenderHint(QPainter::TextAntialiasing);
+		imgp.setCompositionMode(QPainter::CompositionMode_SourceOver);
+		imgp.drawImage(0, 0, tex);
+
+		texture = qCompress(QByteArray(reinterpret_cast<const char *>(img.bits()), 600*60*4));
+	}
+	MumbleProto::UserState mpus;
+	mpus.set_texture(blob(texture));
+	sendMessage(mpus);
+
+	if (! texture.isEmpty())
+		Database::setBlob(sha1(texture), texture);
+}
+
+void ServerHandler::requestBanList() {
+	MumbleProto::BanList mpbl;
+	mpbl.set_query(true);
+	sendMessage(mpbl);
+}
+
+void ServerHandler::requestUserList() {
+	MumbleProto::UserList mpul;
+	sendMessage(mpul);
+}
+
+void ServerHandler::requestACL(unsigned int channel) {
+	MumbleProto::ACL mpacl;
+	mpacl.set_channel_id(channel);
+	mpacl.set_query(true);
+	sendMessage(mpacl);
+}
+
+void ServerHandler::registerUser(unsigned int uiSession) {
+	MumbleProto::UserState mpus;
+	mpus.set_session(uiSession);
+	mpus.set_user_id(0);
+	sendMessage(mpus);
+}
+
+void ServerHandler::kickBanUser(unsigned int uiSession, const QString &reason, bool ban) {
+	MumbleProto::UserRemove mpur;
+	mpur.set_session(uiSession);
+	mpur.set_reason(u8(reason));
+	mpur.set_ban(ban);
+	sendMessage(mpur);
+}
+
+void ServerHandler::sendUserTextMessage(unsigned int uiSession, const QString &message_) {
+	MumbleProto::TextMessage mptm;
+	mptm.add_session(uiSession);
+	mptm.set_message(u8(message_));
+	sendMessage(mptm);
+}
+
+void ServerHandler::sendChannelTextMessage(unsigned int channel, const QString &message_, bool tree) {
+	MumbleProto::TextMessage mptm;
+	if (tree) {
+		mptm.add_tree_id(channel);
+	} else {
+		mptm.add_channel_id(channel);
+	}
+	mptm.set_message(u8(message_));
+	sendMessage(mptm);
+}
+
+void ServerHandler::setUserComment(unsigned int uiSession, const QString &comment) {
+	MumbleProto::UserState mpus;
+	mpus.set_session(uiSession);
+	mpus.set_comment(u8(comment));
+	sendMessage(mpus);
+}
+
+void ServerHandler::removeChannel(unsigned int channel) {
+	MumbleProto::ChannelRemove mpcr;
+	mpcr.set_channel_id(channel);
+	sendMessage(mpcr);
+}
+
+void ServerHandler::addChannelLink(unsigned int channel, unsigned int link) {
+	MumbleProto::ChannelState mpcs;
+	mpcs.set_channel_id(channel);
+	mpcs.add_links_add(link);
+	sendMessage(mpcs);
+}
+
+void ServerHandler::removeChannelLink(unsigned int channel, unsigned int link) {
+	MumbleProto::ChannelState mpcs;
+	mpcs.set_channel_id(channel);
+	mpcs.add_links_remove(link);
+	sendMessage(mpcs);
+}
+
+void ServerHandler::requestChannelPermissions(unsigned int channel) {
+	MumbleProto::PermissionQuery mppq;
+	mppq.set_channel_id(channel);
+	sendMessage(mppq);
+}
+
+void ServerHandler::setSelfMuteDeafState(bool mute, bool deaf) {
+	MumbleProto::UserState mpus;
+	mpus.set_self_mute(mute);
+	mpus.set_self_deaf(deaf);
+	sendMessage(mpus);
+}
