@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -164,7 +164,6 @@ Settings::Settings() {
 	bFirstTime = true;
 	ceExpand = ChannelsWithUsers;
 	ceChannelDrag = Ask;
-	bShowChatbar = true;
 	bMinimalView = false;
 	bHideFrame = false;
 	aotbAlwaysOnTop = OnTopNever;
@@ -174,6 +173,7 @@ Settings::Settings() {
 	bUsage = true;
 	bShowUserCount = false;
 	wlWindowLayout = LayoutClassic;
+	bShowContextMenuInMenuBar = false;
 
 	ssFilter = ShowReachable;
 
@@ -196,7 +196,6 @@ Settings::Settings() {
 	fAudioBloom = 0.5f;
 
 	bOverlayEnable = true;
-	bOverlayUserTextures=true;
 	osOverlay = All;
 	bOverlayAlwaysSelf = true;
 	fOverlayX = 0.8f;
@@ -232,6 +231,9 @@ Settings::Settings() {
 	iMaxImageHeight = 1024;
 	bSuppressIdentity = false;
 
+	// Accessibility
+	bHighContrast = false;
+
 #if defined(AUDIO_TEST)
 	lmLoopMode = Server;
 #else
@@ -376,7 +378,6 @@ void Settings::load() {
 
 	SAVELOAD(bOverlayEnable, "overlay/enable");
 	LOADENUM(osOverlay, "overlay/show");
-	SAVELOAD(bOverlayUserTextures, "overlay/usertextures");
 	SAVELOAD(bOverlayAlwaysSelf, "overlay/alwaysself");
 	SAVELOAD(fOverlayX, "overlay/x");
 	SAVELOAD(fOverlayY, "overlay/y");
@@ -414,7 +415,6 @@ void Settings::load() {
 	LOADENUM(ceChannelDrag, "ui/drag");
 	LOADENUM(aotbAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
-	SAVELOAD(bShowChatbar, "ui/shotchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
@@ -422,6 +422,7 @@ void Settings::load() {
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
+	SAVELOAD(qbaMinimalViewState, "ui/minimalviewstate");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
 	LOADENUM(wlWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
@@ -438,6 +439,10 @@ void Settings::load() {
 	SAVELOAD(bUsage, "ui/usage");
 	SAVELOAD(bShowUserCount, "ui/showusercount");
 	SAVELOAD(qsImagePath, "ui/imagepath");
+	SAVELOAD(bShowContextMenuInMenuBar, "ui/showcontextmenuinmenubar");
+	SAVELOAD(qbaConnectDialogGeometry, "ui/connect/geometry");
+	SAVELOAD(qbaConnectDialogHeader, "ui/connect/header");
+	SAVELOAD(bHighContrast, "ui/HighContrast");
 
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
@@ -561,7 +566,6 @@ void Settings::save() {
 
 	SAVELOAD(bOverlayEnable, "overlay/enable");
 	SAVELOAD(osOverlay, "overlay/show");
-	SAVELOAD(bOverlayUserTextures, "overlay/usertextures");
 	SAVELOAD(bOverlayAlwaysSelf, "overlay/alwaysself");
 	SAVELOAD(fOverlayX, "overlay/x");
 	SAVELOAD(fOverlayY, "overlay/y");
@@ -598,7 +602,6 @@ void Settings::save() {
 	SAVELOAD(ceChannelDrag, "ui/drag");
 	SAVELOAD(aotbAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
-	SAVELOAD(bShowChatbar, "ui/showchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
@@ -606,6 +609,7 @@ void Settings::save() {
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
+	SAVELOAD(qbaMinimalViewState, "ui/minimalviewstate");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
 	SAVELOAD(wlWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
@@ -620,6 +624,10 @@ void Settings::save() {
 	SAVELOAD(bUsage, "ui/usage");
 	SAVELOAD(bShowUserCount, "ui/showusercount");
 	SAVELOAD(qsImagePath, "ui/imagepath");
+	SAVELOAD(bShowContextMenuInMenuBar, "ui/showcontextmenuinmenubar");
+	SAVELOAD(qbaConnectDialogGeometry, "ui/connect/geometry");
+	SAVELOAD(qbaConnectDialogHeader, "ui/connect/header");
+	SAVELOAD(bHighContrast, "ui/HighContrast");
 
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
