@@ -1,4 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -30,69 +31,62 @@
 
 #include "TextMessage.h"
 
-TextMessage::TextMessage(QWidget *p) : QDialog(p) {
+TextMessage::TextMessage(QWidget *p, QString title, bool bChannel) : QDialog(p) {
 	setupUi(this);
-	qtbPreview->document()->setDefaultStyleSheet(qApp->styleSheet());
-	qteEdit->installEventFilter(this);
-	qteEdit->setFocus();
+	if (!bChannel)
+		qcbTreeMessage->setHidden(true);
+	setWindowTitle(title);
+	rteMessage->installEventFilter(this);
+	bTreeMessage = false;
 }
 
-void TextMessage::on_qcbRawMessage_stateChanged(int) {
-	on_qteEdit_textChanged();
+void TextMessage::on_qcbTreeMessage_stateChanged(int s) {
+	bTreeMessage = s == Qt::Checked ? true : false;
 }
 
-void TextMessage::on_qteEdit_textChanged() {
-	qsRep = qteEdit->toPlainText();
+QString TextMessage::autoFormat(QString qsPlain) {
+	QRegExp qr;
+	qr.setMinimal(true);
+	qr.setPatternSyntax(QRegExp::RegExp2);
+	qr.setCaseSensitivity(Qt::CaseInsensitive);
 
-	if (qcbRawMessage->isChecked()) {
-		qsRep = Qt::convertFromPlainText(qsRep);
-	}
+	qr.setPattern(QLatin1String("[\\r\\n]+"));
+	qsPlain.replace(qr, QLatin1String("<br />"));
 
-	if (! Qt::mightBeRichText(qsRep) && !qcbRawMessage->isChecked())  {
-		QRegExp qr;
-		qr.setMinimal(true);
-		qr.setPatternSyntax(QRegExp::RegExp2);
-		qr.setCaseSensitivity(Qt::CaseInsensitive);
+	qr.setPattern(QLatin1String("\\*(\\w+)\\*"));
+	qsPlain.replace(qr, QLatin1String("<b>\\1</b>"));
 
-		qr.setPattern(QLatin1String("[\\r\\n]+"));
-		qsRep.replace(qr, QLatin1String("<br />"));
+	qr.setPattern(QLatin1String("\"([^\"]+)\""));
+	qsPlain.replace(qr, QLatin1String("\"<i>\\1</i>\""));
 
-		qr.setPattern(QLatin1String("\\*(\\w+)\\*"));
-		qsRep.replace(qr, QLatin1String("<b>\\1</b>"));
+	qr.setPattern(QLatin1String("[a-z]+://[^ <$]*"));
+	qr.setMinimal(false);
 
-		qr.setPattern(QLatin1String("\"([^\"]+)\""));
-		qsRep.replace(qr, QLatin1String("\"<i>\\1</i>\""));
-
-		qr.setPattern(QLatin1String("[a-z]+://[^ <$]*"));
-		qr.setMinimal(false);
-
-		int idx = 0;
-		do {
-			idx = qr.indexIn(qsRep, idx);
-			if (idx >= 0) {
-				QString url = qr.capturedTexts().at(0);
-				QUrl u(url);
-				if (u.isValid()) {
-					int len = qr.matchedLength();
-					QString replacement = QString::fromLatin1("<a href=\"%1\">%1</a>").arg(url);
-					qsRep.replace(idx, len, replacement);
-					idx += replacement.length();
-				} else {
-					idx++;
-				}
+	int idx = 0;
+	do {
+		idx = qr.indexIn(qsPlain, idx);
+		if (idx >= 0) {
+			QString url = qr.capturedTexts().at(0);
+			QUrl u(url);
+			if (u.isValid()) {
+				int len = qr.matchedLength();
+				QString replacement = QString::fromLatin1("<a href=\"%1\">%1</a>").arg(url);
+				qsPlain.replace(idx, len, replacement);
+				idx += replacement.length();
+			} else {
+				idx++;
 			}
-		} while (idx >= 0);
-	}
-
-	qtbPreview->setHtml(qsRep);
+		}
+	} while (idx >= 0);
+	return qsPlain;
 }
 
 QString TextMessage::message() {
-	return qsRep;
+	return rteMessage->text();
 }
 
 bool TextMessage::eventFilter(QObject *obj, QEvent *evt) {
-	if (obj != qteEdit)
+	if (obj != rteMessage)
 		return false;
 	if (evt->type() == QEvent::KeyPress) {
 		QKeyEvent *keyEvent = static_cast<QKeyEvent *>(evt);
