@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -78,6 +78,7 @@ public:
 
 class UserModel : public QAbstractItemModel {
 		friend struct ModelItem;
+		friend class UserView;
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(UserModel)
@@ -92,6 +93,8 @@ class UserModel : public QAbstractItemModel {
 		QSet<Channel *> qsLinked;
 		QMap<QString, ClientUser *> qmHashes;
 
+		bool bClicked;
+
 		void recursiveClone(const ModelItem *old, ModelItem *item, QModelIndexList &from, QModelIndexList &to);
 		ModelItem *moveItem(ModelItem *oldparent, ModelItem *newparent, ModelItem *item);
 
@@ -132,11 +135,13 @@ class UserModel : public QAbstractItemModel {
 		void setHash(ClientUser *p, const QString &hash);
 		void setFriendName(ClientUser *p, const QString &name);
 		void setComment(ClientUser *p, const QString &comment);
+		void setCommentHash(ClientUser *p, const QByteArray &hash);
 		void seenComment(const QModelIndex &idx);
 
 		void moveUser(ClientUser *p, Channel *c);
 		void moveChannel(Channel *c, Channel *p);
 		void setComment(Channel *c, const QString &comment);
+		void setCommentHash(Channel *c, const QByteArray &hash);
 
 		void removeUser(ClientUser *p);
 		void removeChannel(Channel *c);
@@ -151,6 +156,9 @@ class UserModel : public QAbstractItemModel {
 		void collapseEmpty(Channel *c);
 
 		QVariant otherRoles(const QModelIndex &idx, int role) const;
+
+		unsigned int uiSessionComment;
+		int iChannelDescription;
 	public slots:
 		void userTalkingChanged();
 		void userMuteDeafChanged();
