@@ -52,6 +52,8 @@
 #include "VersionCheck.h"
 #include "NetworkConfig.h"
 #include "CrashReporter.h"
+#include "FileEngine.h"
+#include "SocketRPC.h"
 
 #ifdef BOOST_NO_EXCEPTIONS
 namespace boost {
@@ -138,6 +140,7 @@ int main(int argc, char **argv) {
 			_wputenv_s(L"PATH", buffout);
 		}
 	}
+#endif
 #endif
 
 	if (! bAllowMultiple) {
@@ -148,6 +151,10 @@ int main(int argc, char **argv) {
 			patch = 0;
 
 			QString version = url.queryItemValue(QLatin1String("version"));
+#ifndef USE_DBUS
+			QMap<QString, QVariant> param;
+			param.insert(QLatin1String("href"), url);
+#endif
 
 			QRegExp rx(QLatin1String("(\\d+)\\.(\\d+)\\.(\\d+)"));
 			if (rx.exactMatch(version)) {
@@ -157,10 +164,16 @@ int main(int argc, char **argv) {
 			}
 
 			if ((major == 1) && (minor == 1)) {
+				bool sent = false;
+#ifdef USE_DBUS
 				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble11x"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 
 				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
-				if (reply.type() == QDBusMessage::ReplyMessage) {
+				sent = (reply.type() == QDBusMessage::ReplyMessage);
+#else
+				sent = SocketRPC::send(QLatin1String("Mumble11x"), QLatin1String("url"), param);
+#endif
+				if (sent) {
 					return 0;
 				} else {
 					QString executable = a.applicationFilePath();
@@ -175,24 +188,51 @@ int main(int argc, char **argv) {
 					}
 				}
 			} else {
+				bool sent = false;
+#ifdef USE_DBUS
 				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 
 				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
-				if (reply.type() == QDBusMessage::ReplyMessage)
+				sent = (reply.type() == QDBusMessage::ReplyMessage);
+#else
+				sent = SocketRPC::send(QLatin1String("Mumble"), QLatin1String("url"), param);
+#endif
+				if (sent)
 					return 0;
 			}
 		} else {
+			bool sent = false;
+#ifdef USE_DBUS
 			QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 
 			QDBusMessage reply=qdbi.call(QLatin1String("focus"));
-			if (reply.type() == QDBusMessage::ReplyMessage)
+			sent = (reply.type() == QDBusMessage::ReplyMessage);
+#else
+			sent = SocketRPC::send(QLatin1String("Mumble"), QLatin1String("focus"));
+#endif
+			if (sent)
 				return 0;
+
 		}
 	}
-#endif
+
 	// Load preferences
 	g.s.load();
 
+	// Check wheter we need to enable accessbility features
+#ifdef Q_OS_WIN
+	// Only windows for now. Could not find any information on how to query this for osx or linux
+	{
+		HIGHCONTRAST hc;
+		hc.cbSize = sizeof(HIGHCONTRAST);
+		SystemParametersInfo(SPI_GETHIGHCONTRAST, sizeof(HIGHCONTRAST), &hc, 0);
+
+		if (hc.dwFlags & HCF_HIGHCONTRASTON)
+			g.s.bHighContrast = true;
+
+	}
+#endif
+
 	DeferInit::run_initializers();
 
 	if (! g.s.qsStyle.isEmpty()) {
@@ -293,12 +333,16 @@ int main(int argc, char **argv) {
 	QDBusConnection::sessionBus().registerService(QLatin1String("net.sourceforge.mumble.mumble"));
 #endif
 
+	SocketRPC *srpc = new SocketRPC(QLatin1String("Mumble"));
+
 	g.l->log(Log::Information, MainWindow::tr("Welcome to Mumble."));
 
 	// Plugins
 	g.p = new Plugins(NULL);
 	g.p->rescanPlugins();
 
+	MumbleFileEngineHandler *mfeh = new MumbleFileEngineHandler();
+
 	// And the start the last chosen audio system.
 	g.ai = AudioInputRegistrar::newFromChoice();
 	if (g.ai)
@@ -310,7 +354,7 @@ int main(int argc, char **argv) {
 	a.setQuitOnLastWindowClosed(false);
 
 	if (g.s.bFirstTime) {
-		if (QMessageBox::question(g.mw, MainWindow::tr("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
+		if (QMessageBox::question(g.mw, QLatin1String("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
 			AudioWizard *aw = new AudioWizard(g.mw);
 			aw->exec();
 			delete aw;
@@ -342,6 +386,9 @@ int main(int argc, char **argv) {
 		}
 	}
 
+	if (QDateTime::currentDateTime().daysTo(g.s.kpCertificate.first.first().expiryDate()) < 14)
+		g.l->log(Log::Warning, CertWizard::tr("<b>Certificate Expiry:</b> Your certificate is about to expire. You need to renew it, or you will no longer be able to connect to servers you are registered on."));
+
 	if (g.s.bUpdateCheck)
 		new VersionCheck(true, g.mw);
 
@@ -382,6 +429,11 @@ int main(int argc, char **argv) {
 	ao.reset();
 
 	g.sh->disconnect();
+
+	delete mfeh;
+
+	delete srpc;
+
 	delete g.sh;
 	delete g.mw;
 
