@@ -28,7 +28,6 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "Overlay.h"
 #include "MainWindow.h"
 #include "ServerHandler.h"
 #include "AudioInput.h"
@@ -45,7 +44,7 @@
 #include "VersionCheck.h"
 #include "NetworkConfig.h"
 #include "CrashReporter.h"
-
+#include "SocketRPC.h"
 
 #ifdef BOOST_NO_EXCEPTIONS
 namespace boost {
@@ -110,6 +109,16 @@ int main(int argc, char **argv) {
 			qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
 		return 0;
 	}
+#else
+	if (SocketRPC::send(QLatin1String("Mumble11x"), QLatin1String("focus"))) {
+		if (url.isValid()) {
+			QMap<QString, QVariant> param;
+			param.insert(QLatin1String("href"), url);
+			SocketRPC::send(QLatin1String("Mumble11x"), QLatin1String("url"), param);
+		}
+
+		return 0;
+	}
 #endif
 
 	QFile oldini(QString::fromLatin1("%1/mumble.ini").arg(a.applicationDirPath()));
@@ -196,6 +205,8 @@ int main(int argc, char **argv) {
 	delete cr;
 #endif
 
+	SocketRPC *srpc = new SocketRPC(QLatin1String("Mumble11x"));
+
 	// Initialize logger
 	g.l = new Log();
 
@@ -206,9 +217,6 @@ int main(int argc, char **argv) {
 	g.sh = new ServerHandler();
 	g.sh->moveToThread(g.sh);
 
-	g.o = new Overlay();
-	g.o->setActive(g.s.bOverlayEnable);
-
 	g.lcd = new LCD();
 
 	// Process any waiting events before initializing our MainWindow.
@@ -245,7 +253,7 @@ int main(int argc, char **argv) {
 	a.setQuitOnLastWindowClosed(false);
 
 	if (g.s.bFirstTime) {
-		if (QMessageBox::question(g.mw, MainWindow::tr("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
+		if (QMessageBox::question(g.mw, QLatin1String("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
 			AudioWizard *aw = new AudioWizard(g.mw);
 			aw->exec();
 			delete aw;
@@ -286,6 +294,9 @@ int main(int argc, char **argv) {
 	ao.reset();
 
 	g.sh->disconnect();
+
+	delete srpc;
+
 	delete g.sh;
 	delete g.mw;
 
@@ -295,9 +306,6 @@ int main(int argc, char **argv) {
 	delete g.db;
 	delete g.l;
 
-
-	delete g.o;
-
 	delete g.qs;
 
 	DeferInit::run_destroyers();
