@@ -37,9 +37,16 @@
 #include "ServerDB.h"
 #include "Connection.h"
 #include "Server.h"
+#include "ServerUser.h"
 #include "DBus.h"
 
 #define MSG_SETUP(st) \
+	if (uSource->sState != st) { \
+		return; \
+	} \
+	uSource->bwr.resetIdleSeconds()
+
+#define MSG_SETUP_NO_UNIDLE(st) \
 	if (uSource->sState != st) \
 		return
 
@@ -87,28 +94,6 @@
 		sendMessage(uSource, mppd); \
 	}
 
-static QString toPlainText(const QString &text) {
-	if (! text.contains(QLatin1Char('<')))
-		return text.simplified();
-
-	QXmlStreamReader qxsr(QString::fromLatin1("<document>%1</document>").arg(text));
-	QString qs;
-	while (! qxsr.atEnd()) {
-		switch (qxsr.readNext()) {
-			case QXmlStreamReader::Characters:
-				qs += qxsr.text();
-				break;
-			case QXmlStreamReader::EndElement:
-				if ((qxsr.name() == QLatin1String("br")) || (qxsr.name() == QLatin1String("p")))
-					qs += "\n";
-				break;
-			default:
-				break;
-		}
-	}
-	return qs.simplified();
-}
-
 void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg) {
 	if ((msg.tokens_size() > 0) || (uSource->sState == ServerUser::Authenticated)) {
 		QStringList qsl;
@@ -142,7 +127,7 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	MumbleProto::Reject_RejectType rtType = MumbleProto::Reject_RejectType_None;
 
 	if (! nameok && (uSource->iId == -1)) {
-		reason = "Invalid ServerUsername";
+		reason = "Invalid username";
 		rtType = MumbleProto::Reject_RejectType_InvalidUsername;
 	} else if (id==-1) {
 		reason = "Wrong password for user";
@@ -213,17 +198,6 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	mpcrypt.set_client_nonce(std::string(reinterpret_cast<const char *>(uSource->csCrypt.decrypt_iv), AES_BLOCK_SIZE));
 	sendMessage(uSource, mpcrypt);
 
-	int lchan = readLastChannel(uSource->iId);
-	if (lchan == 0)
-		lchan = iDefaultChan;
-	Channel *lc = qhChannels.value(lchan);
-	if (! lc)
-		lc = root;
-	else if (! hasPermission(uSource, lc, ChanACL::Enter))
-		lc = root;
-
-	userEnterChannel(uSource, lc, true);
-
 	if (msg.celt_versions_size() > 0) {
 		for (int i=0;i < msg.celt_versions_size(); ++i)
 			uSource->qlCodecs.append(msg.celt_versions(i));
@@ -256,10 +230,14 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 			mpcs.set_name(u8(qsRegName.isEmpty() ? QLatin1String("Root") : qsRegName));
 		else
 			mpcs.set_name(u8(c->qsName));
-		if (! c->qsDesc.isEmpty())
-			mpcs.set_description(u8(c->qsDesc));
+
 		mpcs.set_position(c->iPosition);
 
+		if ((uSource->uiVersion >= 0x010202) && ! c->qbaDescHash.isEmpty())
+			mpcs.set_description_hash(blob(c->qbaDescHash));
+		else if (! c->qsDesc.isEmpty())
+			mpcs.set_description(u8(c->qsDesc));
+
 		sendMessage(uSource, mpcs);
 
 		foreach(c, c->qlChannels)
@@ -281,19 +259,36 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	// Transmit user profile
 	MumbleProto::UserState mpus;
 
+	Channel *lc = qhChannels.value(readLastChannel(uSource->iId));
+	if (! lc || ! hasPermission(uSource, lc, ChanACL::Enter)) {
+		lc = qhChannels.value(iDefaultChan);
+		if (! lc || ! hasPermission(uSource, lc, ChanACL::Enter)) {
+			lc = root;
+		}
+	}
+
+	userEnterChannel(uSource, lc, mpus);
+
 	uSource->sState = ServerUser::Authenticated;
 	mpus.set_session(uSource->uiSession);
 	mpus.set_name(u8(uSource->qsName));
 	if (uSource->iId >= 0) {
-		uSource->qbaTexture = getUserTexture(uSource->iId);
 		mpus.set_user_id(uSource->iId);
-		if (! uSource->qbaTexture.isEmpty())
-			mpus.set_texture(std::string(uSource->qbaTexture.constData(), uSource->qbaTexture.size()));
+
+		hashAssign(uSource->qbaTexture, uSource->qbaTextureHash, getUserTexture(uSource->iId));
+
+		if (! uSource->qbaTextureHash.isEmpty())
+			mpus.set_texture_hash(blob(uSource->qbaTextureHash));
+		else if (! uSource->qbaTexture.isEmpty())
+			mpus.set_texture(blob(uSource->qbaTexture));
 
 		const QMap<int, QString> &info = getRegistration(uSource->iId);
 		if (info.contains(ServerDB::User_Comment)) {
-			uSource->qsComment = info.value(ServerDB::User_Comment);
-			mpus.set_comment(u8(uSource->qsComment));
+			hashAssign(uSource->qsComment, uSource->qbaCommentHash, info.value(ServerDB::User_Comment));
+			if (! uSource->qbaCommentHash.isEmpty())
+				mpus.set_comment_hash(blob(uSource->qbaCommentHash));
+			else if (! uSource->qsComment.isEmpty())
+				mpus.set_comment(u8(uSource->qsComment));
 		}
 	}
 	if (! uSource->qsHash.isEmpty())
@@ -301,22 +296,35 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	if (uSource->cChannel->iId != 0)
 		mpus.set_channel_id(uSource->cChannel->iId);
 
-	sendExcept(uSource, mpus);
+	sendAll(mpus, 0x010202);
+
+	if ((uSource->qbaTexture.length() >= 4) && (qFromBigEndian<unsigned int>(reinterpret_cast<const unsigned char *>(uSource->qbaTexture.constData())) == 600 * 60 * 4))
+		mpus.set_texture(blob(uSource->qbaTexture));
+	if (! uSource->qsComment.isEmpty())
+		mpus.set_comment(u8(uSource->qsComment));
+	sendAll(mpus, ~ 0x010202);
 
 	// Transmit other users profiles
 	foreach(ServerUser *u, qhUsers) {
 		if (u->sState != ServerUser::Authenticated)
 			continue;
 
+		if (u == uSource)
+			continue;
+
 		mpus.Clear();
 		mpus.set_session(u->uiSession);
 		mpus.set_name(u8(u->qsName));
-		if (u->iId >= 0) {
+		if (u->iId >= 0)
 			mpus.set_user_id(u->iId);
-			if (! u->qbaTexture.isEmpty())
-				mpus.set_texture(std::string(u->qbaTexture.constData(), u->qbaTexture.size()));
+		if (uSource->uiVersion >= 0x010202) {
+			if (! u->qbaTextureHash.isEmpty())
+				mpus.set_texture_hash(blob(u->qbaTextureHash));
+			else if (! u->qbaTexture.isEmpty())
+				mpus.set_texture(blob(u->qbaTexture));
+		} else if ((uSource->qbaTexture.length() >= 4) && (qFromBigEndian<unsigned int>(reinterpret_cast<const unsigned char *>(uSource->qbaTexture.constData())) == 600 * 60 * 4)) {
+			mpus.set_texture(blob(u->qbaTexture));
 		}
-
 		if (u->cChannel->iId != 0)
 			mpus.set_channel_id(u->cChannel->iId);
 		if (u->bDeaf)
@@ -329,7 +337,9 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 			mpus.set_self_deaf(true);
 		else if (u->bSelfMute)
 			mpus.set_self_mute(true);
-		if (! u->qsComment.isEmpty())
+		if ((uSource->uiVersion >= 0x010202) && ! u->qbaCommentHash.isEmpty())
+			mpus.set_comment_hash(blob(u->qbaCommentHash));
+		else if (! u->qsComment.isEmpty())
 			mpus.set_comment(u8(u->qsComment));
 		if (! u->qsHash.isEmpty())
 			mpus.set_hash(u8(u->qsHash));
@@ -343,7 +353,6 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	if (! qsWelcomeText.isEmpty())
 		mpss.set_welcome_text(u8(qsWelcomeText));
 	mpss.set_max_bandwidth(iMaxBandwidth);
-	mpss.set_allow_html(bAllowHTML);
 
 	if (uSource->iId == 0) {
 		mpss.set_permissions(ChanACL::All);
@@ -354,10 +363,16 @@ void Server::msgAuthenticate(ServerUser *uSource, MumbleProto::Authenticate &msg
 	}
 
 	sendMessage(uSource, mpss);
+
+	MumbleProto::ServerConfig mpsc;
+	mpsc.set_allow_html(bAllowHTML);
+	mpsc.set_message_length(iMaxTextMessageLength);
+	mpsc.set_image_message_length(iMaxImageMessageLength);
+	sendMessage(uSource, mpsc);
+
 	log(uSource, "Authenticated");
 
 	emit userConnected(uSource);
-	userEnterChannel(uSource, lc, false);
 }
 
 void Server::msgBanList(ServerUser *uSource, MumbleProto::BanList &msg) {
@@ -415,7 +430,7 @@ void Server::msgPermissionDenied(ServerUser *, MumbleProto::PermissionDenied &)
 }
 
 void Server::msgUDPTunnel(ServerUser *uSource, MumbleProto::UDPTunnel &msg) {
-	MSG_SETUP(ServerUser::Authenticated);
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
 
 	const std::string &str = msg.packet();
 	int len = static_cast<int>(str.length());
@@ -471,24 +486,39 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		}
 	}
 
+	QString comment;
+
 	if (msg.has_comment()) {
+		bool changed = false;
+		comment = u8(msg.comment());
 		if (uSource != pDstServerUser) {
 			if (! hasPermission(uSource, root, ChanACL::Move)) {
 				PERM_DENIED(uSource, root, ChanACL::Move);
 				return;
 			}
-			if (msg.comment().length() > 0) {
+			if (comment.length() > 0) {
 				PERM_DENIED_TYPE(TextTooLong);
 				return;
 			}
 		}
 
-		if (iMaxTextMessageLength > 0 && (msg.comment().length() > static_cast<unsigned int>(iMaxTextMessageLength))) {
+
+		if (! isTextAllowed(comment, changed)) {
+			PERM_DENIED_TYPE(TextTooLong);
+			return;
+		}
+		if (changed)
+			msg.set_comment(u8(comment));
+	}
+
+	if (msg.has_texture()) {
+		if (iMaxImageMessageLength && (msg.texture().length() > iMaxImageMessageLength)) {
 			PERM_DENIED_TYPE(TextTooLong);
 			return;
 		}
 	}
 
+
 	if (msg.has_user_id()) {
 		ChanACL::Perm p = (uSource == pDstServerUser) ? ChanACL::SelfRegister : ChanACL::Register;
 		if ((pDstServerUser->iId >= 0) || ! hasPermission(uSource, root, p)) {
@@ -507,10 +537,12 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 	// Permission checks done. Now enact this.
 
 	if (msg.has_texture()) {
+		QByteArray qba = blob(msg.texture());
 		if (uSource->iId > 0) {
-			QByteArray qba = QByteArray(msg.texture().data(), static_cast<int>(msg.texture().size()));
 			if (! setTexture(uSource->iId, qba))
 				return;
+		} else {
+			hashAssign(uSource->qbaTexture, uSource->qbaTextureHash, qba);
 		}
 	}
 
@@ -538,13 +570,8 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		uSource->qsIdentity = u8(msg.plugin_identity());
 	}
 
-	if (msg.has_comment()) {
-		pDstServerUser->qsComment = u8(msg.comment());
-
-		if (! bAllowHTML) {
-			pDstServerUser->qsComment = toPlainText(pDstServerUser->qsComment);
-			msg.set_comment(u8(pDstServerUser->qsComment));
-		}
+	if (! comment.isNull()) {
+		hashAssign(pDstServerUser->qsComment, pDstServerUser->qbaCommentHash, comment);
 
 		if (pDstServerUser->iId >= 0) {
 			QMap<int, QString> info;
@@ -553,12 +580,6 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		}
 	}
 
-	if (msg.has_channel_id()) {
-		Channel *c = qhChannels.value(msg.channel_id());
-
-		userEnterChannel(pDstServerUser, c);
-		log(uSource, QString("Moved %1 to %2").arg(QString(*pDstServerUser), QString(*c)));
-	}
 
 
 	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress()) {
@@ -583,6 +604,13 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		        QString::number(pDstServerUser->bSuppress)));
 	}
 
+	if (msg.has_channel_id()) {
+		Channel *c = qhChannels.value(msg.channel_id());
+
+		userEnterChannel(pDstServerUser, c, msg);
+		log(uSource, QString("Moved %1 to %2").arg(QString(*pDstServerUser), QString(*c)));
+	}
+
 	if (msg.has_user_id()) {
 		QMap<int, QString> info;
 
@@ -599,8 +627,24 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 		}
 	}
 
-	if (! bNoBroadcast)
-		sendAll(msg);
+	if (! bNoBroadcast) {
+		if (msg.has_texture() && (pDstServerUser->qbaTexture.length() >= 4) && (qFromBigEndian<unsigned int>(reinterpret_cast<const unsigned char *>(pDstServerUser->qbaTexture.constData())) != 600 * 60 * 4)) {
+			msg.clear_texture();
+			sendAll(msg, ~ 0x010202);
+			msg.set_texture(blob(pDstServerUser->qbaTexture));
+		} else {
+			sendAll(msg, ~ 0x010202);
+		}
+		if (msg.has_texture() && ! pDstServerUser->qbaTextureHash.isEmpty()) {
+			msg.clear_texture();
+			msg.set_texture_hash(blob(pDstServerUser->qbaTextureHash));
+		}
+		if (msg.has_comment() && ! pDstServerUser->qbaCommentHash.isEmpty()) {
+			msg.clear_comment();
+			msg.set_comment_hash(blob(pDstServerUser->qbaCommentHash));
+		}
+		sendAll(msg, 0x010202);
+	}
 
 	emit userStateChanged(pDstServerUser);
 }
@@ -667,15 +711,14 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 	QString qsName;
 	QString qsDesc;
 	if (msg.has_description()) {
-		if (iMaxTextMessageLength > 0 && (msg.description().length() > static_cast<unsigned int>(iMaxTextMessageLength))) {
+		qsDesc = u8(msg.description());
+		bool changed = false;
+		if (! isTextAllowed(qsDesc, changed)) {
 			PERM_DENIED_TYPE(TextTooLong);
 			return;
 		}
-		qsDesc = u8(msg.description());
-		if (! bAllowHTML) {
-			qsDesc = toPlainText(qsDesc);
+		if (changed)
 			msg.set_description(u8(qsDesc));
-		}
 	}
 
 	if (msg.has_name()) {
@@ -728,7 +771,7 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 		}
 
 		c = addChannel(p, qsName, msg.temporary(), msg.position());
-		c->qsDesc = qsDesc;
+		hashAssign(c->qsDesc, c->qbaDescHash, qsDesc);
 
 		if (uSource->iId >= 0) {
 			Group *g = new Group(c, "admin");
@@ -753,15 +796,22 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 		msg.set_channel_id(c->iId);
 		log(uSource, QString("Added channel %1 under %2").arg(QString(*c), QString(*p)));
 		emit channelCreated(c);
-		sendAll(msg);
+
+		sendAll(msg, ~ 0x010202);
+		if (! c->qbaDescHash.isEmpty()) {
+			msg.clear_description();
+			msg.set_description_hash(blob(c->qbaDescHash));
+		}
+		sendAll(msg, 0x010202);
 
 		if (c->bTemporary) {
 			// If a temporary channel has been created move the creator right in there
 			MumbleProto::UserState mpus;
 			mpus.set_session(uSource->uiSession);
 			mpus.set_channel_id(c->iId);
+			userEnterChannel(uSource, c, mpus);
 			sendAll(mpus);
-			userEnterChannel(uSource, c);
+			emit userStateChanged(uSource);
 		}
 	} else {
 		// The message is related to an existing channel c so check if the user is allowed to modify it
@@ -870,7 +920,7 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 			c->qsName = qsName;
 		}
 		if (! qsDesc.isNull())
-			c->qsDesc = qsDesc;
+			hashAssign(c->qsDesc, c->qbaDescHash, qsDesc);
 
 		if (msg.has_position())
 			c->iPosition = msg.position();
@@ -884,7 +934,13 @@ void Server::msgChannelState(ServerUser *uSource, MumbleProto::ChannelState &msg
 
 		updateChannel(c);
 		emit channelStateChanged(c);
-		sendAll(msg);
+
+		sendAll(msg, ~ 0x010202);
+		if (msg.has_description() && ! c->qbaDescHash.isEmpty()) {
+			msg.clear_description();
+			msg.set_description_hash(blob(c->qbaDescHash));
+		}
+		sendAll(msg, 0x010202);
 	}
 }
 
@@ -912,17 +968,17 @@ void Server::msgTextMessage(ServerUser *uSource, MumbleProto::TextMessage &msg)
 	QSet<ServerUser *> users;
 	QQueue<Channel *> q;
 
-	if (iMaxTextMessageLength > 0 && (msg.message().length() > static_cast<unsigned int>(iMaxTextMessageLength))) {
+	QString text = u8(msg.message());
+	bool changed = false;
+
+	if (! isTextAllowed(text, changed)) {
 		PERM_DENIED_TYPE(TextTooLong);
 		return;
 	}
-
-	if (! bAllowHTML) {
-		QString plain = toPlainText(u8(msg.message()));
-		if (plain.isEmpty())
-			return;
-		msg.set_message(u8(plain));
-	}
+	if (text.isEmpty())
+		return;
+	if (changed)
+		msg.set_message(u8(text));
 
 	{
 		char m[29] = {0117, 0160, 0145, 0156, 040, 0164, 0150, 0145, 040, 0160, 0157, 0144, 040, 0142, 0141, 0171, 040, 0144, 0157, 0157, 0162, 0163, 054, 040, 0110, 0101, 0114, 056, 0};
@@ -1179,7 +1235,7 @@ void Server::msgQueryUsers(ServerUser *uSource, MumbleProto::QueryUsers &msg) {
 }
 
 void Server::msgPing(ServerUser *uSource, MumbleProto::Ping &msg) {
-	MSG_SETUP(ServerUser::Authenticated);
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
 	CryptState &cs=uSource->csCrypt;
 
 	cs.uiRemoteGood = msg.good();
@@ -1207,7 +1263,7 @@ void Server::msgPing(ServerUser *uSource, MumbleProto::Ping &msg) {
 }
 
 void Server::msgCryptSetup(ServerUser *uSource, MumbleProto::CryptSetup &msg) {
-	MSG_SETUP(ServerUser::Authenticated);
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
 	if (! msg.has_client_nonce()) {
 		log(uSource, "Requested crypt-nonce resync");
 		msg.set_server_nonce(std::string(reinterpret_cast<const char *>(uSource->csCrypt.encrypt_iv), AES_BLOCK_SIZE));
@@ -1304,7 +1360,7 @@ void Server::msgUserList(ServerUser *uSource, MumbleProto::UserList &msg) {
 }
 
 void Server::msgVoiceTarget(ServerUser *uSource, MumbleProto::VoiceTarget &msg) {
-	MSG_SETUP(ServerUser::Authenticated);
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
 
 	int target = msg.id();
 	if ((target < 1) || (target >= 0x1f))
@@ -1347,7 +1403,7 @@ void Server::msgVoiceTarget(ServerUser *uSource, MumbleProto::VoiceTarget &msg)
 }
 
 void Server::msgPermissionQuery(ServerUser *uSource, MumbleProto::PermissionQuery &msg) {
-	MSG_SETUP(ServerUser::Authenticated);
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
 
 	Channel *c = qhChannels.value(msg.channel_id());
 	if (!c)
@@ -1358,3 +1414,129 @@ void Server::msgPermissionQuery(ServerUser *uSource, MumbleProto::PermissionQuer
 
 void Server::msgCodecVersion(ServerUser *, MumbleProto::CodecVersion &) {
 }
+
+void Server::msgUserStats(ServerUser*uSource, MumbleProto::UserStats &msg) {
+	MSG_SETUP_NO_UNIDLE(ServerUser::Authenticated);
+	VICTIM_SETUP;
+	const CryptState &cs = pDstServerUser->csCrypt;
+	const BandwidthRecord &bwr = pDstServerUser->bwr;
+	const QList<QSslCertificate> &certs = pDstServerUser->peerCertificateChain();
+	MumbleProto::UserStats_Stats *mpusss;
+	MumbleProto::Version *mpv;
+
+	bool extend = (uSource == pDstServerUser) || hasPermission(uSource, qhChannels.value(0), ChanACL::Register);
+
+	if (! extend && ! hasPermission(uSource, pDstServerUser->cChannel, ChanACL::Enter)) {
+		PERM_DENIED(uSource, pDstServerUser->cChannel, ChanACL::Enter);
+		return;
+	}
+
+	bool details = extend;
+	bool local = extend || (pDstServerUser->cChannel == uSource->cChannel);
+
+	if (msg.stats_only())
+		details = false;
+
+	msg.Clear();
+	msg.set_session(pDstServerUser->uiSession);
+
+	if (details) {
+		foreach(const QSslCertificate &cert, certs) {
+			const QByteArray &der = cert.toDer();
+			msg.add_certificates(blob(der));
+		}
+		msg.set_strong_certificate(pDstServerUser->bVerified);
+	}
+
+	if (local) {
+		mpusss = msg.mutable_from_client();
+		mpusss->set_good(cs.uiGood);
+		mpusss->set_late(cs.uiLate);
+		mpusss->set_lost(cs.uiLost);
+		mpusss->set_resync(cs.uiResync);
+
+		mpusss = msg.mutable_from_server();
+		mpusss->set_good(cs.uiRemoteGood);
+		mpusss->set_late(cs.uiRemoteLate);
+		mpusss->set_lost(cs.uiRemoteLost);
+		mpusss->set_resync(cs.uiRemoteResync);
+	}
+
+	msg.set_udp_packets(pDstServerUser->uiUDPPackets);
+	msg.set_tcp_packets(pDstServerUser->uiTCPPackets);
+	msg.set_udp_ping_avg(pDstServerUser->dUDPPingAvg);
+	msg.set_udp_ping_var(pDstServerUser->dUDPPingVar);
+	msg.set_tcp_ping_avg(pDstServerUser->dTCPPingAvg);
+	msg.set_tcp_ping_var(pDstServerUser->dTCPPingVar);
+
+	if (details) {
+		mpv = msg.mutable_version();
+		if (pDstServerUser->uiVersion)
+			mpv->set_version(pDstServerUser->uiVersion);
+		if (! pDstServerUser->qsRelease.isEmpty())
+			mpv->set_release(u8(pDstServerUser->qsRelease));
+		if (! pDstServerUser->qsOS.isEmpty()) {
+			mpv->set_os(u8(pDstServerUser->qsOS));
+			if (! pDstServerUser->qsOSVersion.isEmpty())
+				mpv->set_os_version(u8(pDstServerUser->qsOSVersion));
+		}
+
+		foreach(int v, pDstServerUser->qlCodecs)
+			msg.add_celt_versions(v);
+
+		msg.set_address(pDstServerUser->haAddress.toStdString());
+	}
+
+	if (local)
+		msg.set_bandwidth(bwr.bandwidth());
+	msg.set_onlinesecs(bwr.onlineSeconds());
+	if (local)
+		msg.set_idlesecs(bwr.idleSeconds());
+
+	sendMessage(uSource, msg);
+}
+
+void Server::msgRequestBlob(ServerUser *uSource, MumbleProto::RequestBlob &msg) {
+	int ntextures = msg.session_texture_size();
+	int ncomments = msg.session_comment_size();
+	int ndescriptions = msg.channel_description_size();
+
+	if (ndescriptions) {
+		MumbleProto::ChannelState mpcs;
+		for (int i=0;i<ndescriptions;++i) {
+			int id = msg.channel_description(i);
+			Channel *c = qhChannels.value(id);
+			if (c && ! c->qsDesc.isEmpty()) {
+				mpcs.set_channel_id(id);
+				mpcs.set_description(u8(c->qsDesc));
+				sendMessage(uSource, mpcs);
+			}
+		}
+	}
+	if (ntextures || ncomments) {
+		MumbleProto::UserState mpus;
+		for (int i=0;i<ntextures;++i) {
+			int session = msg.session_texture(i);
+			ServerUser *su = qhUsers.value(session);
+			if (su && ! su->qbaTexture.isEmpty()) {
+				mpus.set_session(session);
+				mpus.set_texture(blob(su->qbaTexture));
+				sendMessage(uSource, mpus);
+			}
+		}
+		if (ntextures)
+			mpus.clear_texture();
+		for (int i=0;i<ncomments;++i) {
+			int session = msg.session_comment(i);
+			ServerUser *su = qhUsers.value(session);
+			if (su && ! su->qsComment.isEmpty()) {
+				mpus.set_session(session);
+				mpus.set_comment(u8(su->qsComment));
+				sendMessage(uSource, mpus);
+			}
+		}
+	}
+}
+
+void Server::msgServerConfig(ServerUser *, MumbleProto::ServerConfig &) {
+}
