@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -48,6 +48,7 @@ MetaParams::MetaParams() {
 	iMaxUsers = 1000;
 	iMaxUsersPerChannel = 0;
 	iMaxTextMessageLength = 5000;
+	iMaxImageMessageLength = 131072;
 	bAllowHTML = true;
 	iDefaultChan = 0;
 	qsWelcomeText = QString("Welcome to this server");
@@ -215,6 +216,7 @@ void MetaParams::read(QString fname) {
 	usPort = static_cast<unsigned short>(qsSettings->value("port", usPort).toUInt());
 	iTimeout = qsSettings->value("timeout", iTimeout).toInt();
 	iMaxTextMessageLength = qsSettings->value("textmessagelength", iMaxTextMessageLength).toInt();
+	iMaxImageMessageLength = qsSettings->value("imagemessagelength", iMaxImageMessageLength).toInt();
 	bAllowHTML = qsSettings->value("allowhtml", bAllowHTML).toBool();
 	iMaxBandwidth = qsSettings->value("bandwidth", iMaxBandwidth).toInt();
 	iDefaultChan = qsSettings->value("defaultchannel", iDefaultChan).toInt();
@@ -234,6 +236,7 @@ void MetaParams::read(QString fname) {
 	iDBPort = qsSettings->value("dbPort", iDBPort).toInt();
 
 	qsIceEndpoint = qsSettings->value("ice", qsIceEndpoint).toString();
+	qsIceSecret = qsSettings->value("icesecret", qsIceSecret).toString();
 
 	iLogDays = qsSettings->value("logdays", iLogDays).toInt();
 
@@ -461,10 +464,18 @@ bool Meta::boot(int srvnum) {
 
 	struct rlimit r;
 	if (getrlimit(RLIMIT_NOFILE, &r) == 0) {
-		if (r.rlim_cur < r.rlim_max) {
-			r.rlim_cur = r.rlim_max;
-			setrlimit(RLIMIT_NOFILE, &r);
-			getrlimit(RLIMIT_NOFILE, &r);
+		if (r.rlim_cur < sockets) {
+			r.rlim_cur = sockets;
+			if (r.rlim_max < sockets)
+				r.rlim_max = sockets;
+			if (setrlimit(RLIMIT_NOFILE, &r) != 0) {
+				getrlimit(RLIMIT_NOFILE, &r);
+				if (r.rlim_cur < r.rlim_max) {
+					r.rlim_cur = r.rlim_max;
+					setrlimit(RLIMIT_NOFILE, &r);
+					getrlimit(RLIMIT_NOFILE, &r);
+				}
+			}
 		}
 		if (r.rlim_cur < sockets)
 			qCritical("Current booted servers require minimum %d file descriptors when all slots are full, but only %d file descriptors are allowed for this process. Your server will crash and burn; read the FAQ for details.", sockets, r.rlim_cur);
