@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -47,6 +47,7 @@ struct MetaParams {
 	int iMaxUsersPerChannel;
 	int iDefaultChan;
 	int iMaxTextMessageLength;
+	int iMaxImageMessageLength;
 	bool bAllowHTML;
 	QString qsPassword;
 	QString qsWelcomeText;
@@ -76,6 +77,7 @@ struct MetaParams {
 	QString qsLogfile;
 	QString qsPid;
 	QString qsIceEndpoint;
+	QString qsIceSecret;
 
 	QString qsRegName;
 	QString qsRegPassword;
@@ -111,6 +113,7 @@ class Meta : public QObject {
 		QHash<QHostAddress, QList<Timer> > qhAttempts;
 		QHash<QHostAddress, Timer> qhBans;
 		QString qsOS, qsOSVersion;
+		Timer tUptime;
 
 #ifdef Q_OS_WIN
 		static HANDLE hQoS;
