@@ -33,6 +33,7 @@
 #include <IceUtil/IceUtil.h>
 #include "Meta.h"
 #include "Server.h"
+#include "ServerUser.h"
 #include "ServerDB.h"
 #include "User.h"
 #include "Channel.h"
@@ -198,12 +199,18 @@ MurmurIce::MurmurIce() {
 	for (i=props.begin(); i != props.end(); ++i) {
 		ipp->setProperty((*i).first, (*i).second);
 	}
+	ipp->setProperty("Ice.ImplicitContext", "Shared");
 
 	Ice::InitializationData idd;
 	idd.properties = ipp;
 
 	try {
 		communicator = Ice::initialize(idd);
+		if (! meta->mp.qsIceSecret.isEmpty()) {
+			::Ice::ImplicitContextPtr impl = communicator->getImplicitContext();
+			if (impl)
+				impl->put("secret", u8(meta->mp.qsIceSecret));
+		}
 		adapter = communicator->createObjectAdapterWithEndpoints("Murmur", qPrintable(meta->mp.qsIceEndpoint));
 		MetaPtr m = new MetaI;
 		MetaPrx mprx = MetaPrx::uncheckedCast(adapter->add(m, communicator->stringToIdentity("Meta")));
@@ -1244,6 +1251,14 @@ static void impl_Server_updateRegistration(const ::Murmur::AMD_Server_updateRegi
 		cb->ice_exception(InvalidUserException());
 		return;
 	}
+
+	if (info.contains(ServerDB::User_Comment)) {
+		foreach(ServerUser *u, server->qhUsers) {
+			if (u->iId == id)
+				server->setUserState(u, u->cChannel, u->bMute, u->bDeaf, u->bSuppress, info.value(ServerDB::User_Comment));
+		}
+	}
+
 	cb->ice_response();
 }
 
@@ -1312,10 +1327,30 @@ static void impl_Server_setTexture(const ::Murmur::AMD_Server_setTexturePtr cb,
 	char *ptr = qba.data();
 	for (unsigned int i=0;i<tex.size();++i)
 		ptr[i] = tex[i];
-	if (! server->setTexture(userid, qba))
+	if (! server->setTexture(userid, qba)) {
 		cb->ice_exception(InvalidTextureException());
-	else
+	} else {
+		ServerUser *user = server->qhUsers.value(userid);
+		if (user) {
+			MumbleProto::UserState mpus;
+			mpus.set_session(user->uiSession);
+			mpus.set_texture(blob(user->qbaTexture));
+
+			server->sendAll(mpus, ~0x010202);
+			if (! user->qbaTextureHash.isEmpty()) {
+				mpus.clear_texture();
+				mpus.set_texture_hash(blob(user->qbaTextureHash));
+			}
+			server->sendAll(mpus, 0x010202);
+		}
+
 		cb->ice_response();
+	}
+}
+
+static void impl_Server_getUptime(const ::Murmur::AMD_Server_getUptimePtr cb, int server_id) {
+	NEED_SERVER;
+	cb->ice_response(static_cast<int>(server->tUptime.elapsed()/1000000LL));
 }
 
 static void impl_Server_addUserToGroup(const ::Murmur::AMD_Server_addUserToGroupPtr cb, int server_id, ::Ice::Int channelid,  ::Ice::Int session,  const ::std::string& group) {
@@ -1441,4 +1476,8 @@ static void impl_Meta_removeCallback(const Murmur::AMD_Meta_removeCallbackPtr cb
 	}
 }
 
+static void impl_Meta_getUptime(const ::Murmur::AMD_Meta_getUptimePtr cb, const Ice::ObjectAdapterPtr) {
+	cb->ice_response(static_cast<int>(meta->tUptime.elapsed()/1000000LL));
+}
+
 #include "MurmurIceWrapper.cpp"
