@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -64,12 +64,12 @@ class MurmurIce : public QObject {
 		void started(Server *);
 		void stopped(Server *);
 
-		void authenticateSlot(int &res, QString &uname, const QString &pw);
-		void registerUserSlot(int &res, const QMap<QString, QString> &);
+		void authenticateSlot(int &res, QString &uname, const QList<QSslCertificate> &certlist, const QString &certhash, bool certstrong, const QString &pw);
+		void registerUserSlot(int &res, const QMap<int, QString> &);
 		void unregisterUserSlot(int &res, int id);
 		void getRegisteredUsersSlot(const QString &filter, QMap<int, QString> &res);
-		void getRegistrationSlot(int &, int, QMap<QString, QString> &);
-		void setInfoSlot(int &, int, const QMap<QString, QString> &);
+		void getRegistrationSlot(int &, int, QMap<int, QString> &);
+		void setInfoSlot(int &, int, const QMap<int, QString> &);
 		void setTextureSlot(int &res, int id, const QByteArray &texture);
 		void nameToIdSlot(int &res, const QString &name);
 		void idToNameSlot(QString &res, int id);
