@@ -42,13 +42,10 @@
 #include "DBus.h"
 #include "Meta.h"
 #include "PacketDataStream.h"
+#include "ServerUser.h"
 
 #define UDP_PACKET_SIZE 1024
 
-uint qHash(const Peer &p) {
-	return p.first ^ p.second;
-}
-
 LogEmitter::LogEmitter(QObject *p) : QObject(p) {
 };
 
@@ -56,6 +53,14 @@ void LogEmitter::addLogEntry(const QString &msg) {
 	emit newLogEntry(msg);
 };
 
+ExecEvent::ExecEvent(boost::function<void ()> f) : QEvent(static_cast<QEvent::Type>(EXEC_QEVENT)) {
+	func = f;
+}
+
+void ExecEvent::execute() {
+	func();
+}
+
 SslServer::SslServer(QObject *p) : QTcpServer(p) {
 }
 
@@ -71,23 +76,6 @@ QSslSocket *SslServer::nextPendingSSLConnection() {
 	return qlSockets.takeFirst();
 }
 
-ServerUser::ServerUser(Server *p, QSslSocket *socket) : Connection(p, socket), User() {
-	sState = ServerUser::Connected;
-	sUdpSocket = INVALID_SOCKET;
-
-	memset(&saiUdpAddress, 0, sizeof(saiUdpAddress));
-
-	bUdp = true;
-	uiVersion = 0;
-	bVerified = true;
-	iLastPermissionCheck = -1;
-}
-
-
-ServerUser::operator const QString() const {
-	return QString::fromLatin1("%1:%2(%3)").arg(qsName).arg(uiSession).arg(iId);
-}
-
 Server::Server(int snum, QObject *p) : QThread(p) {
 	bValid = true;
 	iServerNum = snum;
@@ -105,6 +93,8 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 	iCodecAlpha = iCodecBeta = 0;
 	bPreferAlpha = false;
 
+	qnamNetwork = NULL;
+
 	readParams();
 	initialize();
 
@@ -114,7 +104,7 @@ Server::Server(int snum, QObject *p) : QThread(p) {
 		connect(ss, SIGNAL(newConnection()), this, SLOT(newClient()), Qt::QueuedConnection);
 
 		if (! ss->listen(qha, usPort)) {
-			log(QString("Server: TCP Listen on %1 failed").arg(addressToString(qha,usPort)));
+			log(QString("Server: TCP Listen on %1 failed: %2").arg(addressToString(qha,usPort), ss->errorString()));
 			bValid = false;
 		} else {
 			log(QString("Server listening on %1").arg(addressToString(qha,usPort)));
@@ -305,6 +295,7 @@ void Server::readParams() {
 	iMaxUsers = Meta::mp.iMaxUsers;
 	iMaxUsersPerChannel = Meta::mp.iMaxUsersPerChannel;
 	iMaxTextMessageLength = Meta::mp.iMaxTextMessageLength;
+	iMaxImageMessageLength = Meta::mp.iMaxImageMessageLength;
 	bAllowHTML = Meta::mp.bAllowHTML;
 	iDefaultChan = Meta::mp.iDefaultChan;
 	qsWelcomeText = Meta::mp.qsWelcomeText;
@@ -353,6 +344,7 @@ void Server::readParams() {
 	iMaxUsers = getConf("users", iMaxUsers).toInt();
 	iMaxUsersPerChannel = getConf("usersperchannel", iMaxUsersPerChannel).toInt();
 	iMaxTextMessageLength = getConf("textmessagelength", iMaxTextMessageLength).toInt();
+	iMaxImageMessageLength = getConf("imagemessagelength", iMaxImageMessageLength).toInt();
 	bAllowHTML = getConf("allowhtml", bAllowHTML).toBool();
 	iDefaultChan = getConf("defaultchannel", iDefaultChan).toInt();
 	qsWelcomeText = getConf("welcometext", qsWelcomeText).toString();
@@ -376,23 +368,66 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		qsPassword = !v.isNull() ? v : Meta::mp.qsPassword;
 	else if (key == "timeout")
 		iTimeout = i ? i : Meta::mp.iTimeout;
-	else if (key == "bandwidth")
-		iMaxBandwidth = i ? i : Meta::mp.iMaxBandwidth;
-	else if (key == "users")
+	else if (key == "bandwidth") {
+		int length = i ? i : Meta::mp.iMaxBandwidth;
+		if (length != iMaxBandwidth) {
+			iMaxBandwidth = length;
+			MumbleProto::ServerConfig mpsc;
+			mpsc.set_max_bandwidth(length);
+			sendAll(mpsc);
+		}
+	} else if (key == "users")
 		iMaxUsers = i ? i : Meta::mp.iMaxUsers;
 	else if (key == "usersperchannel")
 		iMaxUsersPerChannel = i ? i : Meta::mp.iMaxUsersPerChannel;
-	else if (key == "textmessagelength")
-		iMaxTextMessageLength = i ? i : Meta::mp.iMaxTextMessageLength;
-	else if (key == "allowhtml")
-		bAllowHTML = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bAllowHTML;
-	else if (key == "defaultchannel")
+	else if (key == "textmessagelength") {
+		int length = i ? i : Meta::mp.iMaxTextMessageLength;
+		if (length != iMaxTextMessageLength) {
+			iMaxTextMessageLength = length;
+			MumbleProto::ServerConfig mpsc;
+			mpsc.set_message_length(length);
+			sendAll(mpsc);
+		}
+	} else if (key == "imagemessagelength") {
+		int length = i ? i : Meta::mp.iMaxImageMessageLength;
+		if (length != iMaxImageMessageLength) {
+			iMaxImageMessageLength = length;
+			MumbleProto::ServerConfig mpsc;
+			mpsc.set_image_message_length(length);
+			sendAll(mpsc);
+		}
+	} else if (key == "allowhtml") {
+		bool allow = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bAllowHTML;
+		if (allow != bAllowHTML) {
+			bAllowHTML = allow;
+			MumbleProto::ServerConfig mpsc;
+			mpsc.set_allow_html(bAllowHTML);
+			sendAll(mpsc);
+		}
+	} else if (key == "defaultchannel")
 		iDefaultChan = i ? i : Meta::mp.iDefaultChan;
-	else if (key == "welcometext")
-		qsWelcomeText = !v.isNull() ? v : Meta::mp.qsWelcomeText;
-	else if (key == "registername")
-		qsRegName = !v.isNull() ? v : Meta::mp.qsRegName;
-	else if (key == "registerpassword")
+	else if (key == "welcometext") {
+		QString text = !v.isNull() ? v : Meta::mp.qsWelcomeText;
+		if (text != qsWelcomeText) {
+			qsWelcomeText = text;
+			if (! qsWelcomeText.isEmpty()) {
+				MumbleProto::ServerConfig mpsc;
+				mpsc.set_welcome_text(u8(qsWelcomeText));
+				sendAll(mpsc);
+			}
+		}
+	} else if (key == "registername") {
+		QString text = !v.isNull() ? v : Meta::mp.qsRegName;
+		if (text != qsRegName) {
+			qsRegName = text;
+			if (! qsRegName.isEmpty()) {
+				MumbleProto::ChannelState mpcs;
+				mpcs.set_channel_id(0);
+				mpcs.set_name(u8(qsRegName));
+				sendAll(mpcs);
+			}
+		}
+	} else if (key == "registerpassword")
 		qsRegPassword = !v.isNull() ? v : Meta::mp.qsRegPassword;
 	else if (key == "registerhostname")
 		qsRegHost = !v.isNull() ? v : Meta::mp.qsRegHost;
@@ -435,65 +470,6 @@ void Server::removeBonjour() {
 }
 #endif
 
-BandwidthRecord::BandwidthRecord() {
-	iRecNum = 0;
-	iSum = 0;
-	for (int i=0;i<N_BANDWIDTH_SLOTS;i++)
-		a_iBW[i] = 0;
-}
-
-bool BandwidthRecord::addFrame(int size, int maxpersec) {
-	quint64 elapsed = a_qtWhen[iRecNum].elapsed();
-
-	if (elapsed == 0)
-		return false;
-
-	int nsum = iSum-a_iBW[iRecNum]+size;
-	int bw = static_cast<int>((nsum * 1000000LL) / elapsed);
-
-	if (bw > maxpersec)
-		return false;
-
-	a_iBW[iRecNum] = static_cast<unsigned short>(size);
-	a_qtWhen[iRecNum].restart();
-
-	iSum = nsum;
-
-	iRecNum++;
-	if (iRecNum == N_BANDWIDTH_SLOTS)
-		iRecNum = 0;
-
-	return true;
-}
-
-int BandwidthRecord::onlineSeconds() const {
-	return static_cast<int>(qtFirst.elapsed() / 1000000LL);
-}
-
-int BandwidthRecord::idleSeconds() const {
-	return static_cast<int>(a_qtWhen[(iRecNum + N_BANDWIDTH_SLOTS - 1) % N_BANDWIDTH_SLOTS].elapsed() / 1000000LL);
-}
-
-int BandwidthRecord::bandwidth() const {
-	int sincelast = static_cast<int>(a_qtWhen[(iRecNum + N_BANDWIDTH_SLOTS - 1) % N_BANDWIDTH_SLOTS].elapsed() / 10000LL);
-	int todo = N_BANDWIDTH_SLOTS - sincelast;
-	if (todo < 0)
-		return 0;
-
-	int sum = 0;
-	for (int i=1;i<=todo;++i)
-		sum += a_iBW[(iRecNum+N_BANDWIDTH_SLOTS - i) % N_BANDWIDTH_SLOTS];
-	return (sum*100)/N_BANDWIDTH_SLOTS;
-}
-
-ExecEvent::ExecEvent(boost::function<void ()> f) : QEvent(static_cast<QEvent::Type>(EXEC_QEVENT)) {
-	func = f;
-}
-
-void ExecEvent::execute() {
-	func();
-}
-
 void Server::customEvent(QEvent *evt) {
 	if (evt->type() == EXEC_QEVENT)
 		static_cast<ExecEvent *>(evt)->execute();
@@ -1213,15 +1189,16 @@ void Server::sendProtoMessage(ServerUser *u, const ::google::protobuf::Message &
 	u->sendMessage(msg, msgType, cache);
 }
 
-void Server::sendProtoAll(const ::google::protobuf::Message &msg, unsigned int msgType) {
-	sendProtoExcept(NULL, msg, msgType);
+void Server::sendProtoAll(const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int minversion) {
+	sendProtoExcept(NULL, msg, msgType, minversion);
 }
 
-void Server::sendProtoExcept(ServerUser *u, const ::google::protobuf::Message &msg, unsigned int msgType) {
+void Server::sendProtoExcept(ServerUser *u, const ::google::protobuf::Message &msg, unsigned int msgType, unsigned int minversion) {
 	QByteArray cache;
 	foreach(ServerUser *usr, qhUsers)
 		if ((usr != u) && (usr->sState == ServerUser::Authenticated))
-			usr->sendMessage(msg, msgType, cache);
+			if ((minversion == 0) || (usr->uiVersion >= minversion) || ((minversion & 0x80000000) && (usr->uiVersion < (~minversion))))
+				usr->sendMessage(msg, msgType, cache);
 }
 
 void Server::removeChannel(int id) {
@@ -1246,12 +1223,16 @@ void Server::removeChannel(Channel *chan, Channel *dest) {
 	foreach(p, chan->qlUsers) {
 		chan->removeUser(p);
 
+		Channel *target = dest;
+		while (target->cParent && ! hasPermission(static_cast<ServerUser *>(p), target, ChanACL::Enter))
+			target = target->cParent;
+
 		MumbleProto::UserState mpus;
 		mpus.set_session(p->uiSession);
-		mpus.set_channel_id(dest->iId);
+		mpus.set_channel_id(target->iId);
+		userEnterChannel(p, target, mpus);
 		sendAll(mpus);
-
-		userEnterChannel(p, dest);
+		emit userStateChanged(p);
 	}
 
 	MumbleProto::ChannelRemove mpcr;
@@ -1311,7 +1292,7 @@ bool Server::unregisterUser(int id) {
 	return true;
 }
 
-void Server::userEnterChannel(User *p, Channel *c, bool quiet, bool ignoretemp) {
+void Server::userEnterChannel(User *p, Channel *c, MumbleProto::UserState &mpus, bool quiet) {
 	if (quiet && (p->cChannel == c))
 		return;
 
@@ -1327,26 +1308,18 @@ void Server::userEnterChannel(User *p, Channel *c, bool quiet, bool ignoretemp)
 	if (quiet)
 		return;
 
-
 	setLastChannel(p);
 
 	bool mayspeak = hasPermission(static_cast<ServerUser *>(p), c, ChanACL::Speak);
 	bool sup = p->bSuppress;
 
-	if (! p->bMute) {
-		if (mayspeak == sup) {
-			// Ok, he can speak and was suppressed, or vice versa
-			p->bSuppress = ! mayspeak;
-
-			MumbleProto::UserState mpus;
-			mpus.set_session(p->uiSession);
-			mpus.set_suppress(p->bSuppress);
-			sendAll(mpus);
-		}
+	if (mayspeak == sup) {
+		// Ok, he can speak and was suppressed, or vice versa
+		p->bSuppress = ! mayspeak;
+		mpus.set_suppress(p->bSuppress);
 	}
-	emit userStateChanged(p);
 
-	if (old && old->bTemporary && old->qlUsers.isEmpty() && ! ignoretemp) {
+	if (old && old->bTemporary && old->qlUsers.isEmpty()) {
 		QCoreApplication::instance()->postEvent(this, new ExecEvent(boost::bind(&Server::removeChannel, this, old->iId)));
 	}
 
@@ -1488,6 +1461,7 @@ void Server::recheckCodecVersions() {
 	QMap<int, unsigned int> qmCodecUsercount;
 	QMap<int, unsigned int>::const_iterator i;
 	int users = 0;
+
 	// Count how many users use which codec
 	foreach(ServerUser *u, qhUsers) {
 		if (u->qlCodecs.isEmpty())
@@ -1540,3 +1514,98 @@ void Server::recheckCodecVersions() {
 
 	log(QString::fromLatin1("CELT codec switch %1 %2 (prefer %3)").arg(iCodecAlpha,0,16).arg(iCodecBeta,0,16).arg(bPreferAlpha ? iCodecAlpha : iCodecBeta,0,16));
 }
+
+void Server::hashAssign(QString &dest, QByteArray &hash, const QString &src) {
+	dest = src;
+	if (src.length() >= 128)
+		hash = sha1(src);
+	else
+		hash = QByteArray();
+}
+
+void Server::hashAssign(QByteArray &dest, QByteArray &hash, const QByteArray &src) {
+	dest = src;
+	if (src.length() >= 128)
+		hash = sha1(src);
+	else
+		hash = QByteArray();
+}
+
+bool Server::isTextAllowed(QString &text, bool &changed) {
+	changed = false;
+
+	if (! bAllowHTML) {
+		if (! text.contains(QLatin1Char('<'))) {
+			text = text.simplified();
+		} else {
+			QXmlStreamReader qxsr(QString::fromLatin1("<document>%1</document>").arg(text));
+			QString qs;
+			while (! qxsr.atEnd()) {
+				switch (qxsr.readNext()) {
+					case QXmlStreamReader::Invalid:
+						return false;
+					case QXmlStreamReader::Characters:
+						qs += qxsr.text();
+						break;
+					case QXmlStreamReader::EndElement:
+						if ((qxsr.name() == QLatin1String("br")) || (qxsr.name() == QLatin1String("p")))
+							qs += "\n";
+						break;
+					default:
+						break;
+				}
+			}
+			text = qs.simplified();
+		}
+		changed = true;
+		return ((iMaxTextMessageLength == 0) || (text.length() <= iMaxTextMessageLength));
+	} else {
+		int length = text.length();
+
+		// No limits
+		if ((iMaxTextMessageLength == 0) && (iMaxImageMessageLength == 0))
+			return true;
+
+		// Over Image limit? (If so, always fail)
+		if ((iMaxImageMessageLength != 0) && (length > iMaxImageMessageLength))
+			return false;
+
+		// Under textlength?
+		if ((iMaxTextMessageLength == 0) || (length <= iMaxTextMessageLength))
+			return true;
+
+		// Over textlength, under imagelength. If no XML, this is a fail.
+		if (! text.contains(QLatin1Char('<')))
+			return false;
+
+		QString qsOut;
+		QXmlStreamReader qxsr(QString::fromLatin1("<document>%1</document>").arg(text));
+		QXmlStreamWriter qxsw(&qsOut);
+		while (! qxsr.atEnd()) {
+			switch (qxsr.readNext()) {
+				case QXmlStreamReader::Invalid:
+					return false;
+				case QXmlStreamReader::StartElement: {
+						if (qxsr.name() == QLatin1String("img")) {
+							QXmlStreamAttributes attr = qxsr.attributes();
+
+							qxsw.writeStartElement(qxsr.namespaceUri().toString(), qxsr.name().toString());
+							foreach(const QXmlStreamAttribute &a, qxsr.attributes())
+								if (a.name() != QLatin1String("src"))
+									qxsw.writeAttribute(a);
+						} else {
+							qxsw.writeCurrentToken(qxsr);
+						}
+					}
+					break;
+				default:
+					qxsw.writeCurrentToken(qxsr);
+					break;
+			}
+		}
+
+		length = qsOut.length();
+
+		return (length <= iMaxTextMessageLength);
+	}
+}
