@@ -34,6 +34,7 @@
 #include "Group.h"
 #include "ACL.h"
 #include "Server.h"
+#include "ServerUser.h"
 #include "Meta.h"
 #include "Connection.h"
 #include "DBus.h"
@@ -290,7 +291,7 @@ ServerDB::ServerDB() {
 			SQLDO("ALTER TABLE `%1channels` ADD CONSTRAINT `%1channels_parent_del` FOREIGN KEY (`server_id`, `parent_id`) REFERENCES `%1channels`(`server_id`,`channel_id`) ON DELETE CASCADE");
 			SQLDO("ALTER TABLE `%1channels` ADD CONSTRAINT `%1channels_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1channel_info` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT)");
+			SQLDO("CREATE TABLE `%1channel_info` (`server_id` INTEGER NOT NULL, `channel_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
 			SQLDO("CREATE UNIQUE INDEX `%1channel_info_id` ON `%1channel_info`(`server_id`, `channel_id`, `key`)");
 			SQLDO("ALTER TABLE `%1channel_info` ADD CONSTRAINT `%1channel_info_del_channel` FOREIGN KEY (`server_id`, `channel_id`) REFERENCES `%1channels`(`server_id`,`channel_id`) ON DELETE CASCADE");
 
@@ -300,7 +301,7 @@ ServerDB::ServerDB() {
 			SQLDO("CREATE UNIQUE INDEX `%1users_id` ON `%1users` (`server_id`, `user_id`)");
 			SQLDO("ALTER TABLE `%1users` ADD CONSTRAINT `%1users_server_del` FOREIGN KEY (`server_id`) REFERENCES `%1servers`(`server_id`) ON DELETE CASCADE");
 
-			SQLDO("CREATE TABLE `%1user_info` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT)");
+			SQLDO("CREATE TABLE `%1user_info` (`server_id` INTEGER NOT NULL, `user_id` INTEGER NOT NULL, `key` INTEGER, `value` LONGTEXT) Type=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci");
 			SQLDO("CREATE UNIQUE INDEX `%1user_info_id` ON `%1user_info`(`server_id`, `user_id`, `key`)");
 			SQLDO("ALTER TABLE `%1user_info` ADD CONSTRAINT `%1user_info_del_user` FOREIGN KEY (`server_id`, `user_id`) REFERENCES `%1users`(`server_id`,`user_id`) ON DELETE CASCADE");
 
@@ -332,7 +333,8 @@ ServerDB::ServerDB() {
 		} else {
 			qWarning("Importing old data...");
 
-			SQLDO("SET FOREIGN_KEY_CHECKS = 0;");
+			if (Meta::mp.qsDBDriver != "QSQLITE")
+				SQLDO("SET FOREIGN_KEY_CHECKS = 0;");
 			SQLDO("INSERT INTO `%1servers` (`server_id`) SELECT `server_id` FROM `%1servers_old`");
 			SQLDO("INSERT INTO `%1slog` (`server_id`, `msg`, `msgtime`) SELECT `server_id`, `msg`, `msgtime` FROM `%1slog_old`");
 			SQLDO("INSERT INTO `%1config` (`server_id`, `key`, `value`) SELECT `server_id`, `keystring`, `value` FROM `%1config_old`");
@@ -342,7 +344,8 @@ ServerDB::ServerDB() {
 			SQLDO("INSERT INTO `%1group_members` (`group_id`, `server_id`, `user_id`, `addit`) SELECT `group_id`, `server_id`, `player_id`, `addit` FROM `%1group_members_old`");
 			SQLDO("INSERT INTO `%1acl` (`server_id`, `channel_id`, `priority`, `user_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv`) SELECT `server_id`, `channel_id`, `priority`, `player_id`, `group_name`, `apply_here`, `apply_sub`, `grantpriv`, `revokepriv` FROM `%1acl_old`");
 			SQLDO("INSERT INTO `%1channel_links` (`server_id`, `channel_id`, `link_id`) SELECT `server_id`, `channel_id`, `link_id` FROM `%1channel_links_old`");
-			SQLDO("SET FOREIGN_KEY_CHECKS = 1;");
+			if (Meta::mp.qsDBDriver != "QSQLITE")
+				SQLDO("SET FOREIGN_KEY_CHECKS = 1;");
 
 			QList<QList<QVariant> > ql;
 			SQLPREP("SELECT `server_id`, `base`, `mask` FROM `%1bans_old`");
@@ -390,6 +393,7 @@ ServerDB::ServerDB() {
 			SQLDO("DROP TABLE IF EXISTS `%1acl_old`");
 			SQLDO("DROP TABLE IF EXISTS `%1channel_links_old`");
 			SQLDO("DROP TABLE IF EXISTS `%1bans_old`");
+			SQLDO("DROP TABLE IF EXISTS `%1servers_old`");
 
 			SQLDO("UPDATE `%1meta` SET `value` = '4' WHERE `keystring` = 'version'");
 		}
@@ -734,6 +738,8 @@ int Server::authenticate(QString &name, const QString &pw, const QStringList &em
 			QSqlQuery &query = *th.qsqQuery;
 
 			int lchan=readLastChannel(res);
+			if (lchan < 0)
+				lchan = 0;
 
 			SQLPREP("REPLACE INTO `%1users` (`server_id`, `user_id`, `name`, `lastchannel`) VALUES (?,?,?,?)");
 			query.addBindValue(iServerNum);
@@ -759,7 +765,7 @@ int Server::authenticate(QString &name, const QString &pw, const QStringList &em
 	if (query.next()) {
 		res = -1;
 		QString storedpw = query.value(2).toString();
-		QString hashedpw = QString::fromLatin1(QCryptographicHash::hash(pw.toUtf8(), QCryptographicHash::Sha1).toHex());
+		QString hashedpw = QString::fromLatin1(sha1(pw).toHex());
 
 		if (! storedpw.isEmpty() && (storedpw == hashedpw)) {
 			name = query.value(1).toString();
@@ -898,40 +904,16 @@ bool Server::setTexture(int id, const QByteArray &texture) {
 		return false;
 
 	QByteArray tex;
-	if (! texture.isEmpty()) {
-		qint32 l = 600*60*4;
-		if (texture.size() < 4)
-			return false;
+	if (texture.size() == 600 * 60 * 4)
+		tex = qCompress(texture);
+	else
+		tex = texture;
 
-		if (texture.size() != l) {
-			const unsigned char *data = reinterpret_cast<const unsigned char *>(texture.constData());
-
-			if (((data[0] << 24) | (data[1] << 16) | (data[2] << 8) | data[3]) == l) {
-				tex = qUncompress(texture);
-			} else {
-				QByteArray qba(4,0);
-				qba[0] = static_cast<char>((l >> 24) & 0xFF);
-				qba[1] = static_cast<char>((l >> 16) & 0xFF);
-				qba[2] = static_cast<char>((l >> 8) & 0xFF);
-				qba[3] = static_cast<char>((l >> 0) & 0xFF);
-				qba.append(texture);
-				tex = qUncompress(qba);
-			}
-		} else {
-			tex = texture;
-		}
-
-		if (tex.size() != 600 * 60 * 4)
-			return false;
+	foreach(ServerUser *u, qhUsers) {
+		if (u->iId == id)
+			hashAssign(u->qbaTexture, u->qbaTextureHash, tex);
 	}
 
-	if (! tex.isEmpty())
-		tex = qCompress(tex);
-
-	ServerUser *u = qhUsers.value(id);
-	if (u)
-		u->qbaTexture = tex;
-
 	int res = -2;
 	emit setTextureSig(res, id, tex);
 	if (res >= 0)
@@ -1245,7 +1227,7 @@ void Server::readChannelPrivs(Channel *c) {
 		int key = query.value(0).toInt();
 		const QString &value = query.value(1).toString();
 		if (key == ServerDB::Channel_Description) {
-			c->qsDesc = value;
+			hashAssign(c->qsDesc, c->qbaDescHash, value);
 		} else if (key == ServerDB::Channel_Position) {
 			c->iPosition = QVariant(value).toInt(); // If the conversion fails it'll return the default value 0
 		}
@@ -1367,7 +1349,7 @@ void Server::setLastChannel(const User *p) {
 
 int Server::readLastChannel(int id) {
 	if (id < 0)
-		return 0;
+		return -1;
 
 	TransactionHolder th;
 	QSqlQuery &query = *th.qsqQuery;
@@ -1382,7 +1364,7 @@ int Server::readLastChannel(int id) {
 		if (qhChannels.contains(cid))
 			return cid;
 	}
-	return 0;
+	return -1;
 }
 
 void Server::dumpChannel(const Channel *c) {
