@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -35,11 +35,13 @@
 #include "Timer.h"
 
 class Channel;
-class Player;
+class User;
 class Connection;
 
 class ServerDB {
 	public:
+		enum ChannelInfo { Channel_Description, Channel_Position };
+		enum UserInfo { User_Name, User_Email, User_Comment, User_Hash, User_Password };
 		ServerDB();
 		~ServerDB();
 		typedef QPair<unsigned int, QString> LogRecord;
@@ -55,8 +57,9 @@ class ServerDB {
 		static QVariant getConf(int server_id, const QString &key, QVariant def = QVariant());
 		static void setConf(int server_id, const QString &key, const QVariant &value = QVariant());
 		static QList<LogRecord> getLog(int server_id, unsigned int offs_min, unsigned int offs_max);
-		static bool prepare(QSqlQuery &, const QString &, bool fatal = true);
-		static bool exec(QSqlQuery &, const QString &str = QString(), bool fatal= true);
+		static bool prepare(QSqlQuery &, const QString &, bool fatal = true, bool warn = true);
+		static bool exec(QSqlQuery &, const QString &str = QString(), bool fatal= true, bool warn = true);
+		static bool execBatch(QSqlQuery &, const QString &str = QString(), bool fatal= true);
 };
 
 #endif
