@@ -1,3 +1,4 @@
+#ifndef Q_MOC_RUN
 #ifndef _MURMUR_PCH_H
 #define _MURMUR_PCH_H
 
@@ -9,6 +10,7 @@
 #endif
 
 #ifdef __APPLE__
+#include <CoreFoundation/CoreFoundation.h>
 #include <CoreServices/CoreServices.h>
 #undef check
 #undef TYPE_BOOL
@@ -51,11 +53,11 @@ extern "C" {
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/time.h>
+#include <sys/resource.h>
 #ifdef Q_OS_LINUX
 #include <sys/capability.h>
 #include <sys/prctl.h>
-#include <sys/time.h>
-#include <sys/resource.h>
 #endif
 #include <pwd.h>
 #ifdef __FreeBSD__
@@ -78,16 +80,10 @@ extern "C" {
 #endif
 #if defined (Q_OS_WIN) && (defined (Q_CC_INTEL) || defined (Q_CC_MSVC))
 #define lroundf(x) ( static_cast<int>( (x) + ((x) >= 0 ? 0.5 : -0.5) ) )
-#define ALLOCA(x) _alloca(x)
 #define snprintf ::_snprintf
+#define STACKVAR(type, varname, count) type *varname=reinterpret_cast<type *>(_alloca(sizeof(type) * (count)))
 #else
-#include <alloca.h>
-#define ALLOCA(x) alloca(x)
-#endif
-#if defined (Q_CC_GNU) || (defined (Q_CC_INTEL) && !defined (Q_OS_WIN))
 #define STACKVAR(type, varname, count) type varname[count]
-#else
-#define STACKVAR(type, varname, count) type *varname=reinterpret_cast<type *>(ALLOCA(sizeof(type) * (count)))
 #endif
 
 #include <boost/function.hpp>
@@ -100,3 +96,4 @@ extern "C" {
 #endif
 
 #endif
+#endif
